###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	matplotlib
import matplotlib.pyplot as plt

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Globalstar File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set flag
savefile_flag = True
savefile_flag = False

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

###################################################################################################
#	inputs
###################################################################################################

#	define read path
read_path = os.path.join(path_data, 'input_dir/mmt_files/globalstar/lightcurves')

specific_object = 32263

specific_object_flag = False
specific_object_flag = True

###################################################################################################
#	plot light curve properties
###################################################################################################

print('\n#####################################################################################')
print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))

###################################################################################################
#	load mmt_properties file

#	define output path
mmt_output_path = 'input_dir/mmt_files/globalstar'

#	define read file
mmt_output_file = 'globalstar_mmt_data_table.txt'

with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'r') as read_file:

	#	print data to output file
	globalstar_data = read_file.readlines()

###################################################################################################
#	figure parameters

fig_avg, ax_avg = plt.subplots()
fig_std, ax_std = plt.subplots()

fig_std_full, ax_std_full = plt.subplots()

###################################################################################################
#	load MMT file

globalstar_dir = os.listdir(read_path)

globalstar_file_list = []
for mmt_file in globalstar_dir:
	globalstar_file_list.append(mmt_file)

for file_itt, mmt_file in enumerate(globalstar_file_list):

	print('\n#############################################################')
	print('Working on: ', mmt_file)
	print('File {} of {}...'.format(int(file_itt), int(len(globalstar_file_list))))

	output_path = os.path.join(path_data, mmt_output_path, 'parsed_lightcurves')
	output_file = mmt_file[:-4] + '_parsed.txt'

	#	parse filename data
	name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
	
	name_str = ''.join(name_str.split('-'))
	norad_id = int(norad_id)

	if specific_object_flag is True:
		if specific_object != norad_id:
			continue

	launches = []
	status_strings = []

	#	get MMT data
	for line in globalstar_data:
		temp_mmt_id = int(line.split(', ')[0])
		temp_name_string = line.split(', ')[1].strip()
		temp_norad_id = int(line.split(', ')[2])
	
		intl_desig = line.split(', ')[3].strip()
	
		status_string = line.split(', ')[4].strip()
		period_string = line.split(', ')[5].strip()

#		print('Status = ', status_string)

		launches.append([temp_norad_id, int(intl_desig.split('-')[0]), int(intl_desig.split('-')[1][0:3])])
		status_strings.append(status_string)
	
		if 'Period:' in period_string:
			period = float(period_string.split(': ')[1].split(' s')[0])
			period_flag = True
	
#		if temp_norad_id == norad_id:
#			break

	#	sort by period
	launches = np.array(launches)
	sort_ids = launches[:, 1].argsort()
	launches = launches[sort_ids]
	status_strings2 = []
	for id in sort_ids:
		status_strings2.append(status_strings[id])
	for (stat_string, launch_line) in zip(status_strings2, launches):
		print(launch_line, stat_string)

	sys.exit()


	if os.path.exists(os.path.join(output_path, output_file)):
		parsed_data = np.loadtxt(os.path.join(output_path, output_file))

	else:
	
		###############################################################################################
		#	object printout
		print('\n#############################################################')
		print('Working on: ', mmt_file)
		
		print('\n	Name string:	', temp_name_string)
		print('	Norad ID:	', temp_norad_id)
		print('	Status:		', status_string)
		print('	Periodicity:	', period_string)
		print('	Intl. Desig.:	', intl_desig)
	
		###############################################################################################
		#	load mmt file
		print('\n	Loading MMT file...')
	
		#	open and read file
		with open(os.path.join(read_path, mmt_file)) as read_file:
			mmt_data = read_file.readlines()
	
		#	set object ID to norad_id
		object_id = temp_norad_id
		
		###########################################################################################
		#	parse MMT file

		#	find start of data
		for line_id, line in enumerate(mmt_data):
			if line[0] != '#':
				break

		#	strip header
		header_text = mmt_data[7]
		raw_data = mmt_data[8:]
		
		print('\n	Parsing MMT data...')
		
		#	initialise output
		n_rows = 14
		parsed_data = np.zeros([len(raw_data), n_rows])
		
		for i, line in enumerate(raw_data):
		
			#	split line
			date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
				distance, phase, channel, track_id = line.split(' ')
		
			#	parse data and set data types
			mag_std = float(mag_std)
			mag_app = float(mag_app)
			distance = float(distance)
			phase = float(phase)
			track_id = int(track_id)
		
			#	parse date
			year, month, day = date_str.split('-')
			year = int(year)
			month = int(month)
			day = int(day)
		
			#	parse time
			hour, minute, second = time_str.split(':')
			hour = int(hour)
			minute = int(minute)
			second = np.round(float(second), 1)
		
			#	calculate julian date
			jday, jfrac = date2jd(year, month, day, hour, minute, second)
		
			#	generate output line
			output_line = [
				track_id, object_id,
				jday, jfrac,
				year, month, day,
				hour, minute, second,
				mag_std, mag_app,
				distance, phase
			]
		
			#	add to output array
			parsed_data[i, :] = output_line
		
		print('		Total data length = ', len(parsed_data))
		print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
		print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))
	
		#	save file
		np.savetxt(os.path.join(output_path, output_file), parsed_data)

	###################################################################################################
	#	extract required data
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])
	
	print('\n	Looping through {} Track IDs...'.format(len(track_id_list)))

	lc_avg = []
	lc_std = []
	
	lc_avg_avg = []
	lc_std_avg = []
	
	full_datevec_array = []
	full_datevec_array_avg = []
	
	#	loop through track IDs
	for track_itt, track_id in enumerate(track_id_list):
	
		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		###########################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]
		
		#	parse line elements
		track_id, norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		track_id, norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
	
		###############################################################################################
		#	extract mmt brightness data
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]
				
		###############################################################################################
		#	generate date vectors
		
		#	convert jd to date
		year, mon, day, hour, minute, sec = [*jd2date(*start_jd)]
		
		#	turn into string
		datestr = str('-'.join(
			['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
		))
		
		timestr = ':'.join(
			['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
		)
		
		datevec  = ' '.join([datestr, timestr])
	
		#	convert to datetime object
		date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
	
		#	append  to output array
		full_datevec_array.append(date_time_obj)
	
		###############################################################################################
		#	calculate moving averages
	
		lc_avg.append(np.average(mmt_signal_app))
		lc_std.append(np.std(mmt_signal_app))


		n_avg_points = 10
	
		if track_itt > (n_avg_points - 2):

			avg_avg = 0
			std_avg = 0

			for avg_id in np.arange(n_avg_points):
				avg_avg += lc_avg[track_itt - avg_id]
				std_avg += lc_std[track_itt - avg_id]

			avg_avg = (1 / n_avg_points) * avg_avg
			lc_avg_avg.append(avg_avg)

			std_avg = (1 / n_avg_points) * std_avg
			lc_std_avg.append(std_avg)
	
			full_datevec_array_avg.append(date_time_obj)

	lc_avg = np.array(lc_avg)
	lc_std = np.array(lc_std)
	
	full_lc_avg_avg = np.average(lc_avg)
	full_lc_std_avg = np.average(lc_std)

	###################################################################################################
	#	final figure output

	print('\n	Plotting data...')

	plot_data = np.full([len(full_datevec_array_avg), 1], full_lc_std_avg)

	if status_string == 'Inactive Satellite':
		ax_avg.scatter(full_datevec_array, lc_avg, marker = 'o', color='r', s=2)
		ax_avg.plot(full_datevec_array_avg, lc_avg_avg, color='r', linewidth=1)
	elif status_string == 'Active Satellite':
		ax_avg.scatter(full_datevec_array, lc_avg, marker = 'o', color='g', s=2)
		ax_avg.plot(full_datevec_array_avg, lc_avg_avg, color='r', linewidth=1)
	else:
		print('FAILED MATCH')
		pass


	if status_string == 'Inactive Satellite':
		ax_std.scatter(full_datevec_array, lc_std, marker = 'o', color='r', s=2)
		ax_std.plot(full_datevec_array_avg, lc_std_avg, color='r', linewidth=1)
	elif status_string == 'Active Satellite':
		ax_std.scatter(full_datevec_array, lc_std, marker = 'o', color='g', s=2)
		ax_std.plot(full_datevec_array_avg, lc_std_avg, color='r', linewidth=1)
	else:
		print('FAILED MATCH')
		pass



	if status_string == 'Inactive Satellite':
#		ax_std_full.scatter(full_datevec_array, lc_std, marker = 'o', color='r', s=2)
		ax_std_full.plot(full_datevec_array_avg, lc_std_avg, color='r', linewidth=1)
#		ax_std_full.plot(full_datevec_array_avg, plot_data, color='r', linewidth=1)

	elif status_string == 'Active Satellite':
#		ax_std_full.scatter(full_datevec_array, lc_std, marker = 'o', color='g', s=2)
		ax_std_full.plot(full_datevec_array_avg, lc_std_avg, color='g', linewidth=1)
#		ax_std_full.plot(full_datevec_array_avg, plot_data, color='g', linewidth=1)
	else:
		print('FAILED MATCH')
		pass


	ax_avg.set_ylim([12.0, 0.0])
	ax_avg.set_title('Average Magnitude')

	ax_std.set_ylim([0.0, 2.0])
	ax_std.set_title('Standard Deviation')

	ax_std_full.set_ylim([0.0, 2.0])
	ax_std_full.set_title('Standard Deviation')

	plt.show()
	sys.exit()

#	plt.pause(0.1)
#	input('\n	Press Enter to continue to next object...')
	

####################################################
#	save figure
fig_name = 'average_magnitudes' + '.eps'
out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	ax_avg.figure.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

####################################################
#	save figure
fig_name = 'standard_deviations' + '.eps'
out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	ax_std.figure.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

####################################################
#	save figure
fig_name = 'standard_deviations_averaged' + '.eps'
out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	ax_std_full.figure.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)




plt.show()
sys.exit()
