###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	matplotlib
import matplotlib.pyplot as plt

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Globalstar File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set flag
savefile_flag = True
savefile_flag = False

#	output file name
fig_name = 'group_all_lightcurve_data' + '.pdf'

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

###################################################################################################
#	inputs
###################################################################################################

#	define read path
read_path = os.path.join(path_data, 'input_dir/mmt_files/globalstar/lightcurves')

specific_objects = [
	26081, 26082, 26083, 26084,	#	inactive batch
#	31571, 31573, 31574, 31576,	#	active batch
#	32263, 32264, 32265, 32266,	#	object of interest batch
]

specific_object_flag = False
specific_object_flag = True

#	set time windows
window_start_date = [2010, 8, 1, 0, 0, 0]
window_stop_date = [2020, 5, 1, 0, 0, 0]

#	simulation time step
time_step = 0.1

#	number of moving average points
n_avg_points = 40

###################################################################################################
#	plot light curve properties
###################################################################################################

print('\n#####################################################################################')
print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))

###################################################################################################
#	load mmt_properties file

#	define output path
mmt_output_path = 'input_dir/mmt_files/globalstar'

#	define read file
mmt_output_file = 'globalstar_mmt_data_table.txt'

with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'r') as read_file:

	#	print data to output file
	globalstar_data = read_file.readlines()

###################################################################################################
#	figure parameters

#	set use of LaTeX font
plt.rc('text', usetex=True)

fig, ax = plt.subplots(2, 1, dpi=300, sharex=True)

# fig_avg, ax_avg = plt.subplots()
# fig_std, ax_std = plt.subplots()

# fig_std_full, ax_std_full = plt.subplots()

threshold_value = 0.27

###################################################################################################
#	load MMT file

globalstar_dir = os.listdir(read_path)

globalstar_file_list = []
for mmt_file in globalstar_dir:

	#	parse filename data
	name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
	name_str = ''.join(name_str.split('-'))
	norad_id = int(norad_id)

	#	skip if not requested object
	if specific_object_flag is True:
		if norad_id not in specific_objects:
			continue

	globalstar_file_list.append(mmt_file)

marker_range = [
	'p', '^', 's', 'h',
	'p', '^', 's', 'h',
	'p', '^', 's', 'h',
#	'p', '^', 's', 'h'
]

style_range = [
	'solid', 'solid', 'solid', 'solid',
	'dotted', 'dotted', 'dotted', 'dotted',
	'dashed', 'dashed', 'dashed', 'dashed',
#	'dashdot', 'dashdot', 'dashdot', 'dashdot'
]


legend_data1 = []
legend_data2 = []

for file_itt, mmt_file in enumerate(globalstar_file_list):

	used_marker = marker_range[file_itt]
	style_used = style_range[file_itt]
	style_used = 'solid'

	#	parse filename data
	name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
	name_str = ''.join(name_str.split('-'))
	norad_id = int(norad_id)

	if norad_id == 26084:
		style_used = 'dotted'

	print('\n#############################################################')
	print('Working on: ', mmt_file)
	print('File {} of {}...'.format(int(file_itt) + 1, int(len(globalstar_file_list))))

	#	get MMT data
	for line in globalstar_data:
		temp_mmt_id = int(line.split(', ')[0])
		temp_name_string = line.split(', ')[1].strip()
		temp_norad_id = int(line.split(', ')[2])
	
		intl_desig = line.split(', ')[3].strip()
	
		status_string = line.split(', ')[4].strip()
		period_string = line.split(', ')[5].strip()
	
		if 'Period:' in period_string:
			period = float(period_string.split(': ')[1].split(' s')[0])
			period_flag = True
	
		if temp_norad_id == norad_id:
			break
	
	#	generate a label string
	label_str = '{} ({})'.format(temp_name_string.split(' ')[1], norad_id)
	print('\n	Label String: ', label_str)

	#	read file properties
	output_path = os.path.join(path_data, mmt_output_path, 'parsed_lightcurves')
	output_file = mmt_file[:-4] + '_parsed.txt'

	if os.path.exists(os.path.join(output_path, output_file)):
		parsed_data = np.loadtxt(os.path.join(output_path, output_file))

	else:
	
		###########################################################################################
		#	object printout
		
		print('\n	Name string:	', temp_name_string)
		print('	Norad ID:	', temp_norad_id)
		print('	Status:		', status_string)
		print('	Periodicity:	', period_string)
		print('	Intl. Desig.:	', intl_desig)
	
		###########################################################################################
		#	load mmt file
		print('\n	Loading MMT file...')
	
		#	open and read file
		with open(os.path.join(read_path, mmt_file)) as read_file:
			mmt_data = read_file.readlines()
	
		#	set object ID to norad_id
		object_id = temp_norad_id
		
		###########################################################################################
		#	parse MMT file

		#	find start of data
		for line_id, line in enumerate(mmt_data):
			if line[0] != '#':
				break

		#	strip header
		header_text = mmt_data[7]
		raw_data = mmt_data[8:]
		
		print('\n	Parsing MMT data...')
		
		#	initialise output
		n_rows = 14
		parsed_data = np.zeros([len(raw_data), n_rows])
		
		for i, line in enumerate(raw_data):
		
			#	split line
			date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
				distance, phase, channel, track_id = line.split(' ')
		
			#	parse data and set data types
			mag_std = float(mag_std)
			mag_app = float(mag_app)
			distance = float(distance)
			phase = float(phase)
			track_id = int(track_id)
		
			#	parse date
			year, month, day = date_str.split('-')
			year = int(year)
			month = int(month)
			day = int(day)
		
			#	parse time
			hour, minute, second = time_str.split(':')
			hour = int(hour)
			minute = int(minute)
			second = np.round(float(second), 1)
		
			#	calculate julian date
			jday, jfrac = date2jd(year, month, day, hour, minute, second)
		
			#	generate output line
			output_line = [
				track_id, object_id,
				jday, jfrac,
				year, month, day,
				hour, minute, second,
				mag_std, mag_app,
				distance, phase
			]
		
			#	add to output array
			parsed_data[i, :] = output_line
		
		print('		Total data length = ', len(parsed_data))
		print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
		print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))
	
		#	save file
		np.savetxt(os.path.join(output_path, output_file), parsed_data)

	###############################################################################################
	#	extracting data between specified windows
	
	print('\n	Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n		Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('		Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])
	extracted_track_id_list = []
	
	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:
	
		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
			#	add track id to output
			extracted_track_id_list.append(track_id)
	
	print('\n		{} track(s) extracted.'.format(ntracks))
	extracted_track_id_list = np.array(extracted_track_id_list)

	###############################################################################################
	#	extract required data
	
	print('\n	Looping through {} Track IDs...'.format(len(track_id_list)))

	lc_avg = []
	lc_std = []
	
	lc_avg_avg = []
	lc_std_avg = []
	
	full_datevec_array = []
	full_datevec_array_avg = []
	
	#	loop through track IDs
	for track_itt, track_id in enumerate(extracted_track_id_list):
	
		print('\n	#############################################################################')
		print('	Working on TrackID {}; {} of {}...'.format(
			int(track_id), int(track_itt) + 1, int(len(extracted_track_id_list))
		))
	
		#	find track ids
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = windowed_data[track_indices, :]
	
		###########################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]
		
		#	parse line elements
		track_id, temp_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		track_id, temp_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
	
		###########################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
		sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

		###########################################################################################
		#	extract mmt brightness data
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]
				
		print('\n		#####################################################################')
		print('		Extracted MMT track parameters: ')
		print(
			'			Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_start_date)
		)
		print(
			'			Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_stop_date)
		)
		print('			Simulation time step:		{}'.format(time_step))
		print('			Duration:			{}		[steps]'.format(lc_len_record))
		print('							{}		[s]'.format(np.round(lc_duration_sec, 1)))

		###########################################################################################
		#	generate date vectors
		
		#	convert jd to date
		year, mon, day, hour, minute, sec = [*jd2date(*start_jd)]
		
		#	turn into string
		datestr = str('-'.join(
			['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
		))
		
		timestr = ':'.join(
			['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
		)
		
		datevec  = ' '.join([datestr, timestr])
	
		#	convert to datetime object
		date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
	
		#	append  to output array
		full_datevec_array.append(date_time_obj)
	
		###########################################################################################
		#	calculate moving averages
	
		lc_avg.append(np.average(mmt_signal_app))
		lc_std.append(np.std(mmt_signal_app))

		if track_itt > (n_avg_points - 2):

			avg_avg_real = 0
			std_avg_real = 0

			for avg_id in np.arange(n_avg_points):
				avg_avg_real += lc_avg[track_itt - avg_id]
				std_avg_real += lc_std[track_itt - avg_id]

			avg_avg_real = (1 / n_avg_points) * avg_avg_real
			std_avg_real = (1 / n_avg_points) * std_avg_real

			#	append to outputs
			lc_avg_avg.append(avg_avg_real)
			lc_std_avg.append(std_avg_real)

			full_datevec_array_avg.append(date_time_obj)

	#	convert to numpy arrays
	lc_avg = np.array(lc_avg)
	lc_std = np.array(lc_std)
	
	full_lc_avg_avg = np.average(lc_avg)
	full_lc_std_avg = np.average(lc_std)

	###############################################################################################
	#	final figure output

	print('\n	Plotting data...')

	plot_data = np.full([len(full_datevec_array_avg), 1], full_lc_std_avg)

	print('\n	Status String: ', status_string)
	if status_string == 'Inactive Satellite':
		color_str = 'r'
	elif status_string == 'Active Satellite':
		color_str = 'g'
	elif status_string == 'Uncertain Satellite':
		color_str = 'b'
	else:
		print('FAILED MATCH')
		continue

	#	object of interest
	if norad_id == 32263:
		color_str = 'm'
	elif norad_id == 32266:
		color_str = 'c'
	elif norad_id == 26081:
		color_str = 'b'

	marker_size = 2

	markerline1, = ax[0].plot(
		full_datevec_array, lc_avg,
		linestyle='None', markeredgecolor='None',
		marker=used_marker, color=color_str, alpha=0.1, ms=3,
	)

	line1, = ax[0].plot(
		full_datevec_array_avg, lc_avg_avg,
		color=color_str,
		linestyle=style_used,
		linewidth=1.5
	)


	legend_data1.append((markerline1, line1))
	legend_data2.append(label_str)

	ax[1].plot(
		full_datevec_array, lc_std,
		linestyle='None', markeredgecolor='None',
		marker=used_marker, color=color_str, alpha=0.2, ms=3,
	)

	ax[1].plot(
		full_datevec_array_avg, lc_std_avg,
		color=color_str,
		linestyle=style_used,
		linewidth=1.5
	)

	lc_std_avg = np.array(lc_std_avg)
	print(lc_std_avg)

	total_len = len(lc_std_avg)
	greater_than = len(np.where(lc_std_avg > threshold_value)[0])
	less_than = len(np.where(lc_std_avg < threshold_value)[0])

	print('\n	Percentage above = ', 100 * (greater_than / total_len))
	print('	Percentage below = ', 100 * (less_than / total_len))


	break

#	plt.pause(0.1)
#	input('\n	Press Enter to continue to next object...')

###################################################################################################
#	finalise output figures

# ax_std[0].set_ylim([0.0, 2.0])
# ax_std[0].set_title('Standard Deviation')
# 
# ax_avg[1].set_ylim([12.0, 0.0])
# ax_avg[1].set_title('Average Magnitude')

#	start
year, mon, day, hour, minute, sec = [2014, 5, 1, 0, 0, 0]
#	turn into string
datestr = str('-'.join(
	['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
))
timestr = ':'.join(
	['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
)
datevec  = ' '.join([datestr, timestr])
#	convert to datetime object
date_time_obj1 = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')

#	end
year, mon, day, hour, minute, sec = [2020, 6, 1, 0, 0, 0]
#	turn into string
datestr = str('-'.join(
	['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
))
timestr = ':'.join(
	['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
)
datevec  = ' '.join([datestr, timestr])
#	convert to datetime object
date_time_obj2 = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')



ax[1].plot(
	[date_time_obj1, date_time_obj2], [threshold_value, threshold_value],
	color='k',
	linewidth=1.0
)

# ax[0].set_ylim([12.0, 0.0])
ax[0].set_ylabel(r'$Average\ Magnitude$')

ax[1].set_ylim([0.0, 1.0])
ax[1].set_ylabel(r'$Standard\ Deviation$')
ax[1].set_xlabel(r'$Time\ /\ Years$')

fig.legend(
	legend_data1, legend_data2,
	loc='center',
	frameon=False,
	ncol=4,
	bbox_to_anchor=(0.54, 0.53),
	markerscale=2.0
)

fig.tight_layout()
fig.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=None, hspace=0.5)


# ax_std_full.set_ylim([0.0, 2.0])
# ax_std_full.set_title('Standard Deviation')

# ####################################################
# #	save figure
# fig_name = 'average_magnitudes' + '.eps'
# out_str = os.path.join(fig_path, fig_name)
# 
# if savefile_flag is True:
# 	print('Saving: ', out_str)
# 	ax_avg.figure.savefig(out_str, bbox_inches="tight", dpi=300)
# 	print('\n#######################################')
# 	print('Figure saved: ', fig_name)
# 
# ####################################################
# #	save figure
# fig_name = 'standard_deviations' + '.eps'
# out_str = os.path.join(fig_path, fig_name)
# 
# if savefile_flag is True:
# 	print('Saving: ', out_str)
# 	ax_std.figure.savefig(out_str, bbox_inches="tight", dpi=300)
# 	print('\n#######################################')
# 	print('Figure saved: ', fig_name)
# 
# ####################################################
# #	save figure
# fig_name = 'standard_deviations_averaged' + '.eps'
# out_str = os.path.join(fig_path, fig_name)
# 
# if savefile_flag is True:
# 	print('Saving: ', out_str)
# 	ax_std_full.figure.savefig(out_str, bbox_inches="tight", dpi=300)
# 	print('\n#######################################')
# 	print('Figure saved: ', fig_name)
# 

####################################################
#	save figure
if savefile_flag is True:

	out_str = os.path.join(fig_path, fig_name)

	print('Saving: ', out_str)
	fig.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)



plt.show()
sys.exit()
