###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	matplotlib
import matplotlib.pyplot as plt

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Globalstar File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	inputs
###################################################################################################

#	define read path
read_path = os.path.join(path_data, 'input_dir/mmt_files/globalstar/lightcurves')

#	define a specific object
input_norad_id = 32263

specific_object_flag = False
specific_object_flag = True

#	define list of specific objects
specific_objects = [70]

###################################################################################################
#	plot light curve properties
###################################################################################################

print('\n#####################################################################################')
print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))

###################################################################################################
#	load mmt_properties file

#	define output path
mmt_output_path = 'input_dir/mmt_files/globalstar'

#	define read file
output_file = 'globalstar_mmt_data_table.txt'

with open(os.path.join(path_data, mmt_output_path, output_file), 'r') as read_file:

	#	print data to output file
	globalstar_data = read_file.readlines()

###################################################################################################
#	figure parameters

# fig_avg, ax_avg = plt.subplots()
fig_std, ax_std = plt.subplots()

fig_std_full, ax_std_full = plt.subplots()

###################################################################################################
#	load MMT file

globalstar_dir = os.listdir(read_path)

globalstar_file_list = []
for mmt_file in globalstar_dir:
	globalstar_file_list.append(mmt_file)


globalstar_file_list = globalstar_file_list[1:5]
period_plot_data = []

for mmt_file in globalstar_file_list:

	#	parse filename data
	name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
	
	name_str = ''.join(name_str.split('-'))
	norad_id = int(norad_id)

	#	get MMT data
	for line in globalstar_data:
		temp_mmt_id = int(line.split(', ')[0])
		temp_name_string = line.split(', ')[1].strip()
		temp_norad_id = int(line.split(', ')[2])

		intl_desig = line.split(', ')[3].strip()

		status_string = line.split(', ')[4].strip()
		period_string = line.split(', ')[5].strip()

		if 'Period:' in period_string:
			period = float(period_string.split(': ')[1].split(' s')[0])
			period_flag = True

		else:

			period_flag = False

		if temp_norad_id == norad_id:
			break

	if period_flag == True:

		###############################################################################################
		#	object printout
		print('\n#############################################################')
		print('Working on: ', mmt_file)
		
		print('\n	Name string:	', temp_name_string)
		print('	Norad ID:	', temp_norad_id)
		print('	Status:		', status_string)
		print('	Periodicity:	', period_string)
		print('	Intl. Desig.:	', intl_desig)
	
		###############################################################################################
		#	load mmt file
		print('\n	Loading MMT file...')
	
		#	open and read file
		with open(os.path.join(read_path, mmt_file)) as read_file:
			mmt_data = read_file.readlines()
	
		#	reset axes
#		ax_avg.cla()
		ax_std.cla()
	
		###################################################################################################
		#	parse mmt file
		
		print('	Parsing MMT file...')
	
		#	find start of data
		for line_id, line in enumerate(mmt_data):
			if line[0] != '#':
				break
		
		#	header line
		header_text = mmt_data[line_id - 1]
		
		#	strip header
		mmt_data = mmt_data[line_id:]
		
		n_rows = 13
		
		out_data = np.zeros([len(mmt_data), n_rows])
		
		for i, line in enumerate(mmt_data):
		
			#	split line
			date, mmt_time, mag_std, mag_app, opt_filter, penumbral, distance, \
				phase, channel, track_id = line.split(' ')
		
			#	parse data and set data types
			mag_std = float(mag_std)
			mag_app = float(mag_app)
			distance = float(distance)
			phase = float(phase)
			track_id = int(track_id)
		
			#	parse date
			year, month, day = date.split('-')
			year = int(year)
			month = int(month)
			day = int(day)
		
			#	parse time
			hour, minute, second = mmt_time.split(':')
			hour = int(hour)
			minute = int(minute)
			second = float(second)
		
			#	calculate julian date
			jday, jfrac = date2jd(year, month, day, hour, minute, second)
		
			#	generate output line
			output_line = [
				track_id,
				jday, jfrac,
				year, month, day,
				hour, minute, second,
				mag_std, mag_app,
				distance, phase
			]
		
			#	add to output array
			out_data[i, :] = output_line
		
		print('\n	Finished MMT file parsing...')
		print('		Total data length = ', len(out_data))
		print('		Total number of light curves = ', len(np.unique(out_data[:, 0])))
		
		###################################################################################################
		#	extract required data
		
		#	extract track ids
		track_id_list = np.unique(out_data[:, 0])
		
		print('\n	Looping through {} Track IDs...'.format(len(track_id_list)))
	
		lc_avg = []
		lc_std = []
		
		lc_avg_avg = []
		lc_std_avg = []
		
		full_datevec_array = []
		full_datevec_array_avg = []
		
		#	loop through starlink IDs
		for track_itt, track_id in enumerate(track_id_list):
		
			#	find track ids
			track_indices = np.where(out_data[:, 0]==track_id)[0]
		
			#	extract full data
			track_data = out_data[track_indices, :]
		
			###########################################################################################
			#	extract track time data
			
			mmt_line = track_data[0]
			
			#	parse line elements
			track_id, jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = track_data[0]
			
			start_jd = [jday, jfrac]
			
			#	parse line elements
			track_id, jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = track_data[-1]
			
			stop_jd = [jday, jfrac]
		
			###############################################################################################
			#	extract mmt brightness data
			mmt_signal_std = track_data[:, 9]
			mmt_signal_app = track_data[:, 10]
					
			###############################################################################################
			#	generate date vectors
			
			#	convert jd to date
			year, mon, day, hour, minute, sec = [*jd2date(*start_jd)]
			
			#	turn into string
			datestr = str('-'.join(
				['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
			))
			
			timestr = ':'.join(
				['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
			)
			
			datevec  = ' '.join([datestr, timestr])
		
			#	convert to datetime object
			date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
		
			#	append  to output array
			full_datevec_array.append(date_time_obj)
		
			###############################################################################################
			#	calculate moving averages
		
			lc_avg.append(np.average(mmt_signal_app))
			lc_std.append(np.std(mmt_signal_app))
		
			if track_itt > 3:
		
				avg_avg = (1 / 5) * (lc_avg[track_itt] + \
					lc_avg[track_itt - 1] + \
					lc_avg[track_itt - 2] + \
					lc_avg[track_itt - 3] + \
					lc_avg[track_itt - 4]) 
		
				lc_avg_avg.append(avg_avg)
		
				std_avg = (1 / 5) * (lc_std[track_itt] + \
					lc_std[track_itt - 1] + \
					lc_std[track_itt - 2] + \
					lc_std[track_itt - 3] + \
					lc_std[track_itt - 4]) 
		
				lc_std_avg.append(std_avg)
		
				full_datevec_array_avg.append(date_time_obj)
		

		lc_avg = np.array(lc_avg)
		lc_std = np.array(lc_std)

		full_lc_avg_avg = np.average(lc_avg)
		full_lc_std_avg = np.average(lc_std)

		output_line = [period, full_lc_std_avg]
		period_plot_data.append(output_line)

		###################################################################################################
		#	final figure output
		
		print('\n	Plotting data...')

#		ax_avg.scatter(full_datevec_array, lc_avg, marker = 'o', color='r', s=2)
#		ax_avg.plot(full_datevec_array_avg, lc_avg_avg, color='r', linewidth=1)
#		ax_avg.set_ylim([12.0, 0.0])
#		ax_avg.set_title('Average Magnitude')

		print('\n	Plotting std data...')
		ax_std.scatter(full_datevec_array, lc_std, marker = 'o', color='r', s=2)
		ax_std.plot(full_datevec_array_avg, lc_std_avg, color='r', linewidth=1)
		ax_std.set_ylim([0.0, 2.0])
		ax_std.set_title('Standard Deviation')




		plot_data = np.full([len(full_datevec_array_avg), 1], full_lc_std_avg)

		print('\n	', plot_data.shape)
		print('\n	', len(full_datevec_array_avg))


		plt.figure()
		plt.plot(
			full_datevec_array_avg,
			plot_data,
			color='r',
			linewidth=1
		)


		print('\n	PAUSING')
		plt.pause(2)



#		if status_string == 'Inactive Satellite':
#			print('\n	Plotting Inactive Satellite data...')
#
#
#			ax_std_full.plot(
#				full_datevec_array_avg,
#				plot_data,
#				color='r',
#				linewidth=1
#			)
#	
#		elif status_string == 'Active Satellite':
#
#			print('\n	Plotting Active Satellite data...')
#			ax_std_full.plot(
#				full_datevec_array_avg,
#				plot_data,
#				color='g',
#				linewidth=1
#			)
#
#		else:
#			print('FAILED MATCH')
#			pass
#	
#		ax_std_full.set_ylim([0.0, 2.0])
#		ax_std_full.set_title('Standard Deviation - Full')



#		plt.pause(0.1)
#		input('\n	Press Enter to continue to next object...')




period_plot_data = np.array(period_plot_data)

#	sort by npasses
period_plot_data = period_plot_data[period_plot_data[:,0].argsort()]

plt.plot(period_plot_data[:, 0], period_plot_data[:, 1])





plt.show()
sys.exit()

#	plt.pause(0.1)
#	input('\n	Press Enter to continue to next object...')
	
plt.show()
sys.exit()
