###################################################################################################
#	spacetrack launch file downloader
###################################################################################################

#	imports
import requests
import datetime
import sys
import os
from subprocess import call
import numpy as np

###################################################################################################
#	initialisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################')
print('Spacetrack Launch File Downloader.')

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

##################################################################################################
#	inputs
###################################################################################################

spacetrack_wildcard = '%5E'

##################################################################################################
#	batch 0
#	name of request
request_string = 'batch0'

#	launch year
launch_year = '19'

#	launch number
launch_number = '029'

#	intl desig
intl_desig = launch_year + launch_number + spacetrack_wildcard

##################################################################################################
#	batch 1
#	name of request
request_string = 'batch1'
#	intl desig
intl_desig = '19029%5E'

#	launch year
launch_year = '19'

#	launch number
launch_number = '074'

#	intl desig
intl_desig = launch_year + launch_number + spacetrack_wildcard

##################################################################################################
#	batch 2
#	name of request
request_string = 'batch2'

#	launch year
launch_year = '20'

#	launch number
launch_number = '001'

#	intl desig
intl_desig = launch_year + launch_number + spacetrack_wildcard

##################################################################################################
#	batch 3
#	name of request
request_string = 'batch3'

#	launch year
launch_year = '20'

#	launch number
launch_number = '006'

#	intl desig
intl_desig = launch_year + launch_number + spacetrack_wildcard
##################################################################################################
#	perform request
###################################################################################################

#	define read file
mmt_output_file = request_string + '_full.txt'

print('\n#####################################################################')
print('Checking for existance of Launch file.')

#	define output path
mmt_output_path = 'input_dir/mmt_files/starlink'

#	look for file
if os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file)):
	print('\n	File exists...')

else:
	print('\n	No file found... Performing request...')

	#	build request URL
	base_site = 'https://www.space-track.org'
	base_query = 'basicspacedata/query'
	req_type = 'class/tle_latest'
	launch_details = 'ORDINAL/1/INTLDES/{}'.format(intl_desig)
	output_ordering = 'orderby/TLE_LINE1%20DESC'
	output_format = 'format/3le'
	
	#	build final url
	req_url = '/'.join([base_site, base_query, req_type, launch_details, output_ordering, output_format])

	print('\n	Requesting from URL:\n', req_url)

	#	spacetrack login information
	login_url = 'https://www.space-track.org/ajaxauth/login'
	username = 'orbitalanalysts@ukspaceagency.gov.uk'
	password = 'Password009!!!!'
	
	payload = {
		'identity': username,
		'password': password
	}
	
	#	create request session
	with requests.Session() as session:
	
		#	perform login
		post = session.post(login_url, data=payload)
	
		#	request data
		output = session.get(req_url).text.split('\n')
	
	print('\n	Request output:\n')
	for line in output:
		print(line)
	
	#	save raw data
	with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'w') as open_file:

		for line in output:
			if line != '':
				open_file.write(line + '\n')

	print('\n	File saved...')

###################################################################################################
#	EOF
###################################################################################################
