###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

import sys
import os
import time
from subprocess import call

import numpy as np

import requests

from bs4 import BeautifulSoup

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Downloader')
print('#####################################################################################')

print('\nInitialising...')

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	define output path
mmt_input_path = 'input_dir/mmt_files/globalstar'

#	load existing flag
load_existing = False
load_existing = True

###################################################################################################
#	inputs
###################################################################################################

#	set name string
name_string = 'Globalstar'

#	add to output path
mmt_output_path = mmt_input_path + '/lightcurves'

###################################################################################################
#	load globalstar lookup table

print('\nLoading Globalstar Lookup File...')

#	load parsed globalstar lookup table file
globalstar_lookup_file = 'globalstar_mmt_lookup_table.txt'
globalstar_lookup_data = np.loadtxt(os.path.join(path_data, mmt_input_path, globalstar_lookup_file))

###################################################################################################
#	load globalstar batch TLE list

print('\nLooping through TLEs...')

objects_notin_mmt = 0

for data_line in globalstar_lookup_data:

	#	extract data IDs
	mmt_id, norad_id = data_line
	norad_id =int(norad_id)
	mmt_id = int(mmt_id)

	#	assemble URL
	base_url = 'http://mmt.favor2.info/satellites/'
	url = base_url + str(mmt_id) + '/download'
	
	#	generate output filename
	mmt_output_file = '_'.join([name_string, str(norad_id), 'MMT', str(mmt_id)]) + '.txt'
	
	print('\n	Using Filename: ', mmt_output_file)

	if os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file)):
		print('\n		File: ', mmt_output_file, ' already exists.')
	
	else:
		#######################################################################################
		#	perform request
	
		print('\n		Performing Request...')
		print('\n		Request URL:	', url)
	
		#	perform request
		r = requests.get(url)
	
		# Retrieve HTTP meta-data
		print('\n		Finished:')
		print('\n			Status code:	', r.status_code)
		print('			File Type:	', r.headers['content-type'])
		print('			Encoding:	', r.encoding)

		if r.status_code == 404:
			print('\n404 ERROR')
			sys.exit()

		with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'wb') as f:
		    f.write(r.content)
	
		print('\n		Finished Request, waiting 5 seconds...')
		time.sleep(5)

	#	add user input requirement for starting next object
	# input('\nWaiting for user input to continue.')

print('\nMMT File Request Complete.')

print('\nNumber of objects not found in MMT database = ', objects_notin_mmt)

###################################################################################################
#	EOF
###################################################################################################
