###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

import sys
import os
from subprocess import call

import requests

import numpy as np

from bs4 import BeautifulSoup


###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Downloader')
print('#####################################################################################')

print('\nInitialising...')

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	define output path
mmt_output_path = 'input_dir/mmt_files/globalstar'

#	define read file
mmt_output_file = 'globalstar_lookup_file.html'

lookup_table_file = 'globalstar_mmt_lookup_table.txt'

#	parsed globalstar lookup table file
output_file = 'globalstar_mmt_data_table.txt'

#	load existing flag
load_existing = False
load_existing = True

###################################################################################################
#	request inputs
###################################################################################################

#	NORAD ID
object_id = ''

#	object name string
name_string = 'globalstar'

#	look for file
print('\n	Generating File...')

###############################################################################################
#	acquire html data
###############################################################################################

#	look for file
if os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file)):
	file_exists = True
else:
	file_exists = False

#	load file if requested
if (file_exists is True) and (load_existing is True):

	print('\n	Loading HTML data...')

	#	load existing file
	with open(os.path.join(path_data, mmt_output_path, mmt_output_file)) as file:
		html_data = file.read()

elif ((file_exists is True) and (load_existing is False)) or \
		((file_exists is False) and (load_existing is True)):

	print('\n	Requesting HTML data...')

	url = 'http://mmt.favor2.info/satellites'

	form_data = {
		'catalogue_id': str(object_id),
		'name': name_string,
		'comments': '',
		'var_nonvariable': 'on',
		'var_variable': 'on',
		'var_periodic': 'on',
		'var_pmin': '',
		'var_pmax': '',
		'action': 'search',
		'type_0': 'on',
		'type_1': 'on',
		'type_2': 'on',
		'type_3': 'on',
		'type_4': 'on',
		'type_5': 'on',
		'type_6': 'on',
		'type_7': 'on',
		'orb_incl_min': '',
		'orb_incl_max': '',
		'orb_period_min': '',
		'orb_period_max': '',
	}

	#	perform request
	response = requests.post(url, data=form_data)
	html_data = response.text

	print('\nHTML_data = ', html_data)

	with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'w') as savefile:
		savefile.write(html_data)

###############################################################################################
#	parse html data
###############################################################################################

#	generate doc
doc = BeautifulSoup(html_data, 'html.parser')

data = doc.find_all('tr')

print('\n\n###################################################################################')

globalstar_data = []
loopup_table = []

for linenum, line in enumerate(data[8:]):

	obj_data = str(line).split('\n')

	mmt_id = int(obj_data[1].split('"')[1].split('/')[2])
	norad_id = int(obj_data[4].strip().split('<')[0])
	intl_desig = obj_data[7].split('>')[1].split('<')[0]
	name_str = obj_data[8].split('>')[1]

	print('\n################################################################################')
	print('\nmmt_id: ', mmt_id)
	print('norad_id: ', norad_id)
	print('intl_desig: ', intl_desig)
	print('name_str: ', name_str)
	print('\n')

	for line_itt1, line1 in enumerate(obj_data):
		print(line_itt1, line1)

	#	no comment
	if '<span' not in obj_data[11]:
		status_str = obj_data[13].split('"')[1]
		rot_state_str = obj_data[22].split('<')[1].split('>')[1]

	#	comment starts and ends in line 11
	elif '<span' in obj_data[11] and '</span>' in obj_data[11]:
		status_str = obj_data[14].split('"')[1]
		rot_state_str = obj_data[23].split('<')[1].split('>')[1]
	
	#	comment starts on line 11 and ends on line 12
	elif '<span' in obj_data[11] and '</span>' in obj_data[12]:
		status_str = obj_data[15].split('"')[1]
		rot_state_str = obj_data[24].split('<')[1].split('>')[1]

	#	comment starts on line 11 and ends on line 12
	elif '<span' in obj_data[11] and '</span>' in obj_data[13]:
		status_str = obj_data[16].split('"')[1]
		rot_state_str = obj_data[25].split('<')[1].split('>')[1]

	else:
		print('EXTRA COMMENT LINES?')
		sys.exit()

	print('\nstatus_str: ', status_str)
	print('rot_state_str: ', rot_state_str)

	data_line = [str(mmt_id), name_str, str(norad_id), intl_desig, status_str, rot_state_str]
	globalstar_data.append(data_line)

	loopup_table.append([mmt_id, norad_id])

lookup_table = np.array(loopup_table)

###################################################################################################
#	parse html data
###################################################################################################
	
print('\nSaving final file to:\n\n	', os.path.join(path_data, mmt_output_path, output_file))

with open(os.path.join(path_data, mmt_output_path, output_file), 'w') as write_file:

	#	print data to output file
	for line in globalstar_data:
		output_line = ', '.join(line) + '\n'
		write_file.write(output_line)
	
print('Saving final lookup table...')
np.savetxt(os.path.join(path_data, mmt_output_path, lookup_table_file), lookup_table)

###################################################################################################
#	EOF
###################################################################################################
