###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.colors as colors

from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('MMT testing script initilised.')

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	figure output

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Conferences',
	'IOC_2019',
	'technical_paper',
	'acta_figures',
	'Final'
)

fig_name = 'polar_comp' + '.eps'
out_str = os.path.join(fig_path, fig_name)

###################################################################################################
#	load polar data files
###################################################################################################

###################################################################################################
#	EG1 Inputs

#	light curve example file used
light_curve_example_file = 'lc_custom_eg1.txt'
polar_data_output_file = light_curve_example_file[:-4] + '_polar_data.txt'

##################################################
#	load polar data

print('\nLoading Polar Data file: ', polar_data_output_file)

#	set output file and path
polar_data_output_path = os.path.join(path_data, 'paper_plots/polar_data', polar_data_output_file)
polar_data_e1 = np.loadtxt(polar_data_output_path, delimiter=',')

#	normalise
polar_data_e1 = np.divide(polar_data_e1, polar_data_e1.max())

###################################################################################################
#	EG2 Inputs

#	light curve example file used
light_curve_example_file = 'lc_custom_eg2.txt'
polar_data_output_file = light_curve_example_file[:-4] + '_polar_data.txt'

##################################################
#	load polar data

print('\nLoading Polar Data file: ', polar_data_output_file)

#	set output file and path
polar_data_output_path = os.path.join(path_data, 'paper_plots/polar_data', polar_data_output_file)
polar_data_e2 = np.loadtxt(polar_data_output_path, delimiter=',')

#	normalise
polar_data_e2 = np.divide(polar_data_e2, polar_data_e2.max())

###################################################################################################
#	EG3 Inputs

#	light curve example file used
light_curve_example_file = 'lc_custom_eg3.txt'
polar_data_output_file = light_curve_example_file[:-4] + '_polar_data.txt'

##################################################
#	load polar data

print('\nLoading Polar Data file: ', polar_data_output_file)

#	set output file and path
polar_data_output_path = os.path.join(path_data, 'paper_plots/polar_data', polar_data_output_file)
polar_data_e3 = np.loadtxt(polar_data_output_path, delimiter=',')

#	normalise
polar_data_e3 = np.divide(polar_data_e3, polar_data_e3.max())

###################################################################################################
#	EG4 Inputs

#	light curve example file used
light_curve_example_file = 'lc_custom_eg4.txt'
polar_data_output_file = light_curve_example_file[:-4] + '_polar_data.txt'

##################################################
#	load polar data

print('\nLoading Polar Data file: ', polar_data_output_file)

#	set output file and path
polar_data_output_path = os.path.join(path_data, 'paper_plots/polar_data', polar_data_output_file)
polar_data_e4 = np.loadtxt(polar_data_output_path, delimiter=',')

#	normalise
polar_data_e4 = np.divide(polar_data_e4, polar_data_e4.max())

###################################################################################################
#	graphical outputs
###################################################################################################

####################################################
#	generate polar mesh grid

#	110 so that 90 gets plotted
inc_angle_range = np.arange(0, 110, 10)
azm_angle_range = np.arange(0, 370, 10)

r, th = np.meshgrid(inc_angle_range, azm_angle_range)

####################################################
#	set figure-wide parameters

dpi_val = 300
small = 1e-15
inch2cm = 2.54

####################################################
#	select colour map
# cmap = plt.cm.get_cmap('cividis_r')
# cmap = plt.cm.get_cmap('Wistia_r')
cmap = plt.cm.get_cmap('summer_r')

for n in np.arange(0, 1.1, 0.1):

	print(np.round(n, 1), np.round(cmap(n)[0:3], 1))

sys.exit()

cmap.set_under('w')
 
####################################################
#	find azimuth offset to align with grid
nazm_vals = len(th) - 1
offset = (360 / nazm_vals) / 2

####################################################
#	figure parameters
font_sizex = 6
font_sizey = 6
font_pad = -3
marker_size = 4
marker_edge = 1

hspace_val = 0.35
wspace_val = 0.35

grid_width = 0.2
grid_color = 'k'
grid_color = [0.4, 0.4, 0.4]
# alt_line_width = 0.01

title_size = 8
title_pad = 10

# ####################################################
# #	set figure wide colour bounds
# 
# #	set number of colours to use
# n_col = 100
# 
# #	get max vals
# e1_max = polar_data_e1.max()
# e2_max = polar_data_e2.max()
# e3_max = polar_data_e3.max()
# e4_max = polar_data_e4.max()
# 
# #	generate array of max vals
# max_vals = np.array([
# 	[e1_max, 0],
# 	[e2_max, 1],
# 	[e3_max, 2],
# 	[e4_max, 3]
# ])
# 
# #	sort decending
# max_vals = max_vals[max_vals[:, 0].argsort()][::-1]
# 
# #	take largest pixel
# max_arg = max_vals[0, 1]
# max_val = max_vals[0, 0]
# 
# #	extract data array containing largest pixel
# if max_arg == 0:
# 	maxdata = polar_data_e1
# elif max_arg == 1:
# 	maxdata = polar_data_e2
# elif max_arg == 2:
# 	maxdata = polar_data_e3
# elif max_arg == 3:
# 	maxdata = polar_data_e4
# 
# #	round max val and set step
# newmax = np.round(maxdata.max(), 1)
# step = 0.1
# 
# #	set bounds
# bounds = np.linspace(0, newmax + step, n_col)
# 
# #	generate colour bar bounds
# bounds[0] = small
# norm = colors.BoundaryNorm(boundaries=bounds, ncolors=256)
# 
# #	set step tickmarks
# tick_marks = np.arange(0.0, newmax + (step), step)

####################################################
#	initialise figure
plt.rc('text', usetex=True)

fig = plt.figure(figsize=(9.0/inch2cm, 9.0/inch2cm), dpi=dpi_val)

plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

###################################################################################################
#	example 1

####################################################
#	initialise axes
ax1 = plt.subplot(221, projection="polar")
ax1.set_theta_zero_location("N")
ax1.set_theta_direction(-1)

ax1.set_title(r'$a)\ Example 1$', pad=title_pad, fontsize=title_size)

ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax1.set_rticks(np.arange(0, 100, 20))
ax1.set_rlim(0, 100)

####################################################
#	colour the plot
# plt.pcolor(np.radians(th - offset), r, polar_data_e1, norm=norm, cmap=cmap)
plt.pcolor(np.radians(th - offset), r, polar_data_e1, cmap=cmap)

####################################################
#	plot rotation vector

#	attitude
rot_vec_azm_deg = 267.0
rot_vec_inc_deg = 65.0 + 10.0

ax1.plot(np.radians(rot_vec_azm_deg), rot_vec_inc_deg,
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
ax1.grid(which='major', axis='both', linewidth=grid_width, color=grid_color)

label_position=ax1.get_rlabel_position()
ax1.text(
	np.radians(70),
	ax1.get_rmax() / 2.,
	r'$\phi$',
	ha='center',
	va='center'
)

ax1.text(
	np.radians(160),
	ax1.get_rmax() + 15,
	r'$\psi$',
	ha='center',
	va='center'
)

###################################################################################################
#	example 2

####################################################
#	initialise axes
ax2 = plt.subplot(222, projection="polar")
ax2.set_theta_zero_location("N")
ax2.set_theta_direction(-1)

ax2.set_title(r'$b)\ Example 2$', pad=title_pad, fontsize=title_size)

ax2.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax2.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax2.set_rticks(np.arange(0, 100, 20))
ax2.set_rlim(0, 100)

####################################################
#	colour the plot
# plt.pcolor(np.radians(th - offset), r, polar_data_e2, norm=norm, cmap=cmap)
plt.pcolor(np.radians(th - offset), r, polar_data_e2, cmap=cmap)

####################################################
#	plot rotation vector

#	attitude
rot_vec_azm_deg = 114.0
rot_vec_inc_deg = 15.0 + 10.0

ax2.plot(np.radians(rot_vec_azm_deg), rot_vec_inc_deg,
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
ax2.grid(which='major', axis='both', linewidth=grid_width, color=grid_color)

#ax2.set_xlabel(r'$\psi$')

label_position=ax2.get_rlabel_position()
ax2.text(
	np.radians(70),
	ax2.get_rmax() / 2.,
	r'$\phi$',
	ha='center',
	va='center'
)

ax2.text(
	np.radians(160),
	ax2.get_rmax() + 15,
	r'$\psi$',
	ha='center',
	va='center'
)

###################################################################################################
#	example 3

####################################################
#	initialise axes
ax3 = plt.subplot(223, projection="polar")
ax3.set_theta_zero_location("N")
ax3.set_theta_direction(-1)

ax3.set_title(r'$c)\ Example 3$', pad=title_pad, fontsize=title_size)

ax3.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax3.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax3.set_rticks(np.arange(0, 100, 20))
ax3.set_rlim(0, 100)

####################################################
#	colour the plot
# plt.pcolor(np.radians(th - offset), r, polar_data_e3, norm=norm, cmap=cmap)
plt.pcolor(np.radians(th - offset), r, polar_data_e3, cmap=cmap)

####################################################
#	plot rotation vector

#	attitude
rot_vec_azm_deg = 160.0
rot_vec_inc_deg = 89.0 + 10.0

ax3.plot(np.radians(rot_vec_azm_deg), rot_vec_inc_deg,
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
ax3.grid(which='major', axis='both', linewidth=grid_width, color=grid_color)

ax3.text(
	np.radians(70),
	ax3.get_rmax() / 2.,
	r'$\phi$',
	ha='center',
	va='center'
)

ax3.text(
	np.radians(160),
	ax3.get_rmax() + 15,
	r'$\psi$',
	ha='center',
	va='center'
)

###################################################################################################
#	example 4

####################################################
#	initialise figure
ax4 = plt.subplot(224, projection="polar")
ax4.set_theta_zero_location("N")
ax4.set_theta_direction(-1)

ax4.set_title(r'$d)\ Example 4$', pad=title_pad, fontsize=title_size)

ax4.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax4.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax4.set_rticks(np.arange(0, 100, 20))
ax4.set_rlim(0, 100)

####################################################
#	colour the plot
# plt.pcolor(np.radians(th - offset), r, polar_data_e4, norm=norm, cmap=cmap)
plt.pcolor(np.radians(th - offset), r, polar_data_e4, cmap=cmap)

####################################################
#	plot rotation vector

#	attitude
rot_vec_azm_deg = 324.0
rot_vec_inc_deg = 9.0 + 10.0

ax4.plot(np.radians(rot_vec_azm_deg), rot_vec_inc_deg,
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
ax4.grid(which='major', axis='both', linewidth=grid_width, color=grid_color)

ax4.text(
	np.radians(70),
	ax4.get_rmax() / 2.,
	r'$\phi$',
	ha='center',
	va='center'
)

ax4.text(
	np.radians(160),
	ax4.get_rmax() + 15,
	r'$\psi$',
	ha='center',
	va='center'
)

####################################################
#	add colour bar
plt.subplots_adjust(bottom=0.1, right=0.8)
cax = plt.axes([0.87, 0.1, 0.03, 0.8])

# plt.colorbar(cax=cax, ticks=tick_marks)
cbar = plt.colorbar(cax=cax)
cbar.set_label(r'$RMS$')

cax.tick_params(axis='y', labelsize=font_sizey, length=3, direction='in')

####################################################
#	saving figure
plt.show()

#	ask to save light curve file for testing
user_input = input(
	'\nSave Light Curve to:\n	{}? [y/n] '.format(out_str)
)

if user_input == 'y':
	
	print('\n	Saving Light Curve...')
	#	save light curve to file
	figure.savefig(out_str, bbox_inches="tight", dpi=300)

elif user_input == 'no':
	print('Not saved.')
sys.exit()
