#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('IOC Plotter.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

time_step = 0.1

#	output file path
paper_path = os.path.join(
	os.environ['HOME'],
	'PhD', 'Documents', 'Conferences', 'IOC_2019', 'technical_paper', 'acta_figures', 'Final'
)

poster_path = os.path.join(
	os.environ['HOME'],
	'PhD', 'Documents', 'Conferences', 'IOC_2019', 'Poster'
)

fig_name = 'synthetic_lc_comp' + '.eps'
out_str = os.path.join(poster_path, fig_name)

###################################################################################################
#	load example 1 data
###################################################################################################

print('\nLoading custom light curve data...')

output_path = os.path.join(path_data, 'random_light_curves')
eg1_file = 'lc_custom_eg1.txt'

#	load custom light curve
custom_eg1 = np.loadtxt(
	os.path.join(output_path, eg1_file),
	delimiter=','
)

example1_time = custom_eg1[:, 1]
example1_app = custom_eg1[:, 2]

###################################################################################################
#	load example 2 data
###################################################################################################

print('\nLoading custom light curve data...')

output_path = os.path.join(path_data, 'random_light_curves')
eg2_file = 'lc_custom_eg2.txt'

#	load custom light curve
custom_eg2 = np.loadtxt(
	os.path.join(output_path, eg2_file),
	delimiter=','
)

example2_time = custom_eg2[:, 1]
example2_app = custom_eg2[:, 2]

###################################################################################################
#	load example 3 data
###################################################################################################

print('\nLoading custom light curve data...')

output_path = os.path.join(path_data, 'random_light_curves')
eg3_file = 'lc_custom_eg3.txt'

#	load custom light curve
custom_eg3 = np.loadtxt(
	os.path.join(output_path, eg3_file),
	delimiter=','
)

example3_time = custom_eg3[:, 1]
example3_app = custom_eg3[:, 2]

###################################################################################################
#	load example 4 data
###################################################################################################

print('\nLoading custom light curve data...')

output_path = os.path.join(path_data, 'random_light_curves')
eg4_file = 'lc_custom_eg4.txt'

#	load custom light curve
custom_eg4 = np.loadtxt(
	os.path.join(output_path, eg4_file),
	delimiter=','
)

example4_time = custom_eg4[:, 1]
example4_app = custom_eg4[:, 2]


###################################################################################################
#	figure parameters
###################################################################################################

####################################################
#	set figure-wide parameters

dpi_val = 300
inch2cm = 2.54

font_sizex = 6
font_sizey = 6
font_pad = 0.5

label_sizex = 6
label_sizey = 6

marker_size = 4
marker_edge = 1

#	set plot index
max_index = int(9.2 * 20)

hspace_val = 0.3
wspace_val = 0.2

grid_width = 0.1
alt_line_width = 0.1

title_size = 8
title_pad = 3

figure = plt.figure(num=None, figsize=(18.0 / inch2cm, 10.0 / inch2cm),
 	dpi=dpi_val, facecolor='w', edgecolor='k'
)

plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
plt.rc('text', usetex=True)
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

font_size = 5

color_spec = [0.0, 0.0, 0.0]
linewidth_val = 0.8

###################################################################################################
#	axis 1
ax1 = plt.subplot(221)
ax1.plot(example1_time, example1_app, linewidth=linewidth_val, color=color_spec)

#	reverse y axis
ax1.invert_yaxis()

#	set grid
# ax1.grid()

#	set title
# ax1.set_title(r'$' + title_str1 + '$', pad=title_pad, fontsize=title_size)
ax1.set_title(r'$a)\ Example\ 1$', pad=title_pad, fontsize=title_size)

ax1.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax1.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)

###################################################################################################
#	axis 2
ax2 = plt.subplot(222)
ax2.plot(example2_time, example2_app, linewidth=linewidth_val, color=color_spec)

#	reverse y axis
ax2.invert_yaxis()

#	set grid
# ax2.grid()

#	set title
ax2.set_title(r'$a)\ Example\ 2$', pad=title_pad, fontsize=title_size)

ax2.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax2.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax2.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax2.tick_params(axis='y', labelsize=font_sizey, pad=0)

###################################################################################################
#	axis 3
ax3 = plt.subplot(223)
ax3.plot(example3_time, example3_app, linewidth=linewidth_val, color=color_spec)

#	reverse y axis
ax3.invert_yaxis()

#	set grid
# ax3.grid()

#	set title
ax3.set_title(r'$a)\ Example\ 3$', pad=title_pad, fontsize=title_size)

ax3.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax3.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax3.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax3.tick_params(axis='y', labelsize=font_sizey, pad=0)

###################################################################################################
#	axis 4
ax4 = plt.subplot(224)
ax4.plot(example4_time, example4_app, linewidth=linewidth_val, color=color_spec)

#	reverse y axis
ax4.invert_yaxis()

#	set grid
# ax1.grid()

#	set title
ax4.set_title(r'$a)\ Example\ 4$', pad=title_pad, fontsize=title_size)

ax4.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax4.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax4.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax4.tick_params(axis='y', labelsize=font_sizey, pad=0)

####################################################
#	saving figure
plt.show()

#	ask to save light curve file for testing
user_input = input(
	'\nSave Light Curve to:\n	{}? [y/n] '.format(out_str)
)

if user_input == 'y':
	
	print('\n	Saving Light Curve...')
	#	save light curve to file
	figure.savefig(out_str, bbox_inches="tight", dpi=300)

elif user_input == 'no':
	print('Not saved.')
sys.exit()
