###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front end
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	GLIMMA 
from GLIMMA import glimma

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	parrellisation
from mpi4py import MPI

import progressbar

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front end script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	MPI initialisation

#	initialising MPI communicator
comm = MPI.COMM_WORLD

#	this is the name of the currently executing node
name = MPI.Get_processor_name()

#	this is the rank of the currently executing process
rank = comm.Get_rank()

#	this is the total number of visible processes
size = comm.Get_size()

# print('Current processor = ', name, ' with rank ', rank)


print('\nExecuting on:-  Node = ', name, '; With ', size, ' Cores.')
print('Current rank is ', rank)


sys.exit()

#	root process code
if rank == 0:

	##################################################
	#	file paths
	root_path = os.path.join(os.environ['HOME'], 'codebase')
	archive_path = 'data_archive/paper_data'
	path_data = os.path.join(root_path, archive_path)
	
	##################################################
	#	simulation parameters
	
	#	case designation
	case_desig = 'bulk_run'

	#	case subdesignation
	case_subdesig = 'Fine'
	
	#	time step in seconds
	time_step = 1
	
	#	length of data reccord [number of points]
	len_reccord = 60 * 4
	
	#	parralellisation flag
	para = False
	
	##################################################
	#	Object 25885 TLE
	
	#	end of 2016 tle
	tle_line1 = '1 25885U 99043C   16347.16557405 -.00000088  00000-0 -43593-3 0  9992'
	tle_line2 = '2 25885  52.0136 114.2293 0006645 165.6082 252.1930 11.39172050765052'
	
	sim_para = [root_path, archive_path, len_reccord, time_step, tle_line1, tle_line2]
	
	##################################################
	#	parse case files
	
	#	define path to lightcurve directory
	lightcurve_path = os.path.join(root_path, archive_path, 'lightcurve_dir', case_desig)
	output_path = os.path.join(lightcurve_path, 'total_lightcurves')
	
	#	set input case path
	case_path = os.path.join(path_data, 'case_files', case_desig, case_subdesig)

	#	find number of case files
	case_list = os.listdir(case_path)
	ncase_files = len(case_list)

	print('Opening: ', case_list[7])
	with open(os.path.join(case_path, case_list[7]), mode='r', encoding='utf-8') as case_file:
		total_case_data = case_file.read()

	ncases = len(total_case_data)
	
	#	split data and convert to numpy array
	total_case_data = total_case_data.split('\n')
	total_case_data = np.array(total_case_data)

	#	remove final newline character
	if total_case_data[-1] == '':
		total_case_data = total_case_data[0:-1]

	print('Total case data lines = ', len(total_case_data))
	print('Total case data shape = ', total_case_data.shape)
	print('Total case data type = ', type(total_case_data))

	sys.exit()


	send_buffer = total_data

#	all other process code
else:
	#	initialise inputs
	root_path = None
	archive_path = None

	tle_line1 = None
	tle_line2 = None

	len_reccord = np.zeros(1, dtype=int)
	time_step = np.zeros(1, dtype=float)

	send_buffer = None



print('\nRank is ', rank, '. root_path = ', root_path)

#	broadcast data to workers
root_path = comm.bcast(root_path, root=0)
archive_path = comm.bcast(archive_path, root=0)
tle_line1 = comm.bcast(tle_line1, root=0)
tle_line2 = comm.bcast(tle_line2, root=0)
 
print('\nRank is ', rank, '. Broadcasted root_path = ', root_path)
print('\nRank is ', rank, '. Broadcasted archive_path = ', archive_path)
print('\nRank is ', rank, '. Broadcasted tle_line1 = ', tle_line1)
print('\nRank is ', rank, '. Broadcasted tle_line2 = ', tle_line2)

len_reccord = comm.bcast(len_reccord, root=0)
time_step = comm.bcast(time_step, root=0)

print('\nRank is ', rank, '. Broadcasted len_reccord = ', len_reccord)
print('\nRank is ', rank, '. Broadcasted time_step = ', time_step)

receive_buffer = None
	
#	receive scattered array from root
comm.scatter(send_buffer, receive_buffer, root=0)

print('\nRank is ', rank, '. Scattered run_data = ', receive_buffer)


#	print(proc_data)

sys.exit()

input('\nhold')
sys.exit()

###################################################################################################
#	run GLIMMA
###################################################################################################

first_flag = True


#	parrellised
if para is True:


	light_curve_data = main(pool, total_data)

	#	generate output array
	for data_array in light_curve_data:
		light_curve_save = np.array(data_array)

		if first_flag is True:
			output_data = light_curve_save
			first_flag = False

		else:
			output_data = np.vstack((output_data, light_curve_save))

#	sequential
elif para is False:

	for run_string in total_data:

		#	generate lightcurve
		light_curve = glimma(run_string)
		light_curve_data = np.array(light_curve[:, -1]).astype(np.float)

		if np.isnan(light_curve_data).any():
			print('ERROR: nan value found')
			sys.exit()


#		#	generate time data
#		time_data = np.zeros([len(light_curve)])
#
#		for i, time_line in enumerate(light_curve[:, 1]):
#			time_data[i] = 86400 * (light_curve[i, 1] - light_curve[0, 1])
#
#		#	plot lightcurve
#		plt.plot(time_data, light_curve[:,3])
#		plt.grid()
#		plt.show()
#
#		sys.exit()

#	save output file
with open(os.path.join(output_path, output_file), mode='w', encoding='utf-8') as case_file:

	for line in output_data:
		output_line = ', '.join([*line]) + '\n'
		case_file.write(output_line)

sim_end = time.time()
print('\n GLIMMA Complete.')
print('Time taken = ', (sim_end - sim_start) / 60)
