import sys
import os
import numpy as np

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.patches import FancyArrowPatch

from tools.functions.functions_transform import site_vec, rot_mat, gen_vectors_render, rot1, rot2, rot3

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	define figure properties

#	coordinate origin
origin = [0, 0, 0]
xaxis = [5, 0, 0]
yaxis = [0, 5, 0]
zaxis = [0, 0, 5]

#	vector length
vec_length = 2

#	set flag
savefile_flag = True
savefile_flag = False

fig_path = os.path.join(os.environ['HOME'],	'PhD', 'Reports', 'Journals', 'ACTA_Astronautica',
	'acta_figures',
	'Final'
)

fig_name = 'reflection_geometry.eps'
out_path = os.path.join(fig_path, fig_name)

###################################################################################################
#	initialise figure

#	graphical output
inch2cm = 2.54

figure = plt.figure(num=None, figsize=(9/inch2cm, 9/inch2cm),
 	dpi=300, facecolor='w', edgecolor='k'
)

plt.rc('text', usetex=True)
plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
ax = plt.axes(projection='3d')

###################################################################################################
#	plot axes

# #	x axis
# xaxis_plot = np.vstack([origin, xaxis])
# c = Arrow3D(xaxis_plot[:, 0], xaxis_plot[:, 1], xaxis_plot[:, 2],
# 	mutation_scale=5,
# 	lw=0.25,
# 	arrowstyle="-|>",
# 	color="k"
# )
# 
# ax.add_artist(c)
# ax.text(xaxis_plot[1, 0], xaxis_plot[1, 1], xaxis_plot[1, 2],
# 	'x-axis',
# 	fontsize=2,
# 	fontweight='bold'
# )
# 
# #	y axis
# yaxis_plot = np.vstack([origin, yaxis])
# c = Arrow3D(yaxis_plot[:, 0], yaxis_plot[:, 1], yaxis_plot[:, 2],
# 	mutation_scale=5,
# 	lw=0.25,
# 	arrowstyle="-|>",
# 	color="k"
# )
# 
# ax.add_artist(c)
# ax.text(yaxis_plot[1, 0], yaxis_plot[1, 1], yaxis_plot[1, 2],
# 	'y-axis',
# 	fontsize=2,
# 	fontweight='bold'
# )
# 
# #	z axis
# zaxis_plot = np.vstack([origin, zaxis])
# c = Arrow3D(zaxis_plot[:, 0], zaxis_plot[:, 1], zaxis_plot[:, 2],
# 	mutation_scale=5,
# 	lw=0.25,
# 	arrowstyle="-|>",
# 	color="k"
# )
# 
# ax.add_artist(c)
# ax.text(zaxis_plot[1, 0], zaxis_plot[1, 1], zaxis_plot[1, 2],
# 	'z-axis',
# 	fontsize=2,
# 	fontweight='bold'
# )

###################################################################################################
#	plot vectors

#	Surface Normal
Nvec = [0, 0, 1]
Nvec = np.divide(Nvec, np.linalg.norm(Nvec))
Nvec = np.multiply(Nvec, vec_length)
Nvec_plot = np.vstack([origin, Nvec])

#	Illumination Vector
illum_vec = [2, 1, 1]
illum_vec = np.divide(illum_vec, np.linalg.norm(illum_vec))
illum_vec = np.multiply(illum_vec, vec_length)
illum_vec_plot = np.vstack([origin, illum_vec])

#	Observation Vector
obs_vec = [-2, 1, 1]
obs_vec = np.divide(obs_vec, np.linalg.norm(obs_vec))
obs_vec = np.multiply(obs_vec, vec_length)
obs_vec_plot = np.vstack([origin, obs_vec])

#	Angular bisector
Hvec = np.divide(np.add(obs_vec, illum_vec), len(np.add(obs_vec, illum_vec)))
Hvec = np.divide(Hvec, np.linalg.norm(Hvec))
Hvec = np.multiply(Hvec, vec_length)
Hvec_plot = np.vstack([origin, Hvec])

###################################################################################################
#	define circle data

#	define radius
radius = 1

#	generate circle data
n = 360
circle_vals = []

for i in range(0, n + 1):
	x = np.cos(2 * np.pi / n * i) * radius
	y = np.sin(2 * np.pi / n * i) * radius
	z = 0

	outvec = np.array([x, y, z])
	circle_vals.append(outvec)

circle_vals = np.array(circle_vals)

#	angle of Hvec from N
arccos_of = np.divide(
	np.dot(Hvec, Nvec),
	np.linalg.norm(Hvec) * np.linalg.norm(Nvec)
)

#	generate circle 1
angle1 = np.arccos(arccos_of)

#	angle of L from xaxis
arccos_of = np.divide(
	np.dot(illum_vec, xaxis),
	np.linalg.norm(illum_vec) * np.linalg.norm(xaxis)
)

angle2 = np.arccos(arccos_of)

#	angle of L from V
arccos_of = np.divide(
	np.dot(illum_vec, obs_vec),
	np.linalg.norm(illum_vec) * np.linalg.norm(obs_vec)
)

angle3 = np.arccos(arccos_of)

#	angle of Hvec from zaxis
arccos_of = np.divide(
	np.dot(Hvec, zaxis),
	np.linalg.norm(Hvec) * np.linalg.norm(zaxis)
)

angle4 = np.arccos(arccos_of)

###################################################################################################
#	plot theta angle circle

#	generate circle 1
circle1 = []

#	rotate for N-Hvec circle
for vec in circle_vals:

	outvec = rot3(vec, -angle2)

	outvec = rot1(outvec, -angle1)

	circle1.append(outvec)

circle1 = np.array(circle1)
circle1_lower = 0
circle1_upper = int(np.degrees(angle3))

#	plot circle
plt.plot(
	circle1[circle1_lower:circle1_upper, 0],
	circle1[circle1_lower:circle1_upper, 1],
	circle1[circle1_lower:circle1_upper, 2] + 2,
	linewidth=0.5,
	color='k'
)

#	add text
ax.text(-0.6, 0.75, 0.6 + 2, r'$\theta$', fontsize=5)
ax.text(0.8, 0.75, 0.6 + 2, r'$\theta$', fontsize=5)

###################################################################################################
#	plot alpha angle circle

#	generate circle 2
angle5 = np.pi / 2
circle2 = []

#	rotate for N-Hvec circle
for vec in circle_vals:
	outvec = rot2(vec, angle5)
	circle2.append(outvec)

circle2 = np.array(circle2)
circle2_lower = 0
circle2_upper = int(np.degrees(angle4))

#	plot circle
plt.plot(
	circle2[circle2_lower:circle2_upper, 0],
	circle2[circle2_lower:circle2_upper, 1],
	circle2[circle2_lower:circle2_upper, 2] + 2,
	linewidth=0.5,
	color='k'
)

#	add text
ax.text(0.0, 0.40, 1.0 + 2, r'$\alpha$', fontsize=5)

###################################################################################################


c = Arrow3D(Nvec_plot[:, 0], Nvec_plot[:, 1], Nvec_plot[:, 2] + 2,
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(Nvec_plot[1, 0], Nvec_plot[1, 1], Nvec_plot[1, 2] + 2,
	r'$\boldsymbol{N}$',
	fontsize=5,
)

c = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2] + 2,
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2] + 2,
	r'$\boldsymbol{L}$',
	fontsize=5,
	fontweight='bold'
)

c = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2] + 2,
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2] + 0.1 + 2,
	r'$\boldsymbol{V}$',
	fontsize=5,
	fontweight='bold'
)

c = Arrow3D(Hvec_plot[:, 0], Hvec_plot[:, 1], Hvec_plot[:, 2] + 2,
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(Hvec_plot[1, 0], Hvec_plot[1, 1], Hvec_plot[1, 2] + 2,
	r'$\boldsymbol{H}$',
	fontsize=5,
	fontweight='bold'
)

###################################################################################################
#	plot surface

#	define surface verts
vert1 = [2, 2, 0]
vert2 = [2, -2, 0]
vert3 = [-2, -2, 0]
vert4 = [-2, 2, 0]

#	generate plotable vectors
vec1 = np.vstack([vert1, vert2])
vec2 = np.vstack([vert2, vert3])
vec3 = np.vstack([vert3, vert4])
vec4 = np.vstack([vert4, vert1])

ax.plot(vec1[:, 0], vec1[:, 1], vec1[:, 2] + 2, color='k', linewidth=1)
ax.plot(vec2[:, 0], vec2[:, 1], vec2[:, 2] + 2, color='k', linewidth=1)
ax.plot(vec3[:, 0], vec3[:, 1], vec3[:, 2] + 2, color='k', linewidth=1)
ax.plot(vec4[:, 0], vec4[:, 1], vec4[:, 2] + 2, color='k', linewidth=1)

ax.text(-1, -1, 0 + 2, r'$A$', fontsize=5, fontweight='bold')

#	plot normal right angle
vert1 = [0, -0.25, 0]
vert2 = [0, -0.25, 0.25]
vert3 = [0, 0, 0.25]

vec1 = np.vstack([vert3, vert2])
vec2 = np.vstack([vert2, vert1])
ax.plot(vec1[:, 0], vec1[:, 1], vec1[:, 2] + 2, color='k', linewidth=1)
ax.plot(vec2[:, 0], vec2[:, 1], vec2[:, 2] + 2, color='k', linewidth=1)

###################################################################################################
#	finalise figure

ax.set_xlim(-1.6, 1.6)
ax.set_ylim(-1.6, 1.6)
ax.set_zlim(0.4, 3.6)

ax.view_init(10, -122)
plt.axis('off')

if savefile_flag is True:
	plt.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
