###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	combined elevation/azimuth polar plot code
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
from subprocess import call

#	numpy
import numpy as np

#	scipy
import scipy
from scipy import optimize

#	numpy-stl
from stl import mesh

#	matplotlib
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from matplotlib.colors import BoundaryNorm
from matplotlib.colors import ListedColormap, LinearSegmentedColormap
from matplotlib.ticker import MaxNLocator
from mpl_toolkits.mplot3d import Axes3D

##################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#   initilisation
###################################################################################################

#	clear terminal
call(['clear'])
print('\nGenerating figures...')

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	figure output

#	set flag
savefile_flag = True
savefile_flag = False

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Journals',
	'ACTA_Astronautica',
	'acta_figures',
	'Final'
)

##################################################
#	inputs

sim_step = 0.1

#	geometry case
geom_case = 'cylinder'

geom_case = 'div1'
geom_case = 'div5'
geom_case = 'cube'

#	reflection case
reflection_case = 'specular'
reflection_case = 'diffuse'

# #	bulk run directory
# if geom_case == 'cube':
# 	bulk_dir = 'distance_data_cube_{}'.format(str(sim_step))
# 	sub_dir = geom_case
# 
# elif geom_case == 'div1':
# 	bulk_dir = 'distance_data_div1_{}'.format(str(sim_step))
# 	sub_dir = 'sphere_' + geom_case
# 
# elif geom_case == 'div5':
# 	bulk_dir = 'distance_data_div5_{}'.format(str(sim_step))
# 	sub_dir = 'sphere_' + geom_case
# 
# elif geom_case == 'cylinder':
# 	bulk_dir = 'distance_data_cylinder_{}'.format(str(sim_step))
# 	sub_dir = geom_case

#	bulk run directory
if geom_case == 'cube':
	bulk_dir = 'rms_data_cube_{}'.format(str(sim_step))
	sub_dir = geom_case

elif geom_case == 'div1':
	bulk_dir = 'rms_data_div1_{}'.format(str(sim_step))
	sub_dir = 'sphere_' + geom_case

elif geom_case == 'div5':
	bulk_dir = 'rms_data_div5_{}'.format(str(sim_step))
	sub_dir = 'sphere_' + geom_case

elif geom_case == 'cylinder':
	bulk_dir = 'rms_data_cylinder_{}'.format(str(sim_step))
	sub_dir = geom_case

###################################################################################################
#	acquiring 0-attit case data
###################################################################################################

#	set input case designation
case_desig_0attit = '_'.join(['0-attit', geom_case])

# #	input data
# distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_0attit)
# distance_dir = os.listdir(distance_path)
# 
# print('\nGetting distances from: ', distance_path)

#	input data
rms_path = os.path.join(path_data, 'rms_dir', bulk_dir, reflection_case, case_desig_0attit)
rms_dir = os.listdir(rms_path)

print('\nGetting rmses from: ', rms_path)

#########################################################################################
#	sort distance files
#distance_files = []
rms_files = []

#coefs_files_0attit = []
case_num = 0

for rms_file in rms_dir:

	attit_case = rms_file.split('_')[1]

	if rms_file.split('.')[1] == 'dist':
		rms_files.append([int(case_num), rms_file])

#	elif distance_file.split('.')[1] == 'coefs':
#		coefs_files_0attit.append([int(case_num), distance_file])

	case_num += 1

#distance_files = sorted(distance_files, key=lambda x: x[0])
rms_files = sorted(rms_files, key=lambda x: x[0])
#coefs_files_0attit = sorted(coefs_files_0attit, key=lambda x: x[0])

#########################################################################################
#	generate data
data = []
#for i, distance_file in enumerate(distance_files):
for i, rms_file in enumerate(rms_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		rms_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	rms = np.genfromtxt(os.path.join(rms_path, rms_file[1]), delimiter=',')
	data_line = [elv, azm, rms]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	extract azimuth and elevation angles
elv_vals = np.unique(data[:, 0])
azm_vals = np.unique(data[:, 1])

#	reset to include 100 so that 90 is plotted
elv_vals = np.arange(0, 110, 10)

#	generate polar mesh grid
r, th = np.meshgrid(elv_vals, azm_vals)

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)

	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		rms_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = rms_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
rms_column = data_of_elv[:, 2]
#data_array[:, column_index] = distance_column
data_array[:, column_index] = rms_column

#	set plot data
attit0_data = data_array

###################################################################################################
#	acquiring x-rot case data
###################################################################################################

#	change case designation and repeat
case_desig_xrot = '_'.join(['x-rot', geom_case])

# #	input data
# distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_xrot)
# distance_dir = os.listdir(distance_path)
# print('\nGetting distances from: ', distance_path)

#	input data
rms_path = os.path.join(path_data, 'rms_dir', bulk_dir, reflection_case, case_desig_xrot)
rms_dir = os.listdir(rms_path)
print('\nGetting rms from: ', rms_path)

#########################################################################################
#	sort rms files
rms_files = []
coefs_files_xrot = []

for rms_file in rms_dir:

	attit_case = rms_file.split('_')[1]

	if rms_file.split('.')[1] == 'dist':
		rms_case_num = int(rms_file.split('_')[0])
		rms_files.append([int(rms_case_num), rms_file])

#	elif distance_file.split('.')[1] == 'coefs':
#		coef_case_num = int(distance_file.split('_')[0])
#		coefs_files_xrot.append([int(coef_case_num), distance_file])


rms_files = sorted(rms_files, key=lambda x: x[0])
# coefs_files_xrot = sorted(coefs_files_xrot, key=lambda x: x[0])

#########################################################################################
#	generate data
data = []
for i, rms_file in enumerate(rms_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		rms_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	rms = np.genfromtxt(os.path.join(rms_path, rms_file[1]), delimiter=',')
	data_line = [elv, azm, rms]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		rms_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = rms_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
rms_column = data_of_elv[:, 2]
data_array[:, column_index] = rms_column

#	set plot data
xrot_data = data_array

###################################################################################################
#	acquiring y-rot case data
###################################################################################################

#	change case designation and repeat
case_desig_yrot = '_'.join(['y-rot', geom_case])

# #	input data
# distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_yrot)
# distance_dir = os.listdir(distance_path)
# print('\nGetting distances from: ', distance_path)

#	input data
rms_path = os.path.join(path_data, 'rms_dir', bulk_dir, reflection_case, case_desig_yrot)
rms_dir = os.listdir(rms_path)
print('\nGetting rmses from: ', rms_path)

#########################################################################################
#	sort rms files
rms_files = []
coefs_files_yrot = []

for rms_file in rms_dir:

	attit_case = rms_file.split('_')[1]

	if rms_file.split('.')[1] == 'dist':
		dist_case_num = int(rms_file.split('_')[0])
		rms_files.append([int(dist_case_num), rms_file])

	elif rms_file.split('.')[1] == 'coefs':
		coef_case_num = int(rms_file.split('_')[0])
		coefs_files_yrot.append([int(coef_case_num), rms_file])

rms_files = sorted(rms_files, key=lambda x: x[0])
coefs_files_yrot = sorted(coefs_files_yrot, key=lambda x: x[0])

#########################################################################################
#	generate data

data = []
for i, rms_file in enumerate(rms_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		rms_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	rms = np.genfromtxt(os.path.join(rms_path, rms_file[1]), delimiter=',')
	data_line = [elv, azm, rms]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		rms_column = data_of_elv[:, 2]

		#	extract rms data into output array column
		data_array[:, column_index] = rms_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
rms_column = data_of_elv[:, 2]
data_array[:, column_index] = rms_column

#	set plot data
yrot_data = data_array

###################################################################################################
#	acquiring z-rot case data
#########################################################################################

#	change case designation and repeat
case_desig_zrot = '_'.join(['z-rot', geom_case])

#	input data
rms_path = os.path.join(path_data, 'rms_dir', bulk_dir, reflection_case, case_desig_zrot)
rms_dir = os.listdir(rms_path)
print('\nGetting rmss from: ', rms_path)

#########################################################################################
#	sort rms files
rms_files = []
coefs_files_zrot = []

for rms_file in rms_dir:

	attit_case = rms_file.split('_')[1]

	if rms_file.split('.')[1] == 'dist':
		dist_case_num = int(rms_file.split('_')[0])
		rms_files.append([int(dist_case_num), rms_file])

	elif rms_file.split('.')[1] == 'coefs':
		coef_case_num = int(rms_file.split('_')[0])
		coefs_files_zrot.append([int(coef_case_num), rms_file])

rms_files = sorted(rms_files, key=lambda x: x[0])
coefs_files_zrot = sorted(coefs_files_zrot, key=lambda x: x[0])

#########################################################################################
#	generate data
data = []
for i, rms_file in enumerate(rms_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		rms_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	rms = np.genfromtxt(os.path.join(rms_path, rms_file[1]), delimiter=',')
	data_line = [elv, azm, rms]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		rms_column = data_of_elv[:, 2]

		#	extract rms data into output array column
		data_array[:, column_index] = rms_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
rms_column = data_of_elv[:, 2]
data_array[:, column_index] = rms_column

#	set plot data
zrot_data = data_array

###################################################################################################
###################################################################################################
#	generating subtraction data
###################################################################################################
###################################################################################################

#	set input directories
rms_path_xrot = os.path.join(path_data, 'rms_dir', bulk_dir, reflection_case, case_desig_xrot)
rms_path_yrot = os.path.join(path_data, 'rms_dir', bulk_dir, reflection_case, case_desig_yrot)
rms_path_zrot = os.path.join(path_data, 'rms_dir', bulk_dir, reflection_case, case_desig_zrot)

#########################################################################################
#	generating x-y data
#########################################################################################

#	set plot data
y_x_data = data_array
y_x_data = abs(np.subtract(yrot_data, xrot_data))

#########################################################################################
#	generating x-z data
#########################################################################################

#	set plot data
z_x_data = data_array
z_x_data = abs(np.subtract(zrot_data, xrot_data))

#########################################################################################
#	generating z-y data
#########################################################################################

#	set plot data
z_y_data = data_array
z_y_data = abs(np.subtract(zrot_data, yrot_data))

###################################################################################################
###################################################################################################
#	graphical outputs
###################################################################################################
###################################################################################################

####################################################
#	set figure-wide parameters

dpi_val = 300
inch2cm = 2.54

small = 1e-15
small = 1e-5

####################################################
#	select colour map
cmap = plt.cm.get_cmap('cividis_r')
cmap.set_under('w')
 
####################################################
#	altitude rings
leo_alt = 400
meo_alt = 2000
geo_alt = 35786

RE = 6371

leo_elv = np.arctan(RE / (RE + leo_alt))
meo_elv = np.arctan(RE / (RE + meo_alt))
geo_elv = np.arctan(RE / (RE + geo_alt))

leo_elv_vals = np.full(elv_vals.shape, leo_elv)
meo_elv_vals = np.full(elv_vals.shape, meo_elv)
geo_elv_vals = np.full(elv_vals.shape, geo_elv)

# #	generate mesh grid for leo
leo_r, leo_th = np.meshgrid(leo_elv_vals, azm_vals)
meo_r, meo_th = np.meshgrid(meo_elv_vals, azm_vals)
geo_r, geo_th = np.meshgrid(geo_elv_vals, azm_vals)

####################################################
#	find azimuth offset to align with grid
nazm_vals = len(th) - 1
offset = (360 / nazm_vals) / 2

###################################################################################################
#	single axis graphs
###################################################################################################

####################################################
#	figure parameters
font_sizex = 6
font_sizey = 6
font_pad = -3
marker_size = 4
marker_edge = 1

hspace_val = 0.35
wspace_val = 0.35

grid_width = 0.1
alt_line_width = 0.1

title_size = 8
title_pad = 10

####################################################
#	set figure wide colour bounds

#	get max vals
attit0_data_raw = attit0_data.max()
xrot_data_raw = xrot_data.max()
yrot_data_raw = yrot_data.max()
zrot_data_raw = zrot_data.max()

#	specific case: remove outermost ring
if geom_case == 'cube' or geom_case == 'cylinder':
	attit0_data_raw = attit0_data[:, 0:-2].max()
	xrot_data_raw = xrot_data[:, 0:-2].max()
	yrot_data_raw = yrot_data[:, 0:-2].max()
	zrot_data_raw = zrot_data[:, 0:-2].max()

#	generate array of max vals
max_vals = np.array([
	[attit0_data_raw, 0],
	[xrot_data_raw, 1],
	[yrot_data_raw, 2],
	[zrot_data_raw, 3]
])

#	sort decending
max_vals = max_vals[max_vals[:, 0].argsort()][::-1]

#	take largest pixel
max_arg = max_vals[0, 1]
zmax = max_vals[0, 0]

# ####################################################
# ####################################################
# ####################################################
# #	specific case: use second highest plot
# if geom_case == 'div1' and reflection_case == 'specular':
# 	#	take second largest to avoid specular glint
# 	max_arg = max_vals[1, 1]
# 	zmax = max_vals[1, 0]
# ####################################################
# ####################################################
# ####################################################

#	extract data array containing largest pixel
if max_arg == 0:
	maxdata = attit0_data
elif max_arg == 1:
	maxdata = xrot_data
elif max_arg == 2:
	maxdata = yrot_data
elif max_arg == 3:
	maxdata = zrot_data

#	specific case: remove outermost ring
if geom_case == 'cube' or geom_case == 'cylinder':
	maxdata = maxdata[:, 0:-2]

#	set colour bounds
n_col = 100

if zmax <= 0.01:
	newmax = np.ceil(maxdata.max() * 10000) / 10000
	step = 0.0001

elif zmax <= 0.1:
	newmax = np.round(maxdata.max(), 3)
	step = 0.001

elif zmax <= 1:
	newmax = np.round(maxdata.max(), 2)
	step = 0.01

elif zmax <= 5:
	newmax = np.round(maxdata.max(), 1)
	step = 0.2

elif zmax <= 10:
	newmax = np.round(maxdata.max(), 1)
	step = 0.1

elif zmax <= 100:
	newmax = np.ceil(maxdata.max())
	step = 1

bounds = np.linspace(0, newmax + step, n_col)

#	generate colour bar bounds
bounds[0] = small
norm = colors.BoundaryNorm(boundaries=bounds, ncolors=256)

####################################################
#	define tick markers

# #	specific case
# if geom_case == 'div5' and reflection_case == 'specular':
# 	tick_marks = np.arange(0.0, newmax + (step * 2), step * 2)
# 
# #	specific case
# elif geom_case == 'cube' and reflection_case == 'specular':
# 	tick_marks = np.arange(0.0, newmax + (step * 5), step * 5)
# else:
# 	#	standard step tickmarks
# 	tick_marks = np.arange(0.0, newmax + (step), step)

####################################################
#	initialise figure

#if reflection_case == 'diffuse':
#	if geom_case == 'div1' or  geom_case == 'div5':
#		fig = plt.figure(figsize=(9.0/inch2cm, 4.5/inch2cm), dpi=dpi_val)
#	else:
#		fig = plt.figure(figsize=(9.0/inch2cm, 9.0/inch2cm), dpi=dpi_val)
#	
#else:
#	fig = plt.figure(figsize=(9.0/inch2cm, 9.0/inch2cm), dpi=dpi_val)


fig = plt.figure(figsize=(4.5/inch2cm, 4.5/inch2cm), dpi=dpi_val)

plt.rc('text', usetex=True)
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

ax2 = plt.subplot(111, projection="polar")

# ###################################################################################################
# #	0-attit
# 
# ####################################################
# #	initialise axes
# 
# if reflection_case == 'diffuse':
# 	if geom_case == 'div1' or  geom_case == 'div5':
# 		ax0 = plt.subplot(121, projection="polar")
# 	else:
# 		ax0 = plt.subplot(221, projection="polar")
# else:
# 	ax0 = plt.subplot(221, projection="polar")
# 
# # ax0 = plt.subplot(221, projection="polar")
# 
# 
# ax0.set_theta_zero_location("N")
# ax0.set_theta_direction(-1)
# 
# ax0.set_title(r'$no-rotation$', pad=title_pad, fontsize=title_size)
# 
# ax0.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
# ax0.tick_params(axis='y', labelsize=font_sizey, pad=0)
# 
# ax0.set_rticks(np.arange(0, 100, 20))
# ax0.set_rlim(0, 100)
# 
# if geom_case == 'cube' or geom_case == 'cylinder':
# 	ax0.set_rlim(0, 90)
# 
# ####################################################
# 
# #	colour the plot
# plt.pcolor(np.radians(th - offset), r, attit0_data, norm=norm, cmap=cmap)
# 
# ####################################################
# #	plot altitude lines
# ax0.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
# ax0.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
# ax0.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)
# 
# ####################################################
# #	plot measurement point
# ax0.plot([0.0], [0.0],
# 	marker='x',
# 	color='g',
# 	markersize=marker_size,
# 	markeredgewidth=marker_edge,
# 	clip_on=False
# )
# 
# ####################################################
# #	finalise figure
# ax0.grid(which='major', axis='both')
# 
# if geom_case == 'cube' or geom_case == 'cylinder':
# 	plt.yticks(np.arange(0, 90, 20))
# 
# ###################################################################################################
# #	x-rot
# 
# ####################################################
# #	initialise axes
# 
# #	diffuse cases
# if reflection_case == 'diffuse':
# 	if geom_case == 'div1' or  geom_case == 'div5':
# 		ax1 = plt.subplot(122, projection="polar")
# 	else:
# 		ax1 = plt.subplot(222, projection="polar")
# 
# #	specular cases
# else:
# 	ax1 = plt.subplot(222, projection="polar")
# 
# # ax1 = plt.subplot(222, projection="polar")
# 
# ax1.set_theta_zero_location("N")
# ax1.set_theta_direction(-1)
# 
# ax1.set_title(r'$x-rotation$', pad=title_pad, fontsize=title_size)
# 
# ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
# ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)
# 
# ax1.set_rticks(np.arange(0, 100, 20))
# ax1.set_rlim(0, 100)
# 
# if geom_case == 'cube' or geom_case == 'cylinder':
# 	ax1.set_rlim(0, 90)
# 
# ####################################################
# #	colour the plot
# plt.pcolormesh(
# 	np.radians(th - offset),
# 	r,
# 	xrot_data,
# 	norm=norm,
# 	cmap=cmap
# )
# 
# ####################################################
# #	plot altitude lines
# ax1.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
# ax1.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
# ax1.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)
# 
# ####################################################
# #	plot rotation vector
# ax1.plot([0.0], [0.0],
# 	marker='+',
# 	color='r',
# 	markersize=marker_size,
# 	markeredgewidth=marker_edge,
# 	clip_on=False
# )
# 
# ####################################################
# #	finalise figure
# ax1.grid(which='major', axis='both')
# 
# if geom_case == 'cube' or geom_case == 'cylinder':
# 	plt.yticks(np.arange(0, 90, 20))

###################################################################################################
#	y-rot

plot_flag = False

if reflection_case == 'diffuse':
	if geom_case == 'div1' or  geom_case == 'div5':
		pass
	else:
		plot_flag = True
else:
	plot_flag = True


if plot_flag is True:

	####################################################
	#	initialise axes
	# ax2 = plt.subplot(223, projection="polar")

	ax2.set_theta_zero_location("N")
	ax2.set_theta_direction(-1)
	
	# ax2.set_title(r'$y-rotation$', pad=title_pad, fontsize=title_size)
	
	ax2.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
	ax2.tick_params(axis='y', labelsize=font_sizey, pad=0)
	
	ax2.set_rticks(np.arange(0, 100, 20))
	ax2.set_rlim(0, 100)
	
	if geom_case == 'cube' or geom_case == 'cylinder':
		ax2.set_rlim(0, 90)
	
	####################################################
	#	colour the plot
	plt.pcolormesh(
		np.radians(th - offset),
		r,
		yrot_data,
		norm=norm,
		cmap=cmap
	)
	
	####################################################
	#	plot altitude lines
	ax2.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
	ax2.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
	ax2.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)

	####################################################
	#	plot phi line
	phi_data = np.linspace(0.0, 80.0, 100)
	psi_data = np.full([100, 1], 0.0)

	ax2.plot(
		np.radians(psi_data), phi_data,
		color='k', linewidth=1.0,
		markevery=[-1], marker=(3, 0, 0), markersize=2,
		clip_on=False
	)
	ax2.text(np.radians(340), 50,
		r'$\phi$',
		color='k',
#		markersize=marker_size,
#		markeredgewidth=marker_edge,
		clip_on=False
	)

	####################################################
	#	plot phi line
	phi_data = np.full([100, 1], 100)
	psi_data = np.linspace(5.0, 35.0, 100)

	ax2.plot(
		np.radians(psi_data), phi_data,
		color='k', linewidth=1.0,
		markevery=[-1], marker=(3, 0, 0), markersize=2,
		clip_on=False
	)
	ax2.text(np.radians(20), 110,
		r'$\psi$',
		color='k',
		clip_on=False
	)

	
#	####################################################
#	#	plot rotation vector
#	ax2.plot([np.pi/2], [90 + offset],
#		marker='+',
#		color='r',
#		markersize=marker_size,
#		markeredgewidth=marker_edge,
#		clip_on=False
#	)
	
	####################################################
	#	finalise figure
	ax2.grid(which='major', axis='both')
	
	if geom_case == 'cube' or geom_case == 'cylinder':
		plt.yticks(np.arange(0, 90, 20))
	
#	###########################################################################################
#	#	z-rot
#	
#	####################################################
#	#	initialise figure
#	ax3 = plt.subplot(224, projection="polar")
#	ax3.set_theta_zero_location("N")
#	ax3.set_theta_direction(-1)
#	
#	ax3.set_title(r'$z-rotation$', pad=title_pad, fontsize=title_size)
#	
#	ax3.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
#	ax3.tick_params(axis='y', labelsize=font_sizey, pad=0)
#	
#	ax3.set_rticks(np.arange(0, 100, 20))
#	ax3.set_rlim(0, 100)
#	
#	if geom_case == 'cube' or geom_case == 'cylinder':
#		ax3.set_rlim(0, 90)
#	
#	####################################################
#	#	colour the plot
#	plt.pcolormesh(
#		np.radians(th - offset),
#		r,
#		zrot_data,
#		norm=norm,
#		cmap=cmap
#	)
#	
#	####################################################
#	#	plot altitude lines
#	ax3.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
#	ax3.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
#	ax3.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)
#	
#	####################################################
#	#	plot rotation vector
#	ax3.plot([0.0], [90 + offset],
#		marker='+',
#		color='r',
#		markersize=marker_size,
#		markeredgewidth=marker_edge,
#		clip_on=False
#	)
#	
#	####################################################
#	#	finalise figure
#	ax3.grid(which='major', axis='both')
#	
#	if geom_case == 'cube' or geom_case == 'cylinder':
#		plt.yticks(np.arange(0, 90, 20))
	
####################################################
#	add colour bar
plt.subplots_adjust(bottom=0.1, right=0.8)
# cax = plt.axes([0.87, 0.1, 0.03, 0.8])
cax = plt.axes([0.9, 0.1, 0.05, 0.8])

tick_marks = [0.0, 0.02, 0.04, 0.06, 0.08, 0.1, 0.12, 0.14, 0.16]

# plt.colorbar(cax=cax, ticks=tick_marks, label=r'$RMSE$')
plt.colorbar(cax=cax, label=r'$RMSE$')

cax.tick_params(axis='y', labelsize=font_sizey, length=3, direction='in')

####################################################
#	saving figure
fig_name = geom_case + '_' + reflection_case + '.eps'
fig_name =  'example_result' + '.eps'

out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	plt.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)


plt.show()
sys.exit()
