###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

import sys
import os
import time
from subprocess import call

import numpy as np

import requests

from bs4 import BeautifulSoup

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Downloader')
print('#####################################################################################')

print('\nInitialising...')

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	define output path
mmt_input_path = 'input_dir/mmt_files/starlink'

#	load existing flag
load_existing = False
load_existing = True

###################################################################################################
#	inputs
###################################################################################################

#	set launch string
launch_string = 'batch9'

#	add to output path
mmt_output_path = os.path.join(mmt_input_path, launch_string)

#	create output directory if it doesnt exist
if not os.path.exists(os.path.join(path_data, mmt_output_path)):
	os.makedirs(os.path.join(path_data, mmt_output_path))

###################################################################################################
#	load starlink lookup table

print('\nLoading Starlink Lookup File...')

#	load parsed starlink lookup table file
starlink_lookup_file = 'starlink_mmt_lookup_table.txt'
starlink_lookup_data = np.loadtxt(os.path.join(path_data, mmt_input_path, starlink_lookup_file))

###################################################################################################
#	load starlink batch TLE list

print('\nLoading Starlink ' + launch_string + ' TLE file...')

batch_file = launch_string + '_full.txt'
with open(os.path.join(path_data, mmt_input_path, batch_file), 'r') as read_file:
	raw_tle_data = read_file.readlines()

print('\nLooping through TLEs...')

objects_notin_mmt = 0

object_index = 1

for line_index in np.arange(0, len(raw_tle_data), 3):

	TLE_line0 = raw_tle_data[line_index].strip()
	TLE_line1 = raw_tle_data[line_index + 1].strip()
	TLE_line2 = raw_tle_data[line_index + 2].strip()

	print('\n	#################################################################################')
	print(
		'	Working on file ({}) of ({})'.format(
			object_index, len(np.arange(0, len(raw_tle_data), 3))
		)
	)

	print('\n	TLE_line0 = ', TLE_line0)
	print('	TLE_line1 = ', TLE_line1)
	print('	TLE_line2 = ', TLE_line2)

	#	extract namestring
	name_string = TLE_line0[2:]

	#	extract norad ID
	norad_id = int(TLE_line2.split(' ')[1])

	#	find lookup ID
	lookup_id = np.where(starlink_lookup_data[:, 0]==norad_id)[0]

	if len(lookup_id) == 0:
		print('\n		Object not found in MMT database...')
		objects_notin_mmt += 1

	else:
		lookup_id = lookup_id[0]

		#	extract mmt ID
		mmt_id = int(starlink_lookup_data[lookup_id, 1])
	
		#	assemble URL
		base_url = 'http://mmt.favor2.info/satellites/'
		url = base_url + str(mmt_id) + '/download'
	
		#	generate output filename
		mmt_output_file = '_'.join([name_string, str(norad_id), 'MMT', str(mmt_id)]) + '.txt'
	
		print('\n	Using Filename: ', mmt_output_file)
	
		if os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file)):
			print('\n	File: ', mmt_output_file, ' already exists.')
	
		else:
			#######################################################################################
			#	perform request
		
			print('\n	Performing Request...')
		
			#	perform request
			r = requests.get(url)
		
			# Retrieve HTTP meta-data
			print('\n	Finished:')
			print('\n		Status code:	', r.status_code)
			print('		File Type:	', r.headers['content-type'])
			print('		Encoding:	', r.encoding)
		
		
			with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'wb') as f:
			    f.write(r.content)
		
			print('\n	Finished Reqest, waiting 5 seconds...')
			time.sleep(5)


	object_index += 1

#	#	add user input requirement for starting next object
#	input('\nWaiting for user input to continue.')


print('\nMMT File Request Complete.')

print('\nNumber of objects not found in MMT database = ', objects_notin_mmt)

###################################################################################################
#	EOF
###################################################################################################
