###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

import sys
import os
from subprocess import call

import requests

import numpy as np

from bs4 import BeautifulSoup


###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Downloader')
print('#####################################################################################')

print('\nInitialising...')

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	define output path
mmt_output_path = 'input_dir/mmt_files/test_folder'

#	define read file
mmt_output_file = 'starlink_lookup_file.html'

#	parsed starlink lookup table file
output_file = 'starlink_mmt_lookup_table.txt'

#	load existing flag
load_existing = False
load_existing = True

###################################################################################################
#	request inputs
###################################################################################################

#	NORAD ID
object_id = ''

#	object name string
name_string = 'Starlink'

#	look for file
if os.path.isfile(os.path.join(path_data, mmt_output_path, output_file)):

	print('\n	Starlink MMT lookup table already exists.')

else:

	###############################################################################################
	#	acquire html data
	###############################################################################################
	
	#	look for file
	if os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file)):
		file_exists = True
	else:
		file_exists = False
	
	#	load file if requested
	if (file_exists is True) and (load_existing is True):
	
		print('\n	Loading HTML data...')
	
		#	load existing file
		with open(os.path.join(path_data, mmt_output_path, mmt_output_file)) as file:
			html_data = file.read()
	
	elif ((file_exists is True) and (load_existing is False)) or \
			((file_exists is False) and (load_existing is True)):
	
		print('\n	Requesting HTML data...')
	
		url = 'http://mmt.favor2.info/satellites'
	
		form_data = {
			'catalogue_id': str(object_id),
			'name': name_string,
			'comments': '',
			'var_nonvariable': 'on',
			'var_variable': 'on',
			'var_periodic': 'on',
			'var_pmin': '',
			'var_pmax': '',
			'action': 'search',
			'type_0': 'on',
			'type_1': 'on',
			'type_2': 'on',
			'type_3': 'on',
			'type_4': 'on',
			'type_5': 'on',
			'type_6': 'on',
			'type_7': 'on',
			'orb_incl_min': '',
			'orb_incl_max': '',
			'orb_period_min': '',
			'orb_period_max': '',
		}
	
		#	perform request
		response = requests.post(url, data=form_data)
		html_data = response.text
	
		print('\nHTML_data = ', html_data)
	
		with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'w') as savefile:
			savefile.write(html_data)
	
	###############################################################################################
	#	parse html data
	###############################################################################################
	
	starlink_lookup_table = []
	
	#	generate doc
	doc = BeautifulSoup(html_data, 'html.parser')
	
	for row_num, link in enumerate(doc.find_all('a')):
	
		row_string = str(link).replace('\n', ' ').strip().split('href="')[1]
	
		if 'class="glyphicon glyphicon-ok-sign"' in row_string:
	
			#	extract norad id
			norad_id = int(row_string.split('</span>')[1].split('</a>')[0])
	
			#	extract mmt id 
			mmt_id = int(row_string.split('/satellites/')[1].split('"> <span')[0])
	
			print('\n\n		NoradID = ', norad_id)
			print('		MMTID = ', mmt_id)
	
			starlink_lookup_table.append([norad_id, mmt_id])
	
	starlink_lookup_table = np.array(starlink_lookup_table)
	
	###############################################################################################
	#	parse html data
	###############################################################################################
	
	print('Saving final lookup table...')
	np.savetxt(os.path.join(path_data, mmt_output_path, output_file), starlink_lookup_table)
	
###################################################################################################
#	EOF
###################################################################################################
