###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

import time
import sys
import os
from subprocess import call

import requests

import numpy as np

from bs4 import BeautifulSoup


###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Downloader')
print('#####################################################################################')

print('\nInitialising...')

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	define output path
mmt_output_path = 'input_dir/mmt_files/rocket_body_MMT_requests'

#	parsed starlink lookup table file
output_file = 'starlink_mmt_lookup_table.txt'

#	load existing flag
load_existing = False
load_existing = True

###################################################################################################
#	request inputs
###################################################################################################

#	NORAD ID
object_id = ''

#	object name string
search_name_string = 'CZ'
search_name_string = 'DELTA'
search_name_string = 'ATLAS'

#	define read file
mmt_output_file = search_name_string + '_html_file.html'

#	look for file
if os.path.isfile(os.path.join(path_data, mmt_output_path, output_file)):

	print('\n	Starlink MMT lookup table already exists.')

else:

	###############################################################################################
	#	acquire html data
	###############################################################################################
	
	#	look for file
	if os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file)):
		file_exists = True
	else:
		file_exists = False
	
	#	load file if requested
	if (file_exists is True) and (load_existing is True):
	
		print('\n	Loading HTML data...')
	
		#	load existing file
		with open(os.path.join(path_data, mmt_output_path, mmt_output_file)) as file:
			html_data = file.read()
	
	elif ((file_exists is True) and (load_existing is False)) or \
			((file_exists is False) and (load_existing is True)):
	
		print('\n	Requesting HTML data...')
	
		url = 'http://mmt.favor2.info/satellites'
	
		form_data = {
			'catalogue_id': str(object_id),
			'name': search_name_string,
			'comments': '',
			'var_nonvariable': 'on',
			'var_variable': 'on',
			'var_periodic': 'on',
			'var_pmin': '',
			'var_pmax': '',
			'action': 'search',
			'type_0': 'on',
			'type_1': 'on',
			'type_2': 'on',
			'type_3': 'on',
			'type_4': 'on',
			'type_5': 'on',
			'type_6': 'on',
			'type_7': 'on',
			'orb_incl_min': '',
			'orb_incl_max': '',
			'orb_period_min': '',
			'orb_period_max': '',
		}
	
		#	perform request
		response = requests.post(url, data=form_data)
		html_data = response.text
	
		print('\nHTML_data = ', html_data)
	
		with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'w') as savefile:
			savefile.write(html_data)

	###############################################################################################
	#	parse html data
	###############################################################################################
	
	lookup_table = []
	
	#	generate doc
	doc = BeautifulSoup(html_data, 'html.parser')

	table_data = doc.find_all('tr')[5:]
	
	for row_num, link in enumerate(table_data):

		print('\n	#############################################################################')
		print('	Working on Object: ', row_num + 1)

		id_string_line = str(link.find_all('td')[0]).replace('\n', ' ')[4:-5].strip()

		#	extract norad id
		norad_id = int(id_string_line.split('</span>')[1].split('</a>')[0])

		#	extract mmt id 
		mmt_id = int(id_string_line.split('/satellites/')[1].split('"> <span')[0])

		print('\n		Norad_ID:		', norad_id)
		print('		MMT_ID:			', mmt_id)

		#	extract international designator
		intl_desig = str(link.find_all('td')[1]).replace('\n', ' ')[4:-5].strip()
		print('		Intel. Desig.:		', intl_desig)

		#	extract name string
		name_str = str(link.find_all('td')[2]).replace('\n', ' ')[4:-5].strip().replace('/', '-')

		if '<span' in name_str:
			name_str = name_str.split('<span')[0].strip()
		print('		Object Name:		', name_str)
		if 'DEB' in name_str:
			print('\n			Is Debris, Skipping...')
			continue

		#	extract type string
		type_str = str(link.find_all('td')[3]).replace('\n', ' ')[4:-5].strip().split('"')[1]
		print('		Object Type:		', type_str)

		#	extract launch date and state
		launch_date_str = str(link.find_all('td')[4]).replace('\n', ' ')[4:-5].strip().split(' / ')[0]
		launch_state_str = str(link.find_all('td')[4]).replace('\n', ' ')[4:-5].strip().split(' / ')[1]
		print('\n		Launched On:		', launch_date_str)
		print('		By:			', launch_state_str)

		#	extract latest track date
		latest_track_str = str(link.find_all('td')[5]).replace('\n', ' ')[4:-5].strip()
		print('\n		First Track:		', latest_track_str)

		#	extract total number of tracks
		ntracks = int(str(link.find_all('td')[6]).replace('\n', ' ')[4:-5].strip())
		print('		NTracks:		', ntracks)

#		#	extract brightness data string
#		brightness_data_str = str(link.find_all('td')[7]).replace('\n', ' ')[4:-5].strip().split('"')[1]
#		print('\n		Brightness Data:	', brightness_data_str)

		#	extract periodicity status string
		period_string_line = str(link.find_all('td')[8]).replace('\n', ' ')[4:-5].strip()

		if period_string_line == '':
			period_str = 'Aperiodic'
		else:
			period_str = period_string_line.split('<span class="text-default">')[1].split('</span>')[0]
		print('\n		Periodicity Status:	', period_str)

#		#	extract radar cross section size
#		rcs = float(str(link.find_all('td')[9]).replace('\n', ' ')[4:-5].strip())
#		print('\n		RCS:			', rcs)

		###########################################################################################
		#	check for periodicity status

		if period_str != 'Aperiodic':

			print('\n		#####################################################################')
			print('		Periodic, downloading data...')

			#	assemble URL
			base_url = 'http://mmt.favor2.info/satellites/'
			url = base_url + str(mmt_id) + '/download'
			url_periods = base_url + str(mmt_id) + '/download/period'
	
			#	generate output filenames
			mmt_output_file = '_'.join([name_str, str(norad_id), 'MMT', str(mmt_id)]) + '.txt'
			mmt_output_file_periods = '_'.join(
				[name_str, str(norad_id), 'MMT', str(mmt_id), 'periods']
			) + '.txt'

			print('\n			MMT File:		', mmt_output_file)
			print('			MMT Periods File:	', mmt_output_file_periods)

			#######################################################################################
			#	check for existing file

			if os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file_periods)) and \
					os.path.isfile(os.path.join(path_data, mmt_output_path, mmt_output_file)):

				print('\n			Data File Already Exists.')
				print('			Periods File Already Exists.')
		
			else:
				###################################################################################
				#	perform request
			
				print('\n			Performing Requests...')
			
				print('\n				Requesting Data File...')
				#	perform request
				r = requests.get(url)
			
				# Retrieve HTTP meta-data
				print('\n				Finished:')
				print('\n					Status code:	', r.status_code)
				print('					File Type:	', r.headers['content-type'])
				print('					Encoding:	', r.encoding)
			
			
				with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'wb') as f:
				    f.write(r.content)

				print('\n			Requesting Periods File...')
				#	perform request
				r = requests.get(url_periods)
			
				# Retrieve HTTP meta-data
				print('\n				Finished:')
				print('\n					Status code:	', r.status_code)
				print('					File Type:	', r.headers['content-type'])
				print('					Encoding:	', r.encoding)
			
			
				with open(os.path.join(path_data, mmt_output_path, mmt_output_file_periods), 'wb') as f:
				    f.write(r.content)


				print('\n			Finished Reqest, waiting 5 seconds...')
				time.sleep(5)

	print('Data Acquired.')
	sys.exit()

	starlink_lookup_table = np.array(starlink_lookup_table)
	
	###############################################################################################
	#	parse html data
	###############################################################################################
	
	print('Saving final lookup table...')
	np.savetxt(os.path.join(path_data, mmt_output_path, output_file), starlink_lookup_table)
	
###################################################################################################
#	EOF
###################################################################################################
	










