###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	read and sort starlink files
###################################################################################################

#	set input directory
read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'rocket_body')
# read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'starlink', 'batch-1')

print('\n[{}] Generating Starlink file name list...'.format(datetime.datetime.now()))

starlink_dir = os.listdir(read_path)
starlink_files = []

#	order by starlink id
for file in starlink_dir:

	print('\n###################################################################')
	#	open and read file
	with open(os.path.join(read_path, file)) as read_file:

		#	get only first line
		mmt_line = read_file.readline()

		#	remove line header
		mmt_line = mmt_line[len('# satellite: '):]

		#	remove launch nation
		mmt_line = mmt_line.split(' / ')[0]

		#	extract norad_id and name string
		norad_id, name_str = mmt_line.split(' ', maxsplit=1)
		name_split = name_str.split(' ')

		print('\n	Norad ID: {},	name_str = {}'.format(norad_id, name_split))
		name_array = []

		for name_part in name_split:
			if '/' not in name_part: 
				name_array.append(name_part)

		print('\n	name_array = ', name_array)


		if len(name_array) == 1:
			name_str = name_array[0]

		else:
			name_str = '-'.join([*name_array])

		print('	name_str = ', name_str)

	#	generate new file name
	new_file = name_str + '_{:d}.txt'.format(int(norad_id))

	if file != new_file:

		print('\n	Renaming file from:\n			', file)
		print('	Renaming file to:\n			', new_file)

		#	rename file
		os.rename(os.path.join(read_path, file), os.path.join(read_path, new_file))

print('\nDone.')
