###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Case File Generation
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

###################################################################################################
#	clear terminal
call(['clear'])

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output
# geom_dir = 'geometry_dir/geom_dir/Delta_IV'
# geom_file = 'delta_iv.stl'
# brdf_file = 'delta_iv.brdf'
# 
# geom_dir = 'geometry_dir/geom_dir/SL-8'
# geom_file = 'sl8.stl'
# brdf_file = 'sl8.brdf'
# 
# geom_dir = 'geometry_dir/geom_dir/sphere'
# geom_file = 'sl8.stl'
# brdf_file = 'sl8.brdf'

#geom_dir = 'geometry_dir'
#geom_file = '05_div5_sphere.stl'
#brdf_file = '05_div5_sphere.brdf'

geom_dir = 'geometry_dir/geom_dir/CZ-2D'
geom_file = 'cz-2d.stl'
brdf_file = 'cz-2d.brdf'

geom_path = os.path.join(path_data, geom_dir)

#############################################
#	BDRF parameters

#	Diffuse albedo
omega_brdf = 0.5

#	Refractive Index
n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))

#	Root-Mean-Square slope
m_brdf = 0.85
m_brdf = 0.25

#	Specular and Diffuse ratio
d_brdf = 0.5
s_brdf = 1.0 - d_brdf

###################################################################################################
#	generating brdf file
print('\nGenerating BDRF File for {}...'.format(geom_file))

#############################################
#	load object mesh
obj_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))

#	extract normals
normals = obj_mesh.normals

#	get number of mesh faces
nfaces = len(normals)
nfaces_unique = len(np.unique(normals, axis=0))

##################################################
#	Generate BDRF array
brdf_data = np.zeros([nfaces, 5])

for i in range(nfaces):
	brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]

##################################################
#	save brdf_file
print('\nSaving File...')
np.savetxt(os.path.join(geom_path, brdf_file), brdf_data, delimiter=',')
