###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	one time run: parse raw EOP files into code input files
#
#
#
#
#
#
###################################################################################################

#	imports

#	standard packages
import os
import sys
import time

#	numpy
import numpy as np

#	scipy
from scipy import interpolate

#	function list
from functions.functions_list import date2jd, jd2date, get_times

###################################################################################################
#	initilisation
###################################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'

#	path to data
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	EOP data
###################################################################################################

eop_data_raw = []
dat_data = []
eop_dir = 'eop_files'
eop_file = 'eop_c04_14_1962-now.csv'
eop_file = 'eopc04_14_IAU2000.62-now.csv'
dat_file = 'tai-utc.dat'

eop_parsed_file = 'eop_parsed.csv'
dat_parsed_file = 'dat_parsed.csv'

##################################################
#	IERS

with open(os.path.join(path_data, eop_dir, eop_file)) as input_file:
	for line in input_file:
		eop_data_raw.append(line.strip().split(';'))

#	arcsecond to radian conversion
as2r = np.pi / (180.0 * 3600.0)

#	assemble eop_data
eop_data_temp = np.array(eop_data_raw[1:])

#	remove header lines
eop_data_temp = eop_data_temp

#	extract values
eop_data = np.concatenate(
	[
		eop_data_temp[:, 0:4],		#	date
		eop_data_temp[:, 5:7],		#	x-pole
		eop_data_temp[:, 7:9],		#	y-pole
		eop_data_temp[:, 10:12],	#	ut1 - utc
		eop_data_temp[:, 12:14],	#	lod
		eop_data_temp[:, 19:21],	#	dX
		eop_data_temp[:, 21:23]		#	dY
	],
	axis=1
).astype(np.float)

#	perform conversion to radians
eop_data[:, 4:6] = np.multiply(eop_data[:, 4:6], as2r)		#	x-pole
eop_data[:, 6:8] = np.multiply(eop_data[:, 6:8], as2r)		#	y-pole
eop_data[:, 12:14] = np.multiply(eop_data[:, 12:14], as2r)	#	dX
eop_data[:, 14:16] = np.multiply(eop_data[:, 14:16], as2r)	#	dY

# save data file
np.savetxt(os.path.join(path_data, eop_dir, eop_parsed_file), eop_data, delimiter=',')

# ###################################################################################################
# #	DAT data
# ###################################################################################################
# 
# # dat_data = []
# eop_dir = 'eop_files'
# dat_file = 'tai-utc.dat'
# dat_parsed_file = 'dat_parsed.csv'
# 
# #	set earliest required year
# first_year = 1980
# 
# #	read in dat data
# with open(os.path.join(path_data, eop_dir, dat_file)) as input_file:
# 	dat_data_in = input_file.read().splitlines()
# 
# #	extract 1980 onwards
# dat_data_in = dat_data_in[22:]
# 
# 
# #	initilise data
# dat_data_out = np.zeros((len(dat_data_in), 5))
# 
# #	extract dat data
# for i, line in enumerate(dat_data_in):
# 
# 	#	parse strings
# 	date_str = line.split('=')[0]
# 	jd_str = line.split('=')[1]
# 	dat_str = line.split('=')[2]
# 
# 	#	date
# 	year, month, _, day = date_str.strip().split(' ')
# 	year = int(year)
# 	day = int(day)
# 
# 	if month == 'JAN':
# 		month = 1
# 	elif month == 'JUL':
# 		month = 7
# 
# 	#	julian date
# 	jdate = float(jd_str.split(' ')[1])
# 
# 	#	dat
# 	dat = float(dat_str.strip().split(' ')[0])
# 
# 	dat_data_out[i, :] = [jdate, year, month, day, dat]
# 
# # save data file
# np.savetxt(os.path.join(path_data, eop_dir, dat_parsed_file), dat_data_out, delimiter=',')
