###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Case generation function for AMOS2018
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	matplotlib
from mpl_toolkits import mplot3d
from matplotlib import pyplot

###################################################################################################
#	case generation function
###################################################################################################

#	number of header data line in case file
data_nlines = 16

def gen_case(
	case_indent, start_time_str, stop_time_str, geom_path, geom_file, attit_mot, attit,
		obs_vec, illum_vec, d_bdrf, s_bdrf, omega_bdrf, n_bdrf, m_bdrf
	):

	##################################################
	#	generate file strings

	#	Attitude 
	attit_str = 'attit = {:1.8}, {:1.8}, {:1.8}'.format(
		attit[0], attit[1], attit[2]
	)

	#	Attitude Motion
	attit_mot_str = 'attit_mot = {:1.8}, {:1.8}, {:1.8}'.format(
		attit_mot[0], attit_mot[1], attit_mot[2]
	)

	#	observation vec
	obs_vec_str = 'obs_vec = {:1.4}, {:1.4}, {:1.4}'.format(
		obs_vec[0], obs_vec[1], obs_vec[2]
	)

	#	illumination vec
	illum_vec_str = 'illum_vec = {:1.4}, {:1.4}, {:1.4}'.format(
		illum_vec[0], illum_vec[1], illum_vec[2]
	)
	##################################################
	#	Load mesh for number of faces
	my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
	geom_faces = len(my_mesh.areas)

	##################################################
	#	Generate BDRF parameters
	bdrf_nlines = geom_faces
	bdrf_para = np.zeros([bdrf_nlines, 5])

	for i in range(geom_faces):
		bdrf_para[i, :] = [m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf]

	print_data = []

	##################################################
	#	Print header information
	print_data.append('# object config file')
	print_data.append('')
	print_data.append('norad_id = 25885')
	print_data.append('site_latgd = 43.64986')
	print_data.append('site_lon = 41.43147')
	print_data.append('site_alt = 2030.00000')
	print_data.append(start_time_str)
	print_data.append(stop_time_str)
	print_data.append(attit_str)
	print_data.append(attit_mot_str)
	print_data.append(obs_vec_str)
	print_data.append(illum_vec_str)
	print_data.append('')
	print_data.append('# Geometry')
	print_data.append('geometry_file=' + geom_file)
	print_data.append('')

	##################################################
	#	print BDRF data
	print_data.append('# BDRF')
	print_data.append('m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf')

	for i, bdrf_line in enumerate(bdrf_para):
		case_line = '{:1.4}, {:1.4}, {:1.4}, {:1.4}, {:1.4}'.format(
			bdrf_line[0],
			bdrf_line[1],
			bdrf_line[2],
			bdrf_line[3],
			bdrf_line[4]
		)

		print_data.append(case_line)

	return print_data

def gen_case_obs(
		case_ident,
		start_time_str, stop_time_str,
		geom_path, geom_file,
		attitude_x, attitude_y, attitude_z,
		rot_axis, rot_angle,
		obs_azm, obs_inc,
		illum_azm, illum_inc,
		d_bdrf, s_bdrf, omega_bdrf, n_bdrf, m_bdrf
	):

	##################################################
	#	generate file strings

	#	Attitude 
	attit_str = 'attit = {:1.14}, {:1.14}, {:1.14}'.format(
		attitude_x, attitude_y, attitude_z
	)

	#	Attitude Motion
	rot_axis_str = 'rot_axis = {:1.14}, {:1.14}, {:1.14}'.format(
		rot_axis[0], rot_axis[1], rot_axis[2]
	)

	#	Attitude Motion
	rot_angle_str = 'rot_angle = {:1.14}'.format(
		rot_angle
	)

	#	observation vec
	obs_str = 'obs_angles = {:1.14}, {:1.14}'.format(
		obs_azm, obs_inc
	)

	#	illumination vec
	illum_str = 'illum_angles = {:1.14}, {:1.14}'.format(
		illum_azm, illum_inc
	)
	##################################################
	#	Load mesh for number of faces
	my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
	geom_faces = len(my_mesh.areas)

	##################################################
	#	Generate BDRF parameters
	bdrf_nlines = geom_faces
	bdrf_para = np.zeros([bdrf_nlines, 5])

	for i in range(geom_faces):
		bdrf_para[i, :] = [m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf]

	print_data = []

	##################################################
	#	Print header information
	print_data.append('# object config file')
	print_data.append('')
	print_data.append('norad_id = 25885')
	print_data.append('site_latgd = 43.64986')
	print_data.append('site_lon = 41.43147')
	print_data.append('site_alt = 2030.00000')
	print_data.append(start_time_str)
	print_data.append(stop_time_str)
	print_data.append(attit_str)
	print_data.append(rot_axis_str)
	print_data.append(rot_angle_str)
	print_data.append(obs_str)
	print_data.append(illum_str)
	print_data.append('')
	print_data.append('# Geometry')
	print_data.append('geometry_file=' + geom_file)
	print_data.append('')

	##################################################
	#	print BDRF data
	print_data.append('# BDRF')
	print_data.append('m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf')

	for i, bdrf_line in enumerate(bdrf_para):
		case_line = '{:1.4}, {:1.4}, {:1.4}, {:1.4}, {:1.4}'.format(
			bdrf_line[0],
			bdrf_line[1],
			bdrf_line[2],
			bdrf_line[3],
			bdrf_line[4]
		)

		print_data.append(case_line)

	return print_data
