###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	import progressbar
import progressbar

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d
from descartes import PolygonPatch

#	shapely
import shapely.geometry as sg
from shapely.ops import cascaded_union


#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors_render2, \
	rot1, rot2, rot3, rot_mat, axis2mat, mat2angles

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	Function: 		glim_slc_shadow_attit
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties. Includes self shadowing
#
###################################################################################################

def glim_slc_shadowing_attit(
		sun_body_data, site_body_data, sun_orb_data, site_orb_data,
		light_curve,
		jd_start, render_time_step,
		body_mesh,
		init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		brdf_data,
		log_level
	):

	slc_shadow_start = time.time()
	preloop_start = time.time()

	if log_level > 0:
		print('#####################################################################################')
		print('Running glim_slc_shadowing script...')

	#	define unit vectors
	x_vec = np.array([1.0, 0.0, 0.0])
	y_vec = np.array([0.0, 1.0, 0.0])
	z_vec = np.array([0.0, 0.0, 1.0])

	if log_level > 0:
		time_loop_log_level = 1
		brightness_loop_log_level = 1

	else:
		time_loop_log_level = 0
		brightness_loop_log_level = 0

	vec_round_level = 4

	sys.exit()

# 	###############################################################################################
# 	#	initialisation
# 	###############################################################################################
# 
# 	###############################################################################################
# 	#	parse attit inputs
# 
# 	if log_level > 0:
# 		print('\n	Parsing Attitude Inputs...')
# 
# 	################################################
# 	#	initial attitude
# 
# 	#	attitude matrix
# 	init_attit_mat = rot_mat(init_attit)
# 
# 	#	transpose for inverse matrix
# 	init_attit_mat_inv = np.transpose(init_attit_mat)
# 
# 	################################################
# 	#	attitude motion in orbital frame
# 
# 	#	rotation_vector in orbital frame
# 	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
# 
# 	#	generate attitude motion matrix in orbital frame
# 	attit_mot_mat_orb = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_orb_unit)
# 
# 	#	extract euler angles
# 	angles1 = np.array(mat2angles(attit_mot_mat_orb)[0:3])
# 	angles2 = np.array(mat2angles(attit_mot_mat_orb)[3:6])
# 	
# 	#	convert negative zeros
# 	for i, ele in enumerate(angles1):
# 		if str(ele)[0] == '-' and ele == 0:
# 			angles1[i] = 0.0
# 	
# 	#	extract attitude motion in the body frame
# 	attit_mot_orb = angles1
# 
# 	################################################
# 	#	attitude motion in body frame
# 
# 	#	generate attitude motion matrix in body frame
# 	attit_mot_mat_body = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_body_unit)
# 
# 	#	extract euler angles
# 	angles1 = np.array(mat2angles(attit_mot_mat_body)[0:3])
# 	angles2 = np.array(mat2angles(attit_mot_mat_body)[3:6])
# 	
# 	#	convert negative zeros
# 	for i, ele in enumerate(angles1):
# 		if str(ele)[0] == '-' and ele == 0:
# 			angles1[i] = 0.0
# 	
# 	#	extract attitude motion in the body frame
# 	attit_mot_body = angles1
# 
# 	###############################################################################################
# 	#	initialise step data
# 
# 	total_rot_vec_vel_angle = np.array([0.0, 0.0, 0.0])
# 
# 	#	generate orbital mesh from the body mesh
# 	orb_mesh_step = mesh.Mesh(body_mesh.data.copy())
# 
# 	#	rotate into orbital frame
# 	orb_mesh_step.rotate_using_matrix(init_attit_mat)
# 	orb_vectors_step = orb_mesh_step.vectors
# 	orb_areas = orb_mesh_step.areas
# 
# 	body_vectors_step = body_mesh.vectors
# 
# 	###############################################################################################
# 	#	initialise loop
# 
# 	#	sensor detection limit [apparent magnitude]
# 	mag_limit = 18
# 
# 	#	total number of facets
# 	tot_facets = len(orb_vectors_step)
# 
# 	#	number of data points set to limit
# 	n_limits = 0
# 	tot_points = len(light_curve)
# 
# 	#	standard distance of 1000km
# 	dist_vec_std = np.array([1000.0, 0.0, 0.0]) * 1000
# 	r_obs_std = np.linalg.norm(dist_vec_std)
# 
# 	slc_log_level = 0
# 	if slc_log_level == 0:
# 		loop_str = light_curve
# 	else:
# 		loop_str = progressbar.progressbar(light_curve)
# 
# 	if log_level > 0:
# 		#	calculate sim time
# 		preloop_init_time = time.time() - preloop_start
# 		print('\n	Pre Loop Init Time 			= {:1.4f} ms.'.format(1000 * preloop_init_time))
# 
# 	###############################################################################################
# 	#	time loop
# 	###############################################################################################
# 
# 	if log_level > 0:
# 		print('\n	Looping through time steps...')
# 
# 	for time_step, row in enumerate(loop_str):
# 
# 		###########################################################################################
# 		#	loop initialisation
# 		###########################################################################################
# 
# 		if time_loop_log_level > 0:
# 			timeloop_start = time.time()
# 			print('\n		#####################################################################')
# 			print('		Working on time step: ', time_step)
# 
# 		###########################################################################################
# 		#	apply orbital attitude motion to geometry if i != 0
# 		if time_step != 0:
# 			orb_mesh_step.rotate_using_matrix(attit_mot_mat_orb)
# 			orb_vectors_step = orb_mesh_step.vectors
# 
# 		###########################################################################################
# 		#	extract step data
# 		
# 		if time_loop_log_level > 0:
# 			print('\n		Initalising Loop Time Step...')
# 
# 		#	time data
# 		time_since = light_curve[time_step, 1]
# 		jdate = jd_start + (time_since / 86400)
# 		light_curve[time_step, 0] = jdate
# 	
# 		#	body position data
# 		sun_body_step = sun_body_data[time_step, :]
# 		site_body_step = site_body_data[time_step, :]
# 
# 		#	orb position data
# 		sun_orb_step = sun_orb_data[time_step, :]
# 		site_orb_step = site_orb_data[time_step, :]
# 
# 		#	using actual distance
# 		dist_vec = np.copy(site_body_step) * 1000
# 		r_obs_act = np.linalg.norm(dist_vec)
# 
# 		#	unit vectors
# 		sun_orb_unit = sun_orb_step / np.linalg.norm(sun_orb_step)
# 		site_orb_unit = site_orb_step / np.linalg.norm(site_orb_step)
# 		sun_body_unit = sun_body_step / np.linalg.norm(sun_body_step)
# 		site_body_unit = site_body_step / np.linalg.norm(site_body_step)
# 
# 		if time_loop_log_level > 0:
# 			#	calculate sim time
# 			loop_init_time = time.time() - timeloop_start
# 			print('\n			Loop Init Time 				= {:1.4f} ms.'.format(
# 				1000 * loop_init_time)
# 			)
# 
# 		###########################################################################################
# 		#	face calculation loop
# 		###########################################################################################
# 	
# 		if time_loop_log_level > 0:
# 			print('\n		Calculating face properties...')
# 			face_calc_start = time.time()
# 
# 		obs_faces = []
# 		illum_faces = []
# 		illum_obs_faces = []
# 		illum_or_obs_faces = []
# 
# 		angles_data = np.zeros([len(orb_vectors_step), 2])
# 
# 		for face_index, vector_step in enumerate(orb_vectors_step):
# 			
# 			#	parse input vectors and calculate facet normal
# 			vect12 = vector_step[1] - vector_step[0]
# 			vect23 = vector_step[2] - vector_step[1]
# 			normal = np.cross(vect12, vect23)
# 			normal_unit = normal / np.linalg.norm(normal)
# 
# 			#	calculate ilumination angle
# 			sun_arccos_of = np.divide(
# 				np.dot(normal, sun_orb_unit),
# 				np.linalg.norm(normal) * np.linalg.norm(sun_orb_unit)
# 			)
# 
# 			if sun_arccos_of > 1.0:
# 				if sun_arccos_of < 1.0 + 1e-6:
# 					sun_arccos_of = 1.0
# 			elif sun_arccos_of < -1.0:
# 				if sun_arccos_of > -1.0 - 1e-6:
# 					sun_arccos_of = -1.0
# 
# 			sun_ang = np.arccos(sun_arccos_of)
# 	
# 			#	calculate observation angle
# 			site_arccos_of = np.divide(
# 				np.dot(normal, site_orb_unit),
# 				np.linalg.norm(normal) * np.linalg.norm(site_orb_unit)
# 			)
# 
# 			if site_arccos_of > 1.0:
# 				if site_arccos_of < 1.0 + 1e-3:
# 					site_arccos_of = 1.0
# 			elif site_arccos_of < -1.0:
# 				if site_arccos_of > -1.0 - 1e-6:
# 					site_arccos_of = -1.0
# 
# 			site_ang = np.arccos(site_arccos_of)
# 
# 			#	add angles to output array
# 			angles_data[face_index, :] = [sun_ang, site_ang]
# 
# 			#	face is observable
# 			if site_ang < (0.5 * np.pi):
# 				obs_faces.append(face_index)
# 	
# 			#	face is illuminated
# 			if sun_ang < (0.5 * np.pi):
# 				illum_faces.append(face_index)
# 
# 			#	face is observable and illuminated
# 			if site_ang < (0.5 * np.pi) and sun_ang < (0.5 * np.pi):
# 				illum_obs_faces.append(face_index)
# 
# 			#	face is observable or illuminated
# 			if site_ang < (0.5 * np.pi) or sun_ang < (0.5 * np.pi):
# 				illum_or_obs_faces.append(face_index)
# 
# 		if time_loop_log_level > 0:
# 
# 			print('\n			Observable Faces:\n				', obs_faces)
# 			print('\n			Illuminated Faces:\n				', illum_faces)
# 			print('\n			Observable and Illuminated Faces:\n				', illum_obs_faces)
# 			print('\n			Observable or Illuminated Faces:\n				', illum_or_obs_faces)
# 
# 			#	calculate sim time
# 			face_calc_time = time.time() - face_calc_start
# 			print('\n			Face Calculation Loop Time 		= {:1.4f} ms.'.format(
# 				1000 * face_calc_time)
# 			)
# 
# 		###############################################################################################
# 		#	initialise projection frames
# 		###############################################################################################
# 
# 		if time_loop_log_level > 0:
# 			print('\n		Initialising Projection Frames...')
# 			proj_start = time.time()
# 
# 		###############################################################################################
# 		#	calculate observation vector projection angles to reference vector
# 
# 		#	reference vector = x_vec
# 		
# 		obs_xy_comp = np.array([site_orb_unit[0], site_orb_unit[1], 0.0])
# 	
# 		#	find azimuthal rotation angle
# 		obs_azm_ang = np.arccos(
# 			np.divide(
# 				np.dot(obs_xy_comp, x_vec),
# 				np.linalg.norm(obs_xy_comp) * np.linalg.norm(x_vec)
# 			)
# 		)
# 		
# 		if site_orb_unit[1] < 0:
# 			obs_azm_ang = -obs_azm_ang
# 		
# 		temp_attit = rot_mat(np.array([0.0, 0.0, obs_azm_ang]))
# 		temp_vec = np.dot(site_orb_unit, temp_attit)
# 		
# 		#	find inclination rotation angle
# 		obs_inc_ang = np.arccos(
# 			np.divide(
# 				np.dot(temp_vec, x_vec),
# 				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
# 			)
# 		)
# 		
# 		#	set obs vector in x direction
# 		if site_orb_unit[2] > 0:
# 			obs_inc_ang = -obs_inc_ang
# 		
# 		#	attitude matrix
# 		obs_proj_attit = rot_mat(np.array([0.0, obs_inc_ang, obs_azm_ang]))
# 		obs_proj_vec = np.round(np.dot(site_orb_unit, obs_proj_attit), vec_round_level)
# 	
# 		###############################################################################################
# 		#	generate observation frame data
# 	
# 		obs_proj_obs_vec = np.round(np.dot(site_orb_unit, obs_proj_attit), vec_round_level)
# 		obs_proj_illum_vec = np.round(np.dot(sun_orb_unit, obs_proj_attit), vec_round_level)
# 	
# 		#	apply obs vec rotation to data
# 	
# 		#	create copy of object mesh
# 		obs_proj_mesh = mesh.Mesh(orb_mesh_step.data.copy())
# 	
# 		#	apply rotation
# 		obs_proj_mesh.rotate_using_matrix(obs_proj_attit)
# 	
# 		obs_proj_vectors = obs_proj_mesh.vectors
# 		obs_proj_normals = obs_proj_mesh.normals
# 		obs_proj_areas = obs_proj_mesh.areas
# 	
# 		###############################################################################################
# 		#	calculate illumination vector projection angles to reference vector
# 	
# 		#	reference vector = x_vec
# 		
# 		illum_xy_comp = np.array([sun_orb_unit[0], sun_orb_unit[1], 0.0])
# 	
# 		#	find azimuthal rotation angle
# 		illum_azm_ang = np.arccos(
# 			np.divide(
# 				np.dot(illum_xy_comp, x_vec),
# 				np.linalg.norm(illum_xy_comp) * np.linalg.norm(x_vec)
# 			)
# 		)
# 		
# 		if sun_orb_unit[1] < 0:
# 			illum_azm_ang = -illum_azm_ang
# 		
# 		temp_attit = rot_mat(np.array([0.0, 0.0, illum_azm_ang]))
# 		temp_vec = np.dot(sun_orb_unit, temp_attit)
# 		
# 		#	find inclination rotation angle
# 		illum_inc_ang = np.arccos(
# 			np.divide(
# 				np.dot(temp_vec, x_vec),
# 				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
# 			)
# 		)
# 		
# 		#	set obs vector in x direction
# 		if sun_orb_unit[2] > 0:
# 			illum_inc_ang = -illum_inc_ang
# 		
# 		#	attitude matrix
# 		illum_proj_attit = rot_mat(np.array([0.0, illum_inc_ang, illum_azm_ang]))
# 		illum_proj_vec = np.round(np.dot(sun_orb_unit, illum_proj_attit), vec_round_level)
# 	
# 		###############################################################################################
# 		#	generate illumination frame data
# 	
# 		illum_proj_obs_vec = np.round(np.dot(site_orb_unit, illum_proj_attit), vec_round_level)
# 		illum_proj_illum_vec = np.round(np.dot(sun_orb_unit, illum_proj_attit), vec_round_level)
# 	
# 		#	apply obs vec rotation to data
# 	
# 		#	create copy of object mesh
# 		illum_proj_mesh = mesh.Mesh(orb_mesh_step.data.copy())
# 	
# 		#	apply rotation
# 		illum_proj_mesh.rotate_using_matrix(illum_proj_attit)
# 	
# 		illum_proj_vectors = illum_proj_mesh.vectors
# 		illum_proj_normals = illum_proj_mesh.normals
# 		illum_proj_areas = illum_proj_mesh.areas
# 	
# 		if time_loop_log_level > 0:
# 			#	calculate sim time
# 			proj_init_time = time.time() - proj_start
# 			print('\n			Projection Init Time 			= {:1.4f} ms.'.format(
# 				1000 * proj_init_time
# 			))
# 
# 		###########################################################################################
# 		#	brightness calculation loop
# 		###########################################################################################
# 
# 		if time_loop_log_level > 0:
# 			print('\n		Brightness Calculation Loop...')
# 			face_loop_start = time.time()
# 
# 		#	round 2d vector data
# 		obs_proj_vectors_2d = np.copy(obs_proj_vectors)
# 		illum_proj_vectors_2d = np.copy(illum_proj_vectors)
# 
# 		###########################################################################################
# 		#	perform loop
# 	
# 		#	angular bisector
# 		H_vec = (sun_body_unit + site_body_unit) / len(sun_body_unit + site_body_unit) 
# 
# 		#	observable facet array
# 		face_obs_app = []
# 		face_obs_std = []
# 		facet_num = 1
# 
# 		step_brightness_data = np.zeros([len(illum_obs_faces), 2])
# 		running_tot_time = 0
# 
# 		for loop_itt, face_index in enumerate(illum_obs_faces):
# 
# 			if brightness_loop_log_level > 0:
# 				faceloop_init_start = time.time()
# 				faceloop_step_start = time.time()
# 
# 				print('\n			############################################################')
# 				print('			Working on Illuminated Observable Facet: ', face_index)
# 				print('\n			Initialising Face Data')
# 
# 			#	get face area
# 			face_area = orb_areas[face_index]
# 
# 			#	parse brdf data
# 			m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf = brdf_data[face_index, :]
# 	
# 			#	get vector data
# 			obs_proj_vector = obs_proj_vectors_2d[face_index, :]
# 			illum_proj_vector = illum_proj_vectors_2d[face_index, :]
# 			body_vector = np.round(body_vectors_step[face_index, :], vec_round_level)
# 		
# 			#	calculate max x-level
# 			obs_proj_x_level = obs_proj_vector[:, 0].max()
# 			illum_proj_x_level = illum_proj_vector[:, 0].max()
# 			
# 			#	generate polygons
# 			obs_proj_step_poly = sg.Polygon([
# 				[obs_proj_vector[0, 1], obs_proj_vector[0, 2]],
# 				[obs_proj_vector[1, 1], obs_proj_vector[1, 2]],
# 				[obs_proj_vector[2, 1], obs_proj_vector[2, 2]]
# 			])
# 
# 			illum_proj_step_poly = sg.Polygon([
# 				[illum_proj_vector[0, 1], illum_proj_vector[0, 2]],
# 				[illum_proj_vector[1, 1], illum_proj_vector[1, 2]],
# 				[illum_proj_vector[2, 1], illum_proj_vector[2, 2]]
# 			])
# 	
# 			#	calculate normal unit vector
# 			vect12 = body_vector[1] - body_vector[0]
# 			vect23 = body_vector[2] - body_vector[1]
# 			normal = np.cross(vect12, vect23)
# 			normal = normal / np.linalg.norm(normal)
# 
# 			if brightness_loop_log_level > 0:
# 
# 				#	calculate sim time
# 				faceloop_init_time = time.time() - faceloop_init_start
# 				print('\n				Brightness Loop Initialisation 		= {:1.4f} ms.'.format(
# 					1000 * faceloop_init_time
# 				))
# 
# 			#######################################################################################
# 			#	shadowing facets
# 	
# 			if brightness_loop_log_level > 0:
# 				shad_calc_start = time.time()
# 
# 				print('\n			Calculating Shadowing from [{}] faces...'.format(
# 					len(illum_or_obs_faces))
# 				)
# 
# 			#	initialise
# 			obs_shadowing_first = True
# 			illum_shadowing_first = True
# 
# 			obs_shadowing_polys = 0
# 			illum_shadowing_polys = 0
# 
# 			#	looking through to find shadowing facets
# 			for temp_index in illum_or_obs_faces:
# 				if temp_index != face_index:
# 
# 					if face_index == 15 and time_step == 16:
# 						print('\nWorking on Shadowing Face: ', temp_index)
# 
# 					#	parse vector data
# 					obs_proj_vector_temp = obs_proj_vectors_2d[temp_index, :]
# 					illum_proj_vector_temp = illum_proj_vectors_2d[temp_index, :]
# 
# 					#	parse angles data
# 					angles_line = angles_data[temp_index, :]
# 					sun_ang_temp, site_ang_temp = angles_line
# 	
# 					#	generate polygon
# 					obs_proj_poly_temp = sg.Polygon([
# 						[obs_proj_vector_temp[0, 1], obs_proj_vector_temp[0, 2]],
# 						[obs_proj_vector_temp[1, 1], obs_proj_vector_temp[1, 2]],
# 						[obs_proj_vector_temp[2, 1], obs_proj_vector_temp[2, 2]]
# 					])
# 
# 					#	generate polygon
# 					illum_proj_poly_temp = sg.Polygon([
# 						[illum_proj_vector_temp[0, 1], illum_proj_vector_temp[0, 2]],
# 						[illum_proj_vector_temp[1, 1], illum_proj_vector_temp[1, 2]],
# 						[illum_proj_vector_temp[2, 1], illum_proj_vector_temp[2, 2]]
# 					])
# 
# 					###############################################################################
# 					#	observation projection
# 			
# 					#	calculate x level
# 					obs_proj_x_level_temp = obs_proj_vector_temp[:, 0].max()
# 
# 					#	add to foreground or background polygon arrays
# 					if obs_proj_x_level_temp > obs_proj_x_level:
# 
# 						if obs_shadowing_first is True:
# 
# 							#	initialise on first shadowing facet
# 							obs_run_tot_poly = cascaded_union(obs_proj_poly_temp)
# 							obs_shadowing_first = False
# 							obs_shadowing_polys += 1
# 
# 						elif obs_shadowing_first is False:
# 
# 							#	check if polygon is fully contained
# 							contains_flag = obs_run_tot_poly.contains(obs_proj_poly_temp)
# 
# 							#	if not, add to output
# 							if contains_flag is False:
# 
# 								obs_run_tot_poly = cascaded_union([
# 									obs_run_tot_poly,
# 									obs_proj_poly_temp
# 								])
# 
# 								#	check for existence of holes
# 								if list(obs_run_tot_poly.interiors) != []:
# 									print('\nHole found, regenerating polygon with outer ring...')
# 
# 									#	regenerate polygon using only outer ring
# 									obs_run_tot_poly = sg.Polygon(obs_run_tot_poly.exterior.coords)
# 
# 								#	itterate number of observation shadowing facets
# 								obs_shadowing_polys += 1
# 
# 					###################################################################################
# 					#	illumination projection
# 
# 					#	calculate x level
# 					illum_proj_x_level_temp = illum_proj_vector_temp[:, 0].max()
# 
# 					#	add to foreground or background polygon arrays
# 					if illum_proj_x_level_temp > illum_proj_x_level:
# 
# 						if illum_shadowing_first is True:
# 
# 							#	initialise on first shadowing facet
# 							illum_run_tot_poly = cascaded_union(illum_proj_poly_temp)
# 							illum_shadowing_first = False
# 							illum_shadowing_polys += 1
# 
# 						elif illum_shadowing_first is False:
# 
# 							#	check if polygon is fully contained
# 							contains_flag = illum_run_tot_poly.contains(illum_proj_poly_temp)
# 							
# 							#	if not, add to output
# 							if contains_flag is False:
# 								illum_run_tot_poly = cascaded_union([
# 									illum_run_tot_poly,
# 									illum_proj_poly_temp
# 								])
# 
# 								#	itterate number of illumination shadowing facets
# 								illum_shadowing_polys += 1
# 
# 			if brightness_loop_log_level > 0:
# 				#	calculate sim time
# 				shad_calc_time = time.time() - shad_calc_start
# 				print('\n				Shadow Calculation Time 		= {:1.4f} ms.'.format(
# 					1000 * shad_calc_time)
# 				)
# 
# 			#######################################################################################
# 			#	calculate visibility fractions
# 			#######################################################################################
# 
# 			if brightness_loop_log_level > 0:
# 				print('\n			Calculating Visibility Fractions...')
# 				frac_calc_start = time.time()
# 
# 			#######################################################################################
# 			#	observation visibility fraction
# 	
# 			if face_index == 15 and time_step == 16:
# 				print('Exited')
# 				sys.exit()
# 
# 			if obs_shadowing_polys == 0:
# 				obs_visible_perc = 100.0
# 			else:
# 
# 				#	find intersecting area
# 				obs_proj_intersection = obs_proj_step_poly.intersection(obs_run_tot_poly)
# 
# 				obs_visible_perc = 100 * (
# 					1.0 - (obs_proj_intersection.area / obs_proj_step_poly.area)
# 				)
# 
# 			#######################################################################################
# 			#	illumination visibility fraction
# 
# 			if illum_shadowing_polys == 0:
# 				illum_visible_perc = 100.0
# 			else:
# 		
# 				#	find intersecting area
# 				illum_proj_intersection = illum_proj_step_poly.intersection(illum_run_tot_poly)
# 	
# 				illum_visible_perc = 100 * (
# 					1.0 - (illum_proj_intersection.area / illum_proj_step_poly.area)
# 				)
# 
# 			if brightness_loop_log_level > 0:
# 				print('\n						Visibility Percentage = ', obs_visible_perc)
# 				print('						Illumination Percentage = ', illum_visible_perc)
# 	
# 				#	calculate sim time
# 				frac_calc_time = time.time() - frac_calc_start
# 				print('\n				Fraction Calculation Time 		= {:1.4f} ms.'.format(
# 					1000 * frac_calc_time)
# 				)
# 
# 			#######################################################################################
# 			#	finalise brightness calculation
# 			#######################################################################################
# 
# 			if brightness_loop_log_level > 0:
# 				print('\n			Performing BRDF Calculation...')
# 				brightness_start = time.time()
# 
# 			#	extract projected area and modify by visibility percentages
# 			proj_area = face_area * (obs_visible_perc / 100) * (illum_visible_perc / 100)
# 
# 			#	apply cook-torrance BDRF
# 			facet_brightness = cook_torrance(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
# 				H_vec, normal, site_body_unit, sun_body_unit, proj_area)
# 
# 			facet_brightness_app = facet_brightness / (4 * np.pi * r_obs_act**2)
# 			facet_brightness_std = facet_brightness / (4 * np.pi * r_obs_std**2)
# 	
# 			#	add facet brightness to output array
# 			face_obs_app.append(facet_brightness_app)
# 			face_obs_std.append(facet_brightness_std)
# 
# 			#	update facet number
# 			facet_num += 1
# 
# 			#######################################################################################
# 			#	end of brightness calculation of face
# 	
# 			if brightness_loop_log_level > 0:
# 				#	calculate sim time
# 				brightness_time = time.time() - brightness_start
# 				print('\n				BRDF Calculation Time 			= {:1.4f} ms.'.format(
# 					1000 * brightness_time)
# 				)
# 
# 				faceloop_step_time = time.time() - faceloop_step_start
# 				print('\n			Face Loop Step Time: {:1.4f} ms.'.format(
# 					1000 * faceloop_step_time)
# 				)
# 
# 		###########################################################################################
# 		#	end of face loop
# 		if brightness_loop_log_level > 0:
# 			print('\n			############################################################')
# 			print('			############################################################')
# 
# 		if time_loop_log_level > 0:
# 			#	calculate sim time
# 			face_loop_time = time.time() - face_loop_start
# 			print('\n			Brightness Calculation Loop Time	= {:1.4f} ms.'.format(
# 				1000 * face_loop_time)
# 			)
# 
# 		################################################################################################
# 		#	finalise data
# 		################################################################################################
# 
# 		if time_loop_log_level > 0:
# 			print('\n		Finalising Data of Time Step...')
# 			final_start = time.time()
# 
# 		################################################################################################
# 		#	apparent data
# 
# 		#	convert to numpy array
# 		face_obs_app = np.array(face_obs_app)
# 
# 		if face_obs_app.size != 0:
# 			brightness_step_app = np.sum(face_obs_app)
# 
# 			#	apparent magnitude calculation
# 			magnitude_app = -26.74 - 2.5 * np.log10(brightness_step_app)
# 
# 		#	no observable facets
# 		elif face_obs_app.size == 0:
# 
# 			#	set to limiting apparent magnitude
# 			magnitude_app = 18.0
# 			n_limits += 1 
# 
# 		################################################################################################
# 		#	standard data
# 
# 		#	convert to numpy array
# 		face_obs_std = np.array(face_obs_std)
# 
# 		if face_obs_std.size != 0:
# 			brightness_step_std = np.sum(face_obs_std)
# 
# 			#	standard magnitude calculation
# 			magnitude_std = -26.74 - 2.5 * np.log10(brightness_step_std)
# 
# 		#	no observable facets
# 		elif face_obs_std.size == 0:
# 
# 			#	set to limiting standard magnitude
# 			magnitude_std = 18.0
# 			n_limits += 1 
# 
# 		################################################################################################
# 		#	populate return array
# 		light_curve[time_step, 2] = magnitude_app
# 		light_curve[time_step, 3] = magnitude_std
# 
# 		if time_loop_log_level > 0:
# 			print('\n			Brightness of Time Step:')
# 			print('				Apparent: ', magnitude_app)
# 			print('				Standard: ', magnitude_std)
# 
# 			#	calculate sim time
# 			final_time = time.time() - final_start
# 			print('\n			Finalisation Time 	= {:1.4f} ms.'.format(1000 * final_time))
# 
# 			#	calculate sim time
# 			timeloop_time = time.time() - timeloop_start
# 			
# 			print('\n		Time Step Complete.')
# 			print('			- Time Step time taken = {:1.4f} ms.'.format(1000 * timeloop_time))
# 
# 	if n_limits != 0:
# 		print('\n	{} of {} Data points set to limiting magnitude.'.format(
# 			int(n_limits),
# 			int(tot_points)
# 		))
# 
# 	###############################################################################################
# 	#	calculate run time
# 	###############################################################################################
# 	
# 	if log_level > 0:
# 
# 		#	calculate sim time
# 		slc_shadow_end = time.time()
# 		slc_shadow_time = np.round((slc_shadow_end - slc_shadow_start), 3)
# 
# 		print('\n	SLC Shadowing Finished.')
# 		print('\n	- Total Time taken = {} s.'.format(slc_shadow_time))
# 
# 	return light_curve

###################################################################################################
#	Function: 		glim_slc_shadow
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties. Includes self shadowing
#
###################################################################################################

def glim_slc_shadowing(
		sun_body_data, site_body_data, sun_orb_data, site_orb_data,
		light_curve,
		jd_start, render_time_step,
		body_mesh,
		init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		brdf_data,
		log_level
	):

	slc_shadow_start = time.time()
	preloop_start = time.time()

	if log_level > 0:
		print('#####################################################################################')
		print('Running glim_slc_shadowing script...')

	#	define unit vectors
	x_vec = np.array([1.0, 0.0, 0.0])
	y_vec = np.array([0.0, 1.0, 0.0])
	z_vec = np.array([0.0, 0.0, 1.0])

	if log_level > 0:
		time_loop_log_level = 1
		brightness_loop_log_level = 1

	else:
		time_loop_log_level = 0
		brightness_loop_log_level = 0

	vec_round_level = 4

	###############################################################################################
	#	initialisation
	###############################################################################################

	###############################################################################################
	#	parse attit inputs

	if log_level > 0:
		print('\n	Parsing Attitude Inputs...')

	################################################
	#	initial attitude

	#	attitude matrix
	init_attit_mat = rot_mat(init_attit)

	#	transpose for inverse matrix
	init_attit_mat_inv = np.transpose(init_attit_mat)

	################################################
	#	attitude motion in orbital frame

	#	rotation_vector in orbital frame
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

	#	generate attitude motion matrix in orbital frame
	attit_mot_mat_orb = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_orb_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_orb)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_orb)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_orb = angles1

	################################################
	#	attitude motion in body frame

	#	generate attitude motion matrix in body frame
	attit_mot_mat_body = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_body_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1

	###############################################################################################
	#	initialise step data

	total_rot_vec_vel_angle = np.array([0.0, 0.0, 0.0])

	#	generate orbital mesh from the body mesh
	orb_mesh_step = mesh.Mesh(body_mesh.data.copy())

	#	rotate into orbital frame
	orb_mesh_step.rotate_using_matrix(init_attit_mat)
	orb_vectors_step = orb_mesh_step.vectors
	orb_areas = orb_mesh_step.areas

	body_vectors_step = body_mesh.vectors

	###############################################################################################
	#	initialise loop

	#	sensor detection limit [apparent magnitude]
	mag_limit = 18

	#	total number of facets
	tot_facets = len(orb_vectors_step)

	#	number of data points set to limit
	n_limits = 0
	tot_points = len(light_curve)

	#	standard distance of 1000km
	dist_vec_std = np.array([1000.0, 0.0, 0.0]) * 1000
	r_obs_std = np.linalg.norm(dist_vec_std)

	slc_log_level = 0
	if slc_log_level == 0:
		loop_str = light_curve
	else:
		loop_str = progressbar.progressbar(light_curve)

	if log_level > 0:
		#	calculate sim time
		preloop_init_time = time.time() - preloop_start
		print('\n	Pre Loop Init Time 			= {:1.4f} ms.'.format(1000 * preloop_init_time))

	###############################################################################################
	#	time loop
	###############################################################################################

	if log_level > 0:
		print('\n	Looping through time steps...')

	for time_step, row in enumerate(loop_str):

		###########################################################################################
		#	loop initialisation
		###########################################################################################

		if time_loop_log_level > 0:
			timeloop_start = time.time()
			print('\n		#####################################################################')
			print('		Working on time step: ', time_step)

		###########################################################################################
		#	apply orbital attitude motion to geometry if i != 0
		if time_step != 0:
			orb_mesh_step.rotate_using_matrix(attit_mot_mat_orb)
			orb_vectors_step = orb_mesh_step.vectors

		###########################################################################################
		#	extract step data
		
		if time_loop_log_level > 0:
			print('\n		Initalising Loop Time Step...')

		#	time data
		time_since = light_curve[time_step, 1]
		jdate = jd_start + (time_since / 86400)
		light_curve[time_step, 0] = jdate
	
		#	body position data
		sun_body_step = sun_body_data[time_step, :]
		site_body_step = site_body_data[time_step, :]

		#	orb position data
		sun_orb_step = sun_orb_data[time_step, :]
		site_orb_step = site_orb_data[time_step, :]

		#	using actual distance
		dist_vec = np.copy(site_body_step) * 1000
		r_obs_act = np.linalg.norm(dist_vec)

		#	unit vectors
		sun_orb_unit = sun_orb_step / np.linalg.norm(sun_orb_step)
		site_orb_unit = site_orb_step / np.linalg.norm(site_orb_step)
		sun_body_unit = sun_body_step / np.linalg.norm(sun_body_step)
		site_body_unit = site_body_step / np.linalg.norm(site_body_step)

		if time_loop_log_level > 0:
			#	calculate sim time
			loop_init_time = time.time() - timeloop_start
			print('\n			Loop Init Time 				= {:1.4f} ms.'.format(
				1000 * loop_init_time)
			)

		###########################################################################################
		#	face calculation loop
		###########################################################################################
	
		if time_loop_log_level > 0:
			print('\n		Calculating face properties...')
			face_calc_start = time.time()

		obs_faces = []
		illum_faces = []
		illum_obs_faces = []
		illum_or_obs_faces = []

		angles_data = np.zeros([len(orb_vectors_step), 2])

		for face_index, vector_step in enumerate(orb_vectors_step):
			
			#	parse input vectors and calculate facet normal
			vect12 = vector_step[1] - vector_step[0]
			vect23 = vector_step[2] - vector_step[1]
			normal = np.cross(vect12, vect23)
			normal_unit = normal / np.linalg.norm(normal)

			#	calculate ilumination angle
			sun_arccos_of = np.divide(
				np.dot(normal, sun_orb_unit),
				np.linalg.norm(normal) * np.linalg.norm(sun_orb_unit)
			)

			if sun_arccos_of > 1.0:
				if sun_arccos_of < 1.0 + 1e-6:
					sun_arccos_of = 1.0
			elif sun_arccos_of < -1.0:
				if sun_arccos_of > -1.0 - 1e-6:
					sun_arccos_of = -1.0

			sun_ang = np.arccos(sun_arccos_of)
	
			#	calculate observation angle
			site_arccos_of = np.divide(
				np.dot(normal, site_orb_unit),
				np.linalg.norm(normal) * np.linalg.norm(site_orb_unit)
			)

			if site_arccos_of > 1.0:
				if site_arccos_of < 1.0 + 1e-3:
					site_arccos_of = 1.0
			elif site_arccos_of < -1.0:
				if site_arccos_of > -1.0 - 1e-6:
					site_arccos_of = -1.0

			site_ang = np.arccos(site_arccos_of)

			#	add angles to output array
			angles_data[face_index, :] = [sun_ang, site_ang]

			#	face is observable
			if site_ang < (0.5 * np.pi):
				obs_faces.append(face_index)
	
			#	face is illuminated
			if sun_ang < (0.5 * np.pi):
				illum_faces.append(face_index)

			#	face is observable and illuminated
			if site_ang < (0.5 * np.pi) and sun_ang < (0.5 * np.pi):
				illum_obs_faces.append(face_index)

			#	face is observable or illuminated
			if site_ang < (0.5 * np.pi) or sun_ang < (0.5 * np.pi):
				illum_or_obs_faces.append(face_index)

		if time_loop_log_level > 0:

			print('\n			Observable Faces:\n				', obs_faces)
			print('\n			Illuminated Faces:\n				', illum_faces)
			print('\n			Observable and Illuminated Faces:\n				', illum_obs_faces)
			print('\n			Observable or Illuminated Faces:\n				', illum_or_obs_faces)

			#	calculate sim time
			face_calc_time = time.time() - face_calc_start
			print('\n			Face Calculation Loop Time 		= {:1.4f} ms.'.format(
				1000 * face_calc_time)
			)

		###############################################################################################
		#	initialise projection frames
		###############################################################################################

		if time_loop_log_level > 0:
			print('\n		Initialising Projection Frames...')
			proj_start = time.time()

		###############################################################################################
		#	calculate observation vector projection angles to reference vector

		#	reference vector = x_vec
		
		obs_xy_comp = np.array([site_orb_unit[0], site_orb_unit[1], 0.0])
	
		#	find azimuthal rotation angle
		obs_azm_ang = np.arccos(
			np.divide(
				np.dot(obs_xy_comp, x_vec),
				np.linalg.norm(obs_xy_comp) * np.linalg.norm(x_vec)
			)
		)
		
		if site_orb_unit[1] < 0:
			obs_azm_ang = -obs_azm_ang
		
		temp_attit = rot_mat(np.array([0.0, 0.0, obs_azm_ang]))
		temp_vec = np.dot(site_orb_unit, temp_attit)
		
		#	find inclination rotation angle
		obs_inc_ang = np.arccos(
			np.divide(
				np.dot(temp_vec, x_vec),
				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
			)
		)
		
		#	set obs vector in x direction
		if site_orb_unit[2] > 0:
			obs_inc_ang = -obs_inc_ang
		
		#	attitude matrix
		obs_proj_attit = rot_mat(np.array([0.0, obs_inc_ang, obs_azm_ang]))
		obs_proj_vec = np.round(np.dot(site_orb_unit, obs_proj_attit), vec_round_level)

		###############################################################################################
		#	generate observation frame data
	
		obs_proj_obs_vec = np.round(np.dot(site_orb_unit, obs_proj_attit), vec_round_level)
		obs_proj_illum_vec = np.round(np.dot(sun_orb_unit, obs_proj_attit), vec_round_level)
	
		#	apply obs vec rotation to data
	
		#	create copy of object mesh
		obs_proj_mesh = mesh.Mesh(orb_mesh_step.data.copy())
	
		#	apply rotation
		obs_proj_mesh.rotate_using_matrix(obs_proj_attit)
	
		obs_proj_vectors = obs_proj_mesh.vectors
		obs_proj_normals = obs_proj_mesh.normals
		obs_proj_areas = obs_proj_mesh.areas
	
		###############################################################################################
		#	calculate illumination vector projection angles to reference vector
	
		#	reference vector = x_vec
		
		illum_xy_comp = np.array([sun_orb_unit[0], sun_orb_unit[1], 0.0])
	
		#	find azimuthal rotation angle
		illum_azm_ang = np.arccos(
			np.divide(
				np.dot(illum_xy_comp, x_vec),
				np.linalg.norm(illum_xy_comp) * np.linalg.norm(x_vec)
			)
		)
		
		if sun_orb_unit[1] < 0:
			illum_azm_ang = -illum_azm_ang
		
		temp_attit = rot_mat(np.array([0.0, 0.0, illum_azm_ang]))
		temp_vec = np.dot(sun_orb_unit, temp_attit)
		
		#	find inclination rotation angle
		illum_inc_ang = np.arccos(
			np.divide(
				np.dot(temp_vec, x_vec),
				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
			)
		)
		
		#	set obs vector in x direction
		if sun_orb_unit[2] > 0:
			illum_inc_ang = -illum_inc_ang
		
		#	attitude matrix
		illum_proj_attit = rot_mat(np.array([0.0, illum_inc_ang, illum_azm_ang]))
		illum_proj_vec = np.round(np.dot(sun_orb_unit, illum_proj_attit), vec_round_level)
	
		###############################################################################################
		#	generate illumination frame data
	
		illum_proj_obs_vec = np.round(np.dot(site_orb_unit, illum_proj_attit), vec_round_level)
		illum_proj_illum_vec = np.round(np.dot(sun_orb_unit, illum_proj_attit), vec_round_level)
	
		#	apply obs vec rotation to data
	
		#	create copy of object mesh
		illum_proj_mesh = mesh.Mesh(orb_mesh_step.data.copy())
	
		#	apply rotation
		illum_proj_mesh.rotate_using_matrix(illum_proj_attit)
	
		illum_proj_vectors = illum_proj_mesh.vectors
		illum_proj_normals = illum_proj_mesh.normals
		illum_proj_areas = illum_proj_mesh.areas
	
		if time_loop_log_level > 0:
			#	calculate sim time
			proj_init_time = time.time() - proj_start
			print('\n			Projection Init Time 			= {:1.4f} ms.'.format(
				1000 * proj_init_time
			))

		###########################################################################################
		#	brightness calculation loop
		###########################################################################################

		if time_loop_log_level > 0:
			print('\n		Brightness Calculation Loop...')
			face_loop_start = time.time()

		#	round 2d vector data
		obs_proj_vectors_2d = np.copy(obs_proj_vectors)
		illum_proj_vectors_2d = np.copy(illum_proj_vectors)

		###########################################################################################
		#	perform loop
	
		#	angular bisector
		H_vec = (sun_body_unit + site_body_unit) / len(sun_body_unit + site_body_unit) 

		#	observable facet array
		face_obs_app = []
		face_obs_std = []
		facet_num = 1

		step_brightness_data = np.zeros([len(illum_obs_faces), 2])
		running_tot_time = 0

		for loop_itt, face_index in enumerate(illum_obs_faces):

			if brightness_loop_log_level > 0:
				faceloop_init_start = time.time()
				faceloop_step_start = time.time()

				print('\n			############################################################')
				print('			Working on Illuminated Observable Facet: ', face_index)
				print('\n			Initialising Face Data')

			#	get face area
			face_area = orb_areas[face_index]

			#	parse brdf data
			m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf = brdf_data[face_index, :]
	
			#	get vector data
			obs_proj_vector = obs_proj_vectors_2d[face_index, :]
			illum_proj_vector = illum_proj_vectors_2d[face_index, :]
			body_vector = np.round(body_vectors_step[face_index, :], vec_round_level)
		
			#	calculate max x-level
			obs_proj_x_level = obs_proj_vector[:, 0].max()
			illum_proj_x_level = illum_proj_vector[:, 0].max()
			
			#	generate polygons
			obs_proj_step_poly = sg.Polygon([
				[obs_proj_vector[0, 1], obs_proj_vector[0, 2]],
				[obs_proj_vector[1, 1], obs_proj_vector[1, 2]],
				[obs_proj_vector[2, 1], obs_proj_vector[2, 2]]
			])

			illum_proj_step_poly = sg.Polygon([
				[illum_proj_vector[0, 1], illum_proj_vector[0, 2]],
				[illum_proj_vector[1, 1], illum_proj_vector[1, 2]],
				[illum_proj_vector[2, 1], illum_proj_vector[2, 2]]
			])
	
			#	calculate normal unit vector
			vect12 = body_vector[1] - body_vector[0]
			vect23 = body_vector[2] - body_vector[1]
			normal = np.cross(vect12, vect23)
			normal = normal / np.linalg.norm(normal)

			if brightness_loop_log_level > 0:

				#	calculate sim time
				faceloop_init_time = time.time() - faceloop_init_start
				print('\n				Brightness Loop Initialisation 		= {:1.4f} ms.'.format(
					1000 * faceloop_init_time
				))

			#######################################################################################
			#	shadowing facets
	
			if brightness_loop_log_level > 0:
				shad_calc_start = time.time()

				print('\n			Calculating Shadowing from [{}] faces...'.format(
					len(illum_or_obs_faces))
				)
			#	initialise
			obs_shadowing_first = True
			illum_shadowing_first = True

			obs_shadowing_polys = 0
			illum_shadowing_polys = 0

			#	looking through to find shadowing facets
			for temp_index in illum_or_obs_faces:
				if temp_index != face_index:

					#	parse vector data
					obs_proj_vector_temp = obs_proj_vectors_2d[temp_index, :]
					illum_proj_vector_temp = illum_proj_vectors_2d[temp_index, :]

					#	parse angles data
					angles_line = angles_data[temp_index, :]
					sun_ang_temp, site_ang_temp = angles_line
	
					#	generate polygon
					obs_proj_poly_temp = sg.Polygon([
						[obs_proj_vector_temp[0, 1], obs_proj_vector_temp[0, 2]],
						[obs_proj_vector_temp[1, 1], obs_proj_vector_temp[1, 2]],
						[obs_proj_vector_temp[2, 1], obs_proj_vector_temp[2, 2]]
					])

					#	generate polygon
					illum_proj_poly_temp = sg.Polygon([
						[illum_proj_vector_temp[0, 1], illum_proj_vector_temp[0, 2]],
						[illum_proj_vector_temp[1, 1], illum_proj_vector_temp[1, 2]],
						[illum_proj_vector_temp[2, 1], illum_proj_vector_temp[2, 2]]
					])

					###############################################################################
					#	observation projection
			
					#	calculate x level
					obs_proj_x_level_temp = obs_proj_vector_temp[:, 0].max()

					#	add to foreground or background polygon arrays
					if obs_proj_x_level_temp > obs_proj_x_level:

						if obs_shadowing_first is True:

							#	initialise on first shadowing facet
							obs_run_tot_poly = cascaded_union(obs_proj_poly_temp)
							obs_shadowing_first = False
							obs_shadowing_polys += 1

						elif obs_shadowing_first is False:

							#	check if polygon is fully contained
							contains_flag = obs_run_tot_poly.contains(obs_proj_poly_temp)

							#	if not, add to output
							if contains_flag is False:
								
								obs_run_tot_poly = cascaded_union([
									obs_run_tot_poly,
									obs_proj_poly_temp
								])

								#	itterate number of observation shadowing facets
								obs_shadowing_polys += 1

					###################################################################################
					#	illumination projection

					#	calculate x level
					illum_proj_x_level_temp = illum_proj_vector_temp[:, 0].max()

					#	add to foreground or background polygon arrays
					if illum_proj_x_level_temp > illum_proj_x_level:

						if illum_shadowing_first is True:

							#	initialise on first shadowing facet
							illum_run_tot_poly = cascaded_union(illum_proj_poly_temp)
							illum_shadowing_first = False
							illum_shadowing_polys += 1

						elif illum_shadowing_first is False:

							#	check if polygon is fully contained
							contains_flag = illum_run_tot_poly.contains(illum_proj_poly_temp)
							
							#	if not, add to output
							if contains_flag is False:
								illum_run_tot_poly = cascaded_union([
									illum_run_tot_poly,
									illum_proj_poly_temp
								])

								#	itterate number of illumination shadowing facets
								illum_shadowing_polys += 1

			if brightness_loop_log_level > 0:
				#	calculate sim time
				shad_calc_time = time.time() - shad_calc_start
				print('\n				Shadow Calculation Time 		= {:1.4f} ms.'.format(
					1000 * shad_calc_time)
				)

			#######################################################################################
			#	calculate visibility fractions
			#######################################################################################

			if brightness_loop_log_level > 0:
				print('\n			Calculating Visibility Fractions...')
				frac_calc_start = time.time()

			#######################################################################################
			#	observation visibility fraction

			if obs_shadowing_polys == 0:
				obs_visible_perc = 100.0
			else:

				#	take convex hull
				obs_run_tot_poly = obs_run_tot_poly.convex_hull

				#	find intersecting area
				obs_proj_intersection = obs_proj_step_poly.intersection(obs_run_tot_poly)

				obs_visible_perc = 100 * (
					1.0 - (obs_proj_intersection.area / obs_proj_step_poly.area)
				)

			#######################################################################################
			#	illumination visibility fraction

			if illum_shadowing_polys == 0:
				illum_visible_perc = 100.0
			else:

				#	take convex hull
				illum_run_tot_poly = illum_run_tot_poly.convex_hull

				#	find intersecting area
				illum_proj_intersection = illum_proj_step_poly.intersection(illum_run_tot_poly)
	
				illum_visible_perc = 100 * (
					1.0 - (illum_proj_intersection.area / illum_proj_step_poly.area)
				)

			if brightness_loop_log_level > 0:
				print('\n						Visibility Percentage = ', obs_visible_perc)
				print('						Illumination Percentage = ', illum_visible_perc)
	
				#	calculate sim time
				frac_calc_time = time.time() - frac_calc_start
				print('\n				Fraction Calculation Time 		= {:1.4f} ms.'.format(
					1000 * frac_calc_time)
				)

			#######################################################################################
			#	finalise brightness calculation
			#######################################################################################

			if brightness_loop_log_level > 0:
				print('\n			Performing BRDF Calculation...')
				brightness_start = time.time()

			#	extract projected area and modify by visibility percentages
			proj_area = face_area * (obs_visible_perc / 100) * (illum_visible_perc / 100)

			#	apply cook-torrance BDRF
			facet_brightness = cook_torrance(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
				H_vec, normal, site_body_unit, sun_body_unit, proj_area, face_index)


			facet_brightness_app = facet_brightness / (4 * np.pi * r_obs_act**2)
			facet_brightness_std = facet_brightness / (4 * np.pi * r_obs_std**2)

			print('\nfacet_brightness_app = ',  facet_brightness_app)
			sys.exit()
	
			#	add facet brightness to output array
			face_obs_app.append(facet_brightness_app)
			face_obs_std.append(facet_brightness_std)

			#	update facet number
			facet_num += 1

			#######################################################################################
			#	end of brightness calculation of face
	
			if brightness_loop_log_level > 0:
				#	calculate sim time
				brightness_time = time.time() - brightness_start
				print('\n				BRDF Calculation Time 			= {:1.4f} ms.'.format(
					1000 * brightness_time)
				)

				faceloop_step_time = time.time() - faceloop_step_start
				print('\n			Face Loop Step Time: {:1.4f} ms.'.format(
					1000 * faceloop_step_time)
				)


		###########################################################################################
		#	end of face loop
		if brightness_loop_log_level > 0:
			print('\n			############################################################')
			print('			############################################################')

		if time_loop_log_level > 0:
			#	calculate sim time
			face_loop_time = time.time() - face_loop_start
			print('\n			Brightness Calculation Loop Time	= {:1.4f} ms.'.format(
				1000 * face_loop_time)
			)

		################################################################################################
		#	finalise data
		################################################################################################

		if time_loop_log_level > 0:
			print('\n		Finalising Data of Time Step...')
			final_start = time.time()

		################################################################################################
		#	apparent data

		#	convert to numpy array
		face_obs_app = np.array(face_obs_app)

		if face_obs_app.size != 0:
			brightness_step_app = np.sum(face_obs_app)

			#	apparent magnitude calculation
			magnitude_app = -26.74 - 2.5 * np.log10(brightness_step_app)

		#	no observable facets
		elif face_obs_app.size == 0:

			#	set to limiting apparent magnitude
			magnitude_app = 18.0
			n_limits += 1 

		################################################################################################
		#	standard data

		#	convert to numpy array
		face_obs_std = np.array(face_obs_std)

		if face_obs_std.size != 0:
			brightness_step_std = np.sum(face_obs_std)

			#	standard magnitude calculation
			magnitude_std = -26.74 - 2.5 * np.log10(brightness_step_std)

		#	no observable facets
		elif face_obs_std.size == 0:

			#	set to limiting standard magnitude
			magnitude_std = 18.0
			n_limits += 1 

		################################################################################################
		#	populate return array
		light_curve[time_step, 2] = magnitude_app
		light_curve[time_step, 3] = magnitude_std

		if time_loop_log_level > 0:
			print('\n			Brightness of Time Step:')
			print('				Apparent: ', magnitude_app)
			print('				Standard: ', magnitude_std)

			#	calculate sim time
			final_time = time.time() - final_start
			print('\n			Finalisation Time 	= {:1.4f} ms.'.format(1000 * final_time))

			#	calculate sim time
			timeloop_time = time.time() - timeloop_start
			
			print('\n		Time Step Complete.')
			print('			- Time Step time taken = {:1.4f} ms.'.format(1000 * timeloop_time))

	if n_limits != 0:
		print('\n	{} of {} Data points set to limiting magnitude.'.format(
			int(n_limits),
			int(tot_points)
		))

	###############################################################################################
	#	calculate run time
	###############################################################################################
	
	if log_level > 0:

		#	calculate sim time
		slc_shadow_end = time.time()
		slc_shadow_time = np.round((slc_shadow_end - slc_shadow_start), 3)

		print('\n	SLC Shadowing Finished.')
		print('\n	- Total Time taken = {} s.'.format(slc_shadow_time))

	return light_curve

###################################################################################################
#	Function: 		glim_slc
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties 
#
###################################################################################################

def glim_slc_new(sun_vec_data, site_vec_data, light_curve, \
	jd_start, vectors, areas, brdf_data):

#	print('\n	#################################################################################')
#	print('	Running glim_slc_new script...')

	#	sensor detection limit [apparent magnitude]
	mag_limit = 18

	#	total number of facets
	tot_facets = len(vectors)

	#	number of data points set to limit
	n_limits = 0
	tot_points = len(light_curve)

#	print('\n		Looping through time steps...')

	#	loop through and populate light curve array
	for time_step, row in enumerate(light_curve):

#		print('\n		#############################################################################')
#		print('		Working on time step: ', time_step)

		#	time data
		time_since = light_curve[time_step, 1]
		jdate = jd_start + (time_since / 86400)
		light_curve[time_step, 0] = jdate
	
		#	position data
		sun_vec_step = sun_vec_data[time_step, :]
		site_vec_step = site_vec_data[time_step, :]

		#	using actual distance
		dist_vec = np.copy(site_vec_step) * 1000
		r_obs_act = np.linalg.norm(dist_vec)

		#	using standard distance of 1000km
		dist_vec_std = np.array([1000.0, 0.0, 0.0]) * 1000
		r_obs_std = np.linalg.norm(dist_vec_std)

		#	unit vectors
		sun_vec_unit = sun_vec_step / np.linalg.norm(sun_vec_step)
		site_vec_unit = site_vec_step / np.linalg.norm(site_vec_step)

		#	sun-obs phase angle
		phase_arccos_of = np.divide(
			np.dot(sun_vec_unit, site_vec_unit),
			np.linalg.norm(sun_vec_unit) * np.linalg.norm(site_vec_unit)
		)

		if phase_arccos_of > 1.0:
			if phase_arccos_of < 1.0 + 1e-3:
				phase_arccos_of = 1.0
		elif phase_arccos_of < -1.0:
			if phase_arccos_of > -1.0 - 1e-6:
				phase_arccos_of = -1.0

		#	sun-obs phase angle
		phase_ang = np.arccos(phase_arccos_of)

		################################################################################################
		#	brightness calculation
	
		#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		#USED:
		#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		H_vec = (sun_vec_unit + site_vec_unit) / len(sun_vec_unit + site_vec_unit) 

		#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		#SHOULD BE:
		#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		#	angular bisector
		H_vec = (sun_vec_unit + site_vec_unit) / np.linalg.norm(sun_vec_unit + site_vec_unit) 

		#	observable facet array
		face_obs_app = []
		face_obs_std = []
		facet_num = 1

#		print('\n			Performing Facet Brightness Calculation Loop...')
		# loop through facets
		for face_index, (vector, area, brdf_line) in enumerate(zip(vectors, areas, brdf_data)):

			#	parse brdf data
			m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf = brdf_line
	
			#	calculate normal unit vector
			vect12 = vector[1] - vector[0]
			vect23 = vector[2] - vector[1]
			normal = np.cross(vect12, vect23)
			normal = normal / np.linalg.norm(normal)
	
			#	calculate ilumination angle
			sun_arccos_of = np.divide(
				np.dot(normal, sun_vec_unit),
				np.linalg.norm(normal) * np.linalg.norm(sun_vec_unit)
			)

			if sun_arccos_of > 1.0:
				if sun_arccos_of < 1.0 + 1e-6:
					sun_arccos_of = 1.0
			elif sun_arccos_of < -1.0:
				if sun_arccos_of > -1.0 - 1e-6:
					sun_arccos_of = -1.0

			sun_ang = np.arccos(sun_arccos_of)
	
			#	calculate observation angle
			site_arccos_of = np.divide(
				np.dot(normal, site_vec_unit),
				np.linalg.norm(normal) * np.linalg.norm(site_vec_unit)
			)

			if site_arccos_of > 1.0:
				if site_arccos_of < 1.0 + 1e-3:
					site_arccos_of = 1.0
			elif site_arccos_of < -1.0:
				if site_arccos_of > -1.0 - 1e-6:
					site_arccos_of = -1.0

			site_ang = np.arccos(site_arccos_of)

			#	illuminated and observable condition
			if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):
#				print('\n				################################################################')
#				print('				Working on Illuminated Observable Facet: ', itt)

				#	apply cook-torrance BDRF
				facet_brightness = cook_torrance(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
					H_vec, normal, site_vec_unit, sun_vec_unit, area, face_index)

				facet_brightness_app = facet_brightness / (4 * np.pi * r_obs_act**2)
				facet_brightness_std = facet_brightness / (4 * np.pi * r_obs_std**2)

#				print('\nfacet_brightness_app = ',  facet_brightness_app)
#				sys.exit()
	
				#	add facet brightness to output array
				face_obs_app.append(facet_brightness_app)
				face_obs_std.append(facet_brightness_std)

			#	update facet number
			facet_num += 1

		################################################################################################
		#	finalise data

		#	apparent data

		#	convert to numpy array
		face_obs_app = np.array(face_obs_app)

		if face_obs_app.size != 0:
			brightness_step_app = np.sum(face_obs_app)
			#	apparent magnitude calculation
			magnitude_app = -26.74 - 2.5 * np.log10(brightness_step_app)

		#	no observable facets
		elif face_obs_app.size == 0:

			#	set to limiting apparent magnitude
			magnitude_app = 18.0
			n_limits += 1 

		#	standard data

		#	convert to numpy array
		face_obs_std = np.array(face_obs_std)

		if face_obs_std.size != 0:
			brightness_step_std = np.sum(face_obs_std)
			#	standard magnitude calculation
			magnitude_std = -26.74 - 2.5 * np.log10(brightness_step_std)

		#	no observable facets
		elif face_obs_std.size == 0:

			#	set to limiting standard magnitude
			magnitude_std = 18.0
			n_limits += 1 

		################################################################################################
		#	populate return array
		light_curve[time_step, 2] = magnitude_app
		light_curve[time_step, 3] = magnitude_std

	if n_limits != 0:
		print('\n	{} of {} Data points set to limiting magnitude.'.format(
			int(n_limits),
			int(tot_points)
		))

	return light_curve

###################################################################################################
#	Function: 		cook_torrance
#	Description: 	calculates an observed flux using the cook-torrance BDRF
#
###################################################################################################

def cook_torrance(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
		H_vec, normal, site_vec_unit, sun_vec_unit, area, face_index):

#	print('\n				Running Cook Torrence:')
#	print('\n					H_vec = ', H_vec)
#	print('					Norm = ', normal)
#	print('					site_vec_unit = ', site_vec_unit)
#	print('					sun_vec_unit = ', sun_vec_unit)
#	print('					area = ', area)

	#	dot product of H and N
	dot_prod = np.dot(H_vec, normal)

	#	product of magnitudes of H and N
	mag_prod = np.linalg.norm(H_vec) * np.linalg.norm(normal)

	#	arccos argument
	arccos_arg = dot_prod / mag_prod

	#	correct for arccos_args above 1.0
	small = 1e-6

	#	alpha calculation
	alpha_ang = np.arccos(arccos_arg)
	
	#	G_brdf calculation
	temp1 = 2 * np.divide(
		np.multiply(np.dot(normal, H_vec), np.dot(normal, site_vec_unit)),
		np.dot(site_vec_unit, H_vec)
	)
	
	temp2 = 2 * np.divide(
		np.multiply(np.dot(normal, H_vec), np.dot(normal, sun_vec_unit)),
		np.dot(sun_vec_unit, H_vec)
	)

	if temp1 >= 1 and temp2 >= 1:
	    G_brdf = 1
	elif temp1 < 1 and temp2 >= 1:
		G_brdf = temp1
	elif temp1 >= 1 and temp2 < 1:
		G_brdf = temp2
	elif temp1 < 1 and temp2 < 1:
		temp3 = np.array((temp1, temp2))
		G_brdf = temp3.min()

	#	c_brdf calculation
	c_brdf = np.dot(site_vec_unit, H_vec)

	#	g_brdf calculation
	g_brdf = n_brdf**2 + c_brdf**2 - 1
	
	#	F_brdf calculation
	F_brdf = (
		((g_brdf - c_brdf)**2 / (2 * (g_brdf + g_brdf))) *
		((1 + (c_brdf * (g_brdf + c_brdf) - 1))**2 / ((c_brdf * (g_brdf - c_brdf) + 1)**2))
	)

	#	D_brdf_calculation
	D_brdf = (
		(1 / (m_brdf**2 * np.cos(alpha_ang)**4)) * np.exp(-((np.tan(alpha_ang) / m_brdf)**2))
	)

	#	specular reflectance calculation
	temp_a = (D_brdf * G_brdf * F_brdf)
	temp_b = (np.pi * np.dot(normal, sun_vec_unit) * np.dot(normal, site_vec_unit))

	if temp_a == 0.0 or temp_b == 0.0:
		R_s = 0.0

	else:

		R_s = (
			temp_a / temp_b
		)

	#	diffuse reflectance calculation
	R_d = omega_brdf / np.pi;
	
	#	brightness calculation
	facet_brightness = (
		(d_brdf * R_d + s_brdf * R_s) *
		area *
		np.pi *
		np.dot(normal, sun_vec_unit) *
		np.dot(normal, site_vec_unit)
	)

	return facet_brightness
