###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	requests
import requests

#	numpy
import numpy as np

#	custom general functions
from tools.functions.functions_list import jd2date, date2jd, jday_jfrac

###################################################################################################
#	generate spacetrack request

def req_tle(start_date, stop_date, norad_id):

#	print('Requesting TLE file from Spacetrack.com...')

	year_start = int(start_date[0])
	mon_start = int(start_date[1])
	day_start = int(start_date[2])

	year_stop = int(stop_date[0])
	mon_stop = int(stop_date[1])
	day_stop = int(stop_date[2])

	#	create date string
	req_datestart = '{:4}-{:02}-{:02}'.format(year_start, mon_start, day_start)
	req_datestop = '{:4}-{:02}-{:02}'.format(year_stop, mon_stop, day_stop)
	req_datestr = '--'.join((req_datestart, req_datestop))

	#	spacetrack login information
	login_url = 'https://www.space-track.org/ajaxauth/login'
	username = 'ldjblacketer@gmail.com'
	password = 'Chromodynamics1992'
	
	payload = {
		'identity': username,
		'password': password
	}
	
	base_url = 'https://www.space-track.org/basicspacedata/query'
	request_class = 'class/tle'
	object_specifier = 'NORAD_CAT_ID/{}'.format(norad_id)
	daterange_specifier = 'EPOCH/{}'.format(req_datestr)
	format_specifier = 'format/{}'.format('3le')
	show_empty_str = 'emptyresult/show'
	
	# spacetrack request url
	req_url = '/'.join([
		base_url,
		request_class,
		object_specifier,
		daterange_specifier,
		format_specifier,
		show_empty_str
	])

	#	create request session
	with requests.Session() as session:
		post = session.post(login_url, data=payload)
		output = session.get(req_url).text.split('\n')

#	print('TLE Request Complete, pausing for 8s...')
#	time.sleep(8)

	return output

###################################################################################################
#	request omm fire
###################################################################################################

def req_omm(start_date, stop_date, norad_id):

	print('Requesting OMM file from Spacetrack.com...')

	year_start = int(start_date[0])
	mon_start = int(start_date[1])
	day_start = int(start_date[2])

	year_stop = int(stop_date[0])
	mon_stop = int(stop_date[1])
	day_stop = int(stop_date[2])

	#	create date string
	req_datestart = '{:4}-{:02}-{:02}'.format(year_start, mon_start, day_start)
	req_datestop = '{:4}-{:02}-{:02}'.format(year_stop, mon_stop, day_stop)
	req_datestr = '--'.join((req_datestart, req_datestop))

	#	spacetrack login information
	login_url = 'https://www.space-track.org/ajaxauth/login'
	username = 'ldjblacketer@gmail.com'
	password = 'Chromodynamics1992'
	
	payload = {
		'identity': username,
		'password': password
	}
	
	base_url = 'https://www.space-track.org/basicspacedata/query'
	request_class = 'class/omm'
	object_specifier = 'NORAD_CAT_ID/{}'.format(norad_id)
	daterange_specifier = 'EPOCH/{}'.format(req_datestr)
	format_specifier = 'format/{}'.format('csv')
	
	# spacetrack request url
	req_url = '/'.join([
		base_url,
		request_class,
		object_specifier,
		daterange_specifier,
		format_specifier,
	])

	#	create request session
	with requests.Session() as session:
		post = session.post(login_url, data=payload)
		output = session.get(req_url).text.split('\n')

	return output

###################################################################################################
#	EOF
###################################################################################################
