#	standard packages
import os
import sys
import time
import datetime
from subprocess import call


import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import optimize

from stl import mesh

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front stop script initilised.')

#########################################################################################
#   inputs
#########################################################################################

ref_case = 'specular'
att_case = 'y-rot_cylinder'

#########################################################################################
# set output directories

case_desig = 'lightcurve_data_cylinder_0.1'

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)
input_path = os.path.join(path_data, 'lightcurve_dir', case_desig)

lightcurve_path = os.path.join(root_path, archive_path, 'lightcurve_dir', case_desig, 'lightcurve_files')
lightcurve_dir = os.listdir(lightcurve_path)
lightcurve_files = []

#	Plot lightcurves
fig_dimx = 8
fig_dimy = 6

fig1 = plt.figure(num=1, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
ax1 = fig1.add_subplot(111)

# ax1.set_xlim(150, 800)
ax1.set_ylim(20, 0)
ax1.set_xlabel('Time / s')
ax1.set_ylabel('brightness / mag')
ax1.tick_params(direction='in')
ax1.grid()

plot_azimuth = 270

for reflection_case in lightcurve_dir:
	read_path = os.path.join(lightcurve_path, reflection_case)
	lightcurve_attit_dir = os.listdir(read_path)

	print('\nWorking on reflection case: ', reflection_case)
	if reflection_case == ref_case:

		for attit_case in lightcurve_attit_dir:
			read_path = os.path.join(lightcurve_path, reflection_case, attit_case)
			lightcurve_case_dir = os.listdir(read_path)
	
			print('\n	Working on attit case: ', attit_case)
			if attit_case == att_case:
	
				for lightcurve_file in lightcurve_case_dir:
		
#					print('\n	Working from lightcurve file: ', lightcurve_file)
		
					lc_id, attit_case_name, azm_case_name, elv_case_name, geom_str, reflection_case = \
							lightcurve_file[0:-11].split('_')

					azimuth = int(azm_case_name[3:])
					elevation = int(elv_case_name[3:])

					case_name = azm_case_name + '_' + elv_case_name
					case_label = case_name

					if azimuth == plot_azimuth:
						case_data = np.genfromtxt(os.path.join(read_path, lightcurve_file), delimiter=',')
						lightcurve = case_data
					
						ax1.plot(
							lightcurve[:, 1],
							lightcurve[:, 2],
			#				color=line_color,
							label=case_label,
						)
				
				#	finalise figure
				ax1.set_title(
					'Lightcurves for a range of Elevation values, for Azimuth = {}'.format(plot_azimuth)
				)
				ax1.legend(prop={'size': 6})
				plt.show()
				sys.exit()

			else:
				pass
