###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	
#	Parse outputs of a bulk run into separate directories
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

# shutil for file copy and recursive delete
import shutil

###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front end script initilised.')

#	set start time
sim_start = time.time()

###################################################################################################
#	inputs
###################################################################################################

#	total light curve file
bulk_lightcurve_file = 'cylinder_total_lightcurves_0.1s.txt'

geom_str, _, _, step_str = bulk_lightcurve_file[0:-5].split('_')

#	generate case designation
case_desig = '_'.join(['lightcurve_data', geom_str, '{}'.format(step_str)])

###################################################################################################
#	initilisation
###################################################################################################

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

#	light curve read path
lightcurve_in = os.path.join(path_data, 'lightcurve_dir', case_desig, 'total_lightcurves')

#	parsed lightcurve file temp path
lightcurve_out = os.path.join(path_data, 'lightcurve_dir', case_desig, 'lightcurve_files')

###################################################################################################
#	parse lightcurves
###################################################################################################

##################################################
#	prepare output directory

#	generate lightcurve output directory
if not os.path.exists(lightcurve_out):
	os.mkdir(lightcurve_out)
	parse_bulkfile = True

#	check for clean output directory
else:
	dir_content = os.listdir(lightcurve_out)

	if dir_content == []:
		print('\nParsed lightcurve output directory is clean.')
		parse_bulkfile = True

	else:
		user_in = input('\nDelete current parsed lightcurve output directory contents? [y/n]: ')

		if user_in == 'y':
			print('Deleting current contents..')

			for sub_dir in dir_content:
				shutil.rmtree(os.path.join(lightcurve_out, sub_dir))

			parse_bulkfile = True

		if user_in == 'n':
			user_in = input('\nUse existing data? [y/n]: ')

			if user_in == 'y':
				parse_bulkfile = False

			elif user_in == 'n':
				print('\nExiting.')
				sys.exit()

		if user_in != 'y' and user_in != 'n':
			print('\nError, exiting.')
			sys.exit()


if parse_bulkfile is True:

	print('\nParsing lightcurve bulk data.')

	##################################################
	#	total light curve data
	#	read in run file
	print('\nReading bulk lightcurve file...')
	bulk_file_data = []
	with open(os.path.join(lightcurve_in, bulk_lightcurve_file), mode='r', encoding='utf-8') as case_file:
	
		for line in case_file:
			line_data = line.strip('\n').split(', ')
	
			#	total data
			bulk_file_data.append(line_data)
	
	##################################################
	#	separating files

	print('\nExtracting individual lightcurve files...')

	first_flag = True
	for line in bulk_file_data:
	
		##################################################
		#	start of a first light curve
		if first_flag is True:
	
			#	parse case data
			case_num = int(line[1])
			case_ident = line[0]
	
			#	parse run data
			attit_case, azm, elv, geom_case, reflection_case = case_ident.split('_')
	
			azm_case = azm
			elv_case = elv
	
			#	Generate file name
			if case_num < 10:
				case_num_str = '00000' + str(case_num)
		
			elif case_num < 100:
				case_num_str = '0000' + str(case_num)
		
			elif case_num < 1000:
				case_num_str = '000' + str(case_num)
		
			elif case_num < 10000:
				case_num_str = '00' + str(case_num)
		
			elif case_num < 100000:
				case_num_str = '0' + str(case_num)
		
			elif case_num > 100000:
				case_num_str = str(case_num)
	
			filename = '{:s}_{:s}_{:s}_{:s}_{:s}_{:s}.lightcurve'.format(
				case_num_str,
				attit_case,
				azm_case,
				elv_case,
				geom_case,
				reflection_case
			)
	
			#	extract line data
			data_line = np.array(line[2:]).astype(np.float)
	
			#	initilise data
			output_data = data_line
			current_id = case_num
			first_flag = False
	
		##################################################
		#	not a first run
		elif first_flag is False:
	
			#	parse case data
			case_num = int(line[1])
			case_ident = line[0]
	
			##################################################
			#	continued run
			if case_num == current_id:
	
				#	parse data line
				data_line = np.array(line[2:]).astype(np.float)
	
				#	append to output data
				output_data = np.vstack((output_data, data_line))
	
			##################################################
			#	start of new run
			elif case_num != current_id:
	
				#	save previous data
				np.savetxt(os.path.join(lightcurve_out, filename), output_data, delimiter=',')
	
				#	parse new run data
				attit_case, azm_case, elv_case, geom_case, reflection_case = case_ident.split('_')
	
				#	Generate file name
				if case_num < 10:
					case_num_str = '00000' + str(case_num)
			
				elif case_num < 100:
					case_num_str = '0000' + str(case_num)
			
				elif case_num < 1000:
					case_num_str = '000' + str(case_num)
			
				elif case_num < 10000:
					case_num_str = '00' + str(case_num)
			
				elif case_num < 100000:
					case_num_str = '0' + str(case_num)
			
				elif case_num > 100000:
					case_num_str = str(case_num)
	
				#	generate new filename
				filename = '{:s}_{:s}_{:s}_{:s}_{:s}_{:s}.lightcurve'.format(
					case_num_str,
					attit_case,
					azm_case,
					elv_case,
					geom_case,
					reflection_case
				)
	
				#	extract line data
				data_line = np.array(line[2:]).astype(np.float)
	
				#	initilise new data
				output_data = data_line
				current_id = case_num

	#	save ongoing case at loop end
	np.savetxt(os.path.join(lightcurve_out, filename), output_data, delimiter=',')

print('\nData parse complete.')

###################################################################################################
#	read in and sort data files
###################################################################################################

lightcurve_files = []
lightcurve_dir = os.listdir(lightcurve_out)

print('\nReading and sorting lightcurve files...')
#	parse case files
for lightcurve_file in lightcurve_dir:

	#	parse case information
	case_id, attit_case, azm, elv, geom_case, reflection_case = \
		lightcurve_file.split('.')[0].split('_')

	lightcurve_files.append(
		[
			int(case_id),
			attit_case,
			azm,
			elv,
			geom_case,
			reflection_case,
			lightcurve_file
		]
	)
	
#	sort files
lightcurve_files = sorted(lightcurve_files, key=lambda x: x[0])

###################################################################################################
#	parse into case directories
###################################################################################################

init = True

##################################################
#	parse case files

print('\nMoving lightcurve files into subdirectories...')

for lightcurve_line in lightcurve_files:

	#	parse case information
	case_id, attit_case, azm, elv, geom_case, reflection_case, lightcurve_file = lightcurve_line

	#	generate case name
	case_name = '_'.join([attit_case, geom_case])

	##################################################
	#	first initilisation
	if init == True:

		#	generate reflection case directory if doesnt exist
		reflection_case_path = os.path.join(lightcurve_out, reflection_case)

		if not os.path.exists(reflection_case_path):
			os.mkdir(reflection_case_path)

		#	generate case directory if doesnt exist
		case_path = os.path.join(reflection_case_path, case_name)

		if not os.path.exists(case_path):
			os.mkdir(case_path)

		#	move current file to new directory
		os.rename(
			os.path.join(lightcurve_out, lightcurve_file),
			os.path.join(case_path, lightcurve_file)
		)

#		#	copy current file to new directory
#		shutil.copyfile(
#			os.path.join(lightcurve_out, lightcurve_file),
#			os.path.join(case_path, lightcurve_file)
#		)

		#	set current reflection case
		current_reflection_case = reflection_case

		#	set current case name
		current_case_name = case_name

		init = False

	##################################################
	#	continued
	elif init == False:

		##################################################
		#	line reflection case matches current case
		if reflection_case == current_reflection_case:
	
			#	reflection case and case name matches current case 
			if case_name == current_case_name:
	
				#	move current file to new directory
				os.rename(
					os.path.join(lightcurve_out, lightcurve_file),
					os.path.join(case_path, lightcurve_file)
				)

#				#	copy current file to new directory
#				shutil.copyfile(
#					os.path.join(lightcurve_out, lightcurve_file),
#					os.path.join(case_path, lightcurve_file)
#				)
	
			#	else if reflection case matches, but new case name
			elif  case_name != current_case_name:
	
				#	generate new case directory
				case_path = os.path.join(reflection_case_path, case_name)
	
				if not os.path.exists(case_path):
					os.mkdir(case_path)
	
				#	move current file to new directory
				os.rename(
					os.path.join(lightcurve_out, lightcurve_file),
					os.path.join(case_path, lightcurve_file)
				)

#				#	copy current file to new directory
#				shutil.copyfile(
#					os.path.join(lightcurve_out, lightcurve_file),
#					os.path.join(case_path, lightcurve_file)
#				)
	
				#	set new current case name
				current_case_name = case_name
	
		##################################################
		#	else if new reflection case

		elif reflection_case != current_reflection_case:
	
			#	generate new reflection case directory
			reflection_case_path = os.path.join(lightcurve_out, reflection_case)
	
			if not os.path.exists(reflection_case_path):
				os.mkdir(reflection_case_path)
	
			#	generate new case directory
			case_path = os.path.join(reflection_case_path, case_name)
	
			if not os.path.exists(case_path):
				os.mkdir(case_path)
	
				#	move current file to new directory
				os.rename(
					os.path.join(lightcurve_out, lightcurve_file),
					os.path.join(case_path, lightcurve_file)
				)

#				#	copy current file to new directory
#				shutil.copyfile(
#					os.path.join(lightcurve_out, lightcurve_file),
#					os.path.join(case_path, lightcurve_file)
#				)
	
			#	reset current reflection case
			current_reflection_case = reflection_case
	
			#	reset current case name
			current_case_name = case_name

print('Complete.')
