###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	geometry file plotter
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors
from matplotlib.patches import FancyArrowPatch

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)


###################################################################################################
#	initilisation
###################################################################################################

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Reports',
	'Journals',
	'ACTA_Astronautica',
	'acta_figures',
	'Final'
)

#	figure save flag
savefile_flag = False

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

geom_dir = 'geometry_dir/geom_dir/CZ-2D'

###################################################################################################
#	figure parameters
###################################################################################################

#	face plot parameters
alpha = 0.5
facecol = [0.1, 0.1, 0.1]
edgecol = [0.5, 0.5, 0.5]

#	graphical output
inch2cm = 2.54

font_sizex = 1
font_sizey = 1
font_pad = 0.5
marker_size = 4
marker_edge = 1

#	initialise figure
fig_3d, ax_3d = plt.subplots(figsize=(8, 5), subplot_kw=dict(projection='3d'))

###################################################################################################
#	plot mesh
###################################################################################################

#	set input geometry
geom_file = 'cz-2d.stl'

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	translate in x
object_mesh.x += 5

vectors = object_mesh.vectors
normals = object_mesh.normals
nfaces = len(normals)

#	facet loop
for vector in vectors:

	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)
	pc.set_edgecolor(edgecol)
	pc.set_facecolor(facecol)

	ax_3d.add_collection3d(pc)


###################################################################################################
#	plot mesh
###################################################################################################

#	set input geometry
geom_file = 'cz-2d_pointed.stl'
geom_file = 'cz-2d_testing.stl'

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	translate in x
object_mesh.x -= 5

vectors = object_mesh.vectors
normals = object_mesh.normals
nfaces = len(normals)

#	facet loop
for vector in vectors:

	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)
	pc.set_edgecolor(edgecol)
	pc.set_facecolor(facecol)

	ax_3d.add_collection3d(pc)

###################################################################################################
#	finalise figure
###################################################################################################

#	set viewing direction
ax_3d.view_init(25, 45)

#	finalise subplot 
ax_3d.set_xlim([-15, 15])
ax_3d.set_ylim([-15, 15])
ax_3d.set_zlim([-15, 15])
ax_3d.set_xlabel('x axis')
ax_3d.set_ylabel('y axis')
ax_3d.set_zlabel('z axis')


plt.show()
sys.exit()
