
import os
import sys
import csv


import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import optimize

from stl import mesh

from domain_splitting import cheb_app
from face_counter import face_count

#########################################################################################
#   initilisation
#########################################################################################

#########################################################################################
# set output directories

case_desig = 'attitude'
case_desig2 = '_d1.0'
# 
# case_desig = 'attitude'
# case_desig2 = '_d0.0'
# 
# case_desig = 'attitude_motion'
# case_desig2 = '_d1.0'
# 
# case_desig = 'attitude_motion'
# case_desig2 = '_d0.0'


save_figs = False

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)
distance_path = os.path.join(path_data, 'distance_dir', case_desig + case_desig2)

print('getting distances from ', distance_path)

fig1name = 'fig_x_' + case_desig + case_desig2 + '.pdf'
fig2name = 'fig_y_' + case_desig + case_desig2 + '.pdf'
fig3name = 'fig_z_' + case_desig + case_desig2 + '.pdf'
fig4name = 'fig_xyz_' + case_desig + case_desig2 + '.pdf'

distance_dir = os.listdir(distance_path)

file_save_path = '/home/ldjb/codebase/sensitivity_study_newdata/new_figs/'
file_save_path = os.path.join(path_data, 'figures')

geom_path = os.path.join(root_path, archive_path, 'geometry_dir', 'geom_files')
face_path = os.path.join(root_path, archive_path, 'geometry_dir')
face_file = 'nfaces_file.csv'

#	power law
def fit_func(x, m, c):
    return x**m * c

# #	exponential
# def fit_func(x, A, K, C):
#     return A * np.exp(K * x) + C
# 
# #	linearise
# def fit_exp_linear(x, y, C=0):
# 
# 	y = y - C
# 	y = np.log(y)
# 	K, A_log = np.polyfit(x, y, 1)
# 	A = np.exp(A_log)
# 	return A, K

def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

#########################################################################################
#	sort distance files

distance_files = []

for distance_file in distance_dir:
	attit_case = distance_file.split('_')[1]

	if 'mot' in attit_case:

		#	attitude motion
		attit_num = attit_case.split('-')[1][-1]

	else:

		#	attitude
		if '-' in attit_case:
			case_str = attit_case.split('-')[0]

			if len(case_str) == 7:
				attit_num = case_str[-2:]

			elif len(case_str) == 8:
				attit_num = case_str[-3:]
		else:
			attit_num = attit_case[-2:]

	distance_files.append([int(attit_num), distance_file])

distance_files = sorted(distance_files, key=lambda x: x[0])

#########################################################################################
#	read geometry files for face numbers

if os.path.isfile(os.path.join(face_path, face_file)):
	print('Found nfaces file')
	nfaces = np.genfromtxt(os.path.join(face_path, face_file), delimiter=',')
else:
	print('Finding face numbers')
	#	set geometry directory
	geom_dir = os.listdir(geom_path)
	
	#	Read and sort geometry files
	geom_files = []
	for case in geom_dir:
		case_num, case_str, _ = case.split('_', maxsplit=2)
		geom_files.append([int(case_num), case_str, case])
	
	geom_files = sorted(geom_files, key=lambda x: x[0])
	
	#	read geometry files for face numbers
	nfaces = np.zeros([len(geom_files)])
	for i, geom_case in enumerate(geom_files):
	
		geom_file = geom_case[2]
	
		#	Load mesh
		my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
		#	count faces
		nfaces[i] = face_count(my_mesh.normals)
	
	np.savetxt(os.path.join(face_path, face_file), nfaces, delimiter=',')

#	number of geometry files
if nfaces.shape == ():
	ngeoms = 1
else:	
	ngeoms = len(nfaces)

#########################################################################################

#	initilise all distances array

ncases = 101

all_dist_x = np.zeros([ncases, ngeoms])
att_case = np.zeros([ncases, 1])

plt.rc('font', family='serif')
plt.rc('xtick', labelsize='x-small')
plt.rc('ytick', labelsize='x-small')

fig_dimx = 8
fig_dimy = 6

#	Plot X cases
fig1 = plt.figure(num=1, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
ax1 = fig1.add_subplot(111)


ax1.set_title('Attitude offsets in X [div1 sphere, phase offset in Z]')
ax1.set_xlabel('Attitude case number')
ax1.set_ylabel('Hilbert Distance')


ax1.tick_params(direction='in')
ax1.grid()

# ax1.set_yscale('symlog')

i = 0

color_settings = np.linspace(0.5, 1.0, 11)
color_index = 0

markers = [
	'o',
	'v',
	's',
	'p',
	'*',
	'+',
	'x',
	'D',
	'h',
	'8',
	'o'
]

if case_desig == 'attitude':
	split_index = 0
	case_match = 'attit0-x'

elif case_desig == 'attitude_motion':
	split_index = 1
	case_match = 'attit-mot0-x'

#########################################################################################
#   x-case
#########################################################################################

attit_range = np.linspace(0, 1, 101)
attit_range = attit_range * np.pi


#	generate distance array

for distance_file in distance_files:

	distance_file = distance_file[1]
	_, case_name, _ = distance_file.split('_')

	if 'x' in case_name:
		print('\nWorking on: ', distance_file)

		case_str = case_name.split('-')[0]

		if len(case_str) == 7:
			case_numid = int(case_str[-2:])

		elif len(case_str) == 8:
			case_numid = int(case_str[-3:])

		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data
 
		att_case[i, 0] = case_numid
		all_dist_x[i, :] = distances
		i += 1


plot_data_x = np.hstack((att_case, all_dist_x[:, 0:1]))

#	sort data
plot_data_x = np.array(sorted(plot_data_x, key=lambda x: x[0]))

ax1.plot(plot_data_x[:, 0], plot_data_x[:, 1])

#########################################################################################
#   y-case
#########################################################################################

#	initilise all distances array
all_dist_y = np.zeros([ncases, ngeoms])
att_case = np.zeros([ncases, 1])

#########################################################################################
#	Plot Y cases
fig2 = plt.figure(num=2, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
ax2 = fig2.add_subplot(111)

ax2.set_title('Attitude offsets in Y [div1 sphere, phase offset in Z]')
ax2.set_xlabel('Attitude case number')
ax2.set_ylabel('Hilbert Distance')
ax2.tick_params(direction='in')
ax2.grid()

i = 0
color_index = 0

for distance_file in distance_files:
	distance_file = distance_file[1]
	case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
	_, case_name, _ = distance_file.split('_')

	if 'y' in case_name:

		print('\nWorking on: ', distance_file)
		case_str = case_name.split('-')[0]

		if len(case_str) == 7:
			case_numid = int(case_str[-2:])

		elif len(case_str) == 8:
			case_numid = int(case_str[-3:])

		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data
 
		att_case[i, 0] = case_numid
		all_dist_y[i, :] = distances
		i += 1


plot_data_y = np.hstack((att_case, all_dist_y[:, 0:1]))

#	sort data
plot_data_y = np.array(sorted(plot_data_y, key=lambda x: x[0]))

#	add x0
plot_data_y = np.vstack((plot_data_x[0], plot_data_y))

ax2.plot(plot_data_y[:, 0], plot_data_y[:, 1])

#########################################################################################
#   z-case
#########################################################################################

#	initilise all distances array
all_dist_z = np.zeros([ncases, ngeoms])
att_case = np.zeros([ncases, 1])

#########################################################################################
#	Plot Z cases

fig3 = plt.figure(num=3, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
ax3 = fig3.add_subplot(111)
# ax3.set_title(case_desig + ' Z-cases' + case_desig2)
# ax3.set_yscale('symlog')

i = 0
color_index = 0

ax3.set_title('Attitude offsets in Z [div1 sphere, phase offset in Z]')
ax3.set_xlabel('Attitude case number')
ax3.set_ylabel('Hilbert Distance')
ax3.tick_params(direction='in')
ax3.legend(prop={'size': 6})
ax3.grid()

for distance_file in distance_files:
	distance_file = distance_file[1]
	case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
	_, case_name, _ = distance_file.split('_')

	if case_name == case_match or 'z' in case_name:

		print('\nWorking on: ', distance_file)
		case_str = case_name.split('-')[0]

		if len(case_str) == 7:
			case_numid = int(case_str[-2:])

		elif len(case_str) == 8:
			case_numid = int(case_str[-3:])

		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data
 
		att_case[i, 0] = case_numid
		all_dist_z[i, :] = distances
		i += 1


plot_data_z = np.hstack((att_case, all_dist_z[:, 0:1]))

#	sort data
plot_data_z = np.array(sorted(plot_data_z, key=lambda x: x[0]))

#	add x0
plot_data_z = np.vstack((plot_data_x[0], plot_data_z))

ax3.plot(plot_data_z[:, 0], plot_data_z[:, 1])

plt.show()
sys.exit()


#########################################################################################
#	Plot All Attitude cases
#fig = plt.figure(num=4, figsize=(9, 8), dpi=80, facecolor='w', edgecolor='k')
fig4 = plt.figure(num=4, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
ax4 = fig4.add_subplot(111)
ax4.set_title(case_desig + '-cases' + case_desig2)
# ax4.set_yscale('symlog')

ax4.set_xlabel('Number of Facets')
ax4.set_ylabel('Hilbert Distance')
ax4.tick_params(direction='in')
ax4.set_xlim(18, 22)
ax4.set_ylim(32.75, 33.75)
ax4.grid()

color_index = 0

for distance_file in distance_files:
	distance_file = distance_file[1]
	case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
	_, case_name, _ = distance_file.split('_')

	if case_name == case_match or (
		'x' not in case_name and \
		'y' not in case_name and \
		'z' not in case_name
		):

		case_numid = int(case_name.split('-')[split_index][-1])

		if case_numid == 0:
			case_label = '0 $pi$'
		if case_numid == 1:
			case_label = '0.2 $pi$'
		if case_numid == 2:
			case_label = '0.4 $pi$'
		if case_numid == 3:
			case_label = '0.6 $pi$'
		if case_numid == 4:
			case_label = '0.8 $pi$'
		if case_numid == 5:
			case_label = '1.0 $pi$'

		distances = case_data

		ax4.plot(
			nfaces[0:],
			distances[0:],
			label=case_label,
			marker = markers[color_index],
		)

		#	perform fitting
		xdata = nfaces[0:]
		ydata = distances[0:]

#		#	numpy curve optimization
#		fit_para, pcov = optimize.curve_fit(fit_func, xdata, ydata, maxfev=20000)
#		xin = np.arange(xdata[0], xdata[-1], 1)
#		y_fit = fit_func(xin, *fit_para)
#
#		color_index += 1
#
#		ax4.plot(
#			xin,
#			y_fit,
#		)

#########################################################################################

ax4.legend(prop={'size': 6})

if save_figs is True:
	fig4.savefig(os.path.join(file_save_path, fig4name), bbox_inches='tight')

plt.show()
