
import os
import sys
import csv

import numpy as np

import scipy
from scipy import optimize

from stl import mesh

from domain_splitting import cheb_app
from face_counter import face_count

import matplotlib
import matplotlib.pyplot as plt
from matplotlib.colors import BoundaryNorm
from matplotlib.ticker import MaxNLocator

#########################################################################################
#   initilisation
#########################################################################################

#########################################################################################
# set output directories

bulk_dir = 'bulk_data'

case_desig = 'x-rot'
case_desig2 = '_div1'

reflection_case = 'diffuse'


save_figs = False

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)
distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig + case_desig2)

print('getting distances from ', distance_path)

fig1name = 'fig_x_' + case_desig + case_desig2 + '.pdf'
fig2name = 'fig_y_' + case_desig + case_desig2 + '.pdf'
fig3name = 'fig_z_' + case_desig + case_desig2 + '.pdf'
fig4name = 'fig_xyz_' + case_desig + case_desig2 + '.pdf'

distance_dir = os.listdir(distance_path)

file_save_path = '/home/ldjb/codebase/sensitivity_study_newdata/new_figs/'
file_save_path = os.path.join(path_data, 'figures')

def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

#########################################################################################
#	sort distance files

distance_files = []

for distance_file in distance_dir:

	num_str, attit_case, geom_case, lat_str, lon_str, reflection_case = \
		distance_file.split('.')[0].split('_')

	distance_files.append([int(num_str), distance_file])

distance_files = sorted(distance_files, key=lambda x: x[0])

###################################################################################################
#	acquire number of faces

#	faces file
face_file = 'nfaces_file.csv'

#	face file path
face_path = os.path.join(path_data, 'geometry_dir')

if os.path.isfile(os.path.join(face_path, face_file)):
	nfaces = np.genfromtxt(os.path.join(face_path, face_file), delimiter=',')
else:

	#	geometry directory
	geom_path = os.path.join(path_data, 'geometry_dir', 'geom_files')

	#	set geometry directory
	geom_dir = os.listdir(geom_path)
	
	#	Read and sort geometry files
	geom_files = []
	for case in geom_dir:
		case_num, case_str, _ = case.split('_', maxsplit=2)
		geom_files.append([int(case_num), case_str, case])
	
	geom_files = sorted(geom_files, key=lambda x: x[0])
	
	#	read geometry files for face numbers
	nfaces = np.zeros([len(geom_files)])
	for i, geom_case in enumerate(geom_files):
	
		geom_file = geom_case[2]
	
		#	Load mesh
		my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
		#	count faces
		nfaces[i] = face_count(my_mesh.normals)
	
	np.savetxt(os.path.join(face_path, face_file), nfaces, delimiter=',')

#	number of geometry files
if nfaces.shape == ():
	ngeom = 1
else:	
	ngeom = len(nfaces)

#########################################################################################

data = []

for i, distance_file in enumerate(distance_files):

	num_str, attit_case, geom_case, lat_str, lon_str, reflection_case = \
		distance_file[1].split('.')[0].split('_')

	lat = int(lat_str[3:])
	lon = int(lon_str[3:])

	distance = np.genfromtxt(os.path.join(distance_path, distance_file[1]), delimiter=',')

	data_line = [lat, lon, distance]
	data.append(data_line)

data = np.array(sorted(data, key=lambda x: x[0]))

#	generate x
plot_x = np.zeros([len(np.unique(data[:, 1])), len(np.unique(data[:, 0]))])
xrow = np.unique(data[:, 0])

for i in range(len(plot_x[:, 0])):
	plot_x[i, :] = xrow

#	generate y
plot_y = np.zeros((plot_x.shape))
ycol = np.unique(data[:, 1])

for i in range(len(plot_x[0, :])):
	plot_y[:, i] = ycol


current_lat = data[0, 0]

data_array = np.zeros([len(np.unique(data[:, 1])), len(np.unique(data[:, 0]))])
data_index = 0

data_of_lat = []

for i, line in enumerate(data):

	#	parse latitude of line
	line_lat = line[0]

	if line_lat == current_lat:
		data_of_lat.append(line)

	elif line_lat != current_lat:

		data_of_lat = np.array(sorted(data_of_lat, key=lambda x: x[1]))

		data_array[:, data_index] = data_of_lat[:, 2]

		#	initilize data of lat on new value
		data_of_lat = []
		data_of_lat.append(line)

		data_index += 1

		first_line = True
		current_lat = line_lat


x = plot_x
y = plot_y
z = data_array

x = x
y = y
z = z

z_plot = np.zeros(z.shape)

thres_lvl = 0

if thres_lvl == 0:

	z_plot = np.copy(z)

else:

	z_plot = np.zeros(z.shape)
	
	for i, line in enumerate(z):
		for j, ele in enumerate(line):
	
			if ele > thres_lvl:
				z_plot[i, j] = thres_lvl
			else:
				z_plot[i, j] = ele


#z_plot = z_plot[:-1, :-1]

levels = MaxNLocator(nbins=20).tick_values(z_plot.min(), z_plot.max())
cmap = plt.get_cmap('YlOrRd')
norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

fig, ax0 = plt.subplots(nrows=1)

im = ax0.pcolormesh(x, y, z_plot, cmap=cmap, norm=norm)
fig.colorbar(im, ax=ax0)
ax0.set_title(case_desig)

ax0.set_xlim([-90, 90])
ax0.set_ylim([-90, 90])

ax0.axis('equal')

plt.show()

