
import os
import sys
import csv


import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import optimize

from stl import mesh

from domain_splitting import cheb_app
from face_counter import face_count
from subprocess import call

#########################################################################################
# set output directories
#	clear terminal
call(['clear'])
print('Started.')

case_desig = 'attitude'
case_desig = 'attitude_motion'

case_desig2 = '_d1.0'
case_desig2 = '_d0.0'

save_figs = False

#	select geom to sample at, 0, 1, 2, 3, 4, 5
samp_geom = 0

if case_desig == 'attitude':
	split_index = 0
	case_match = 'attit0-x'

elif case_desig == 'attitude_motion':
	split_index = 1
	case_match = 'attit-mot0-x'

#	input path for distance files
distance_path = '/home/ldjb/codebase/sensitivity_study_newdata/distances/' + case_desig + case_desig2
distance_dir = os.listdir(distance_path)

print('getting distances from ', distance_path)

#	output figure names
figname = 'fig_' + case_desig + case_desig2 + '.pdf'

#	figure output path
file_save_path = '/home/ldjb/codebase'
file_save_path = '/home/ldjb/codebase/sensitivity_study_newdata/new_figs/'

face_path = '/home/ldjb/codebase/git_dir/python/LCG/sensitivity_study'
geom_path = '/home/ldjb/codebase/sensitivity_study_newdata/Geometry/new_geoms'

#	power law
def fit_func(x, m, c):
     return x**m * c

#########################################################################################
#	extract and sort distance files

distance_files = []

for distance_file in distance_dir:
	attit_case = distance_file.split('_')[1]

	#	for attitude motion cases
	if 'mot' in attit_case:
		attit_num = attit_case.split('-')[1][-1]

	#	for attitude cases
	else:
		if '-' in attit_case:
			attit_num = attit_case.split('-')[0][-1]
		else:
			attit_num = attit_case[-1]

	#	add to list
	distance_files.append([int(attit_num), distance_file])

#	sort
distance_files = sorted(distance_files, key=lambda x: x[0])


#########################################################################################
#	read geometry files for face numbers
face_file = 'nfaces_file.csv'

if os.path.isfile(os.path.join(face_path, face_file)):
	nfaces = np.genfromtxt(os.path.join(face_path, face_file), delimiter=',')
else:
	#	set geometry directory
	geom_dir = os.listdir(geom_path)
	
	#	Read and sort geometry files
	geom_files = []
	for case in geom_dir:
		case_num, case_str, _ = case.split('_', maxsplit=2)
		geom_files.append([int(case_num), case_str, case])
	
	geom_files = sorted(geom_files, key=lambda x: x[0])
	
	#	read geometry files for face numbers
	nfaces = np.zeros([len(geom_files)])
	for i, geom_case in enumerate(geom_files):
	
		geom_file = geom_case[2]
	
		#	Load mesh
		my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
		#	count faces
		nfaces[i] = face_count(my_mesh.normals)
	
#	np.savetxt(os.path.join(cwd, face_path, face_file), nfaces, delimiter=',')

#	number of geometry files
ngeoms = len(nfaces)

#	initilise all distance array
all_dist = np.zeros([11, ngeoms])

#########################################################################################
#	figure initilisation

#	formatting
plt.rc('font', family='serif')
plt.rc('xtick', labelsize='x-small')
plt.rc('ytick', labelsize='x-small')

fig_dimx = 8
fig_dimy = 6

color_settings = np.linspace(0.5, 1.0, 10)

markers = [
	'o',
	'v',
	's',
	'p',
	'*',
	'+',
	'x',
	'D',
	'h',
	'8'
]

if case_desig == 'attitude':
	split_index = 0
	case_match = 'attit0-x'

elif case_desig == 'attitude_motion':
	split_index = 1
	case_match = 'attit-mot0-x'

#########################################################################################
#	Plot X cases
#fig = plt.figure(num=1, figsize=(4, 3), dpi=80, facecolor='w', edgecolor='k')
fig1 = plt.figure(num=1, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
ax1 = fig1.add_subplot(111)
ax1.set_title('')
# ax1.set_yscale('symlog')

i = 0
color_index = 0

viva_dist = []
viva_att = []

for distance_file in distance_files:

	distance_file = distance_file[1]
	_, case_name, _ = distance_file.split('_')


	if 'x' in case_name:

		print('\nWorking on: ', case_name)
	
		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data

		viva_att.append(color_index)
		viva_dist.append(case_data[samp_geom])

		all_dist[i, :] = distances
		i += 1

		#	perform fitting
		xdata = nfaces
		ydata = distances

		fit_para, pcov = optimize.curve_fit(fit_func, xdata, ydata, maxfev=20000)

		xin = np.arange(xdata[0], xdata[-1], 1)
		y_fit = fit_func(xin, *fit_para)

		color_index += 1
	
ax1.plot(
	viva_att,
	viva_dist,
	label='xdata'
)

print(viva_att)
print(viva_dist)

#ax1.set_xlabel('Attitude Motion Case')
ax1.set_xlabel('Attitude Case')
ax1.set_ylabel('Hilbert Distance')
ax1.tick_params(direction='in')
# ax1.set_xlim(150, 800)
# ax1.set_ylim(0, 1e5)
# ax1.legend(prop={'size': 6})
ax1.grid()

#########################################################################################
#	Plot Y cases
# fig = plt.figure(num=1, figsize=(4, 3), dpi=80, facecolor='w', edgecolor='k')
# fig2 = plt.figure(num=2, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
# ax2 = fig2.add_subplot(111)
# ax2.set_title('Y-cases')
# ax2.set_yscale('symlog')

i = 0

color_index = 0
viva_dist = []
viva_att = []

for distance_file in distance_files:

	distance_file = distance_file[1]
	_, case_name, _ = distance_file.split('_')


	if case_name == case_match or 'y' in case_name:

		print('\nWorking on: ', case_name)
	
		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data

		viva_att.append(color_index)
		viva_dist.append(case_data[samp_geom])

		all_dist[i, :] = distances
		i += 1

		#	perform fitting
		xdata = nfaces
		ydata = distances

		fit_para, pcov = optimize.curve_fit(fit_func, xdata, ydata, maxfev=20000)

		xin = np.arange(xdata[0], xdata[-1], 1)
		y_fit = fit_func(xin, *fit_para)

		color_index += 1

ax1.plot(
	viva_att,
	viva_dist,
	label='ydata'
)

print(viva_att)
print(viva_dist)

# ax2.set_xlabel('Attitude Motion Case')
# ax2.set_xlabel('Attitude Case')
# ax2.set_ylabel('Hilbert Distance')
# ax2.tick_params(direction='in')
# ax2.set_xlim(150, 800)
# ax2.set_ylim(0, 1e5)
# ax2.legend(prop={'size': 6})
# ax2.grid()

###############################################################################################
#	Plot Y cases
# fig3 = plt.figure(num=1, figsize=(4, 3), dpi=80, facecolor='w', edgecolor='k')
# fig3 = plt.figure(num=3, figsize=(fig_dimx, fig_dimy), facecolor='w', edgecolor='k')
# ax3 = fig3.add_subplot(111)
# ax3.set_title('Z-cases')
# ax3.set_yscale('symlog')

i = 0

color_index = 0
viva_dist = []
viva_att = []

for distance_file in distance_files:

	distance_file = distance_file[1]
	_, case_name, _ = distance_file.split('_')


	if case_name == case_match or 'z' in case_name:

		print('\nWorking on: ', case_name)
	
		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data

		viva_att.append(color_index)
		viva_dist.append(case_data[samp_geom])

		all_dist[i, :] = distances
		i += 1

		#	perform fitting
		xdata = nfaces
		ydata = distances

		fit_para, pcov = optimize.curve_fit(fit_func, xdata, ydata, maxfev=20000)

		xin = np.arange(xdata[0], xdata[-1], 1)
		y_fit = fit_func(xin, *fit_para)

		color_index += 1

ax1.plot(
	viva_att,
	viva_dist,
	label='zdata'
)

#########################################################################################
#	Plot All Attitude cases

i = 0

if case_desig == 'attitude':
	color_index = 0
	viva_dist = []
	viva_att = []
elif case_desig == 'attitude_motion':
	viva_dist = []
	viva_att = []
	color_index = 0

for distance_file in distance_files:

	distance_file = distance_file[1]
	_, case_name, _ = distance_file.split('_')


	if case_name == case_match or (
		'x' not in case_name and \
		'y' not in case_name and \
		'z' not in case_name
		):

		print('\nWorking on: ', case_name)
	
		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data

		viva_att.append(color_index)
		viva_dist.append(case_data[samp_geom])

		all_dist[i, :] = distances
		i += 1

		#	perform fitting
		xdata = nfaces
		ydata = distances

		fit_para, pcov = optimize.curve_fit(fit_func, xdata, ydata, maxfev=20000)

		xin = np.arange(xdata[0], xdata[-1], 1)
		y_fit = fit_func(xin, *fit_para)

		color_index += 1

ax1.plot(
	viva_att,
	viva_dist,
	label='alldata'
)


ax1.legend(prop={'size': 10})

if save_figs is True:
	fig1.savefig(os.path.join(file_save_path, figname), bbox_inches='tight')

plt.show()
