# Case Generation for AMOS2018

import os
import sys
import numpy as np
from stl import mesh
from mpl_toolkits import mplot3d
from matplotlib import pyplot

#########################################################################################

#	number of header data line in case file
data_nlines = 14

def gen_case(
	case_num,
	geom_path,
	geom_file,
	geom_num,
	run_str,
	case_str,
	output_dir,
	append_str,
	attit_mot,
	attit,
	d_bdrf,
	s_bdrf,
	omega_bdrf,
	n_bdrf,
	m_bdrf
	):

	start_time_str = 'start_time = 2014, 10, 13, 15, 53, 54.212000'
	stop_time_str = 'stop_time = 2014, 10, 13, 15, 57, 47.370000'

	start_time_str = 'start_time = 2016, 12, 22, 02, 05, 27.003000'
	stop_time_str = 'stop_time = 2016, 12, 22, 02, 09, 57.908000'

	#	Load mesh for number of faces
	my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
	geom_faces = len(my_mesh.areas)

	#	Generate BDRF parameters
	bdrf_nlines = geom_faces
	bdrf_para = np.zeros([bdrf_nlines, 5])
	
	for i in range(geom_faces):
		bdrf_para[i, :] = [m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf]

	#	Generate file name
	if case_num < 10:
		case_name = ''.join(('000', str(case_num), '_', case_str, '-', run_str, append_str))

	elif case_num < 100:
		case_name = ''.join(('00', str(case_num), '_', case_str, '-', run_str, append_str))

	elif case_num >= 100:
		case_name = ''.join(('0', str(case_num), '_', case_str, '-', run_str, append_str))

	#	Case line loop
	with open(os.path.join(output_dir, case_name), mode='wt', encoding='utf-8') as case_file:

		#	Print header information
		for i  in range(data_nlines):

			#	Title line
			if i is 0:
				case_line = '# object config file'

			#	newline
			if i is 1:
				case_line = ''

			#	Norad ID
			if i is 2:
				case_line = 'norad_id = 25885'

			#	Site Latitude
			if i is 3:
				case_line = 'site_latgd = 43.64986'

			#	Site Longitude
			if i is 4:
				case_line = 'site_lon = 41.43147'

			#	Site Altitude
			if i is 5:
				case_line = 'site_alt = 2030.00000'

			#	Start Time
			if i is 6:
				case_line = start_time_str

			#	Stop Time
			if i is 7:
				case_line = stop_time_str

			#	Attitude 
			if i is 8:
				case_line = 'attit = {:1.4}, {:1.4}, {:1.4}'.format(attit[0], attit[1], attit[2])

			#	Attitude Motion
			if i is 9:
				case_line = 'attit_mot = {:1.4}, {:1.4}, {:1.4}'.format(attit_mot[0], attit_mot[1], attit_mot[2])

			#	newline
			if i is 10:
				case_line = ''

			#	Geometry title
			if i is 11:
				case_line = '# Geometry'

			#	Geometry file
			if i is 12:
				case_line = 'geometry_file=' + geom_file

			#	newline
			if i is 13:
				case_line = ''

			case_file.write(case_line + '\n')


		#	BDRF Headers
		case_file.write('# BDRF' + '\n')
		case_file.write('m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf' + '\n')
		for i, bdrf_line in enumerate(bdrf_para):

			case_line = '{:1.4}, {:1.4}, {:1.4}, {:1.4}, {:1.4}'.format(
				bdrf_line[0],
				bdrf_line[1],
				bdrf_line[2],
				bdrf_line[3],
				bdrf_line[4]
			)

			if i is len(bdrf_para) - 1:
				case_file.write(case_line)
			else:
				case_file.write(case_line + '\n')

	return case_num
