####################################################################################################
#
#	SYNTHETIC LIGHT CURVE GENERATION
#
#	INPUTS:		OBJECT DATA, LIGHT CURVE, HORIZONS AND TLE FILE NAMES,
#
#	OUTPUTS:	SYNTHETIC LIGHT CURVE
#
####################################################################################################
import numpy as np
import os
import sys

from function_list import date2jd, jd2date

from transform import transform
from slc import slc

#	case loop
def fred_std(inputs, bdrf_data):

	#	Initilization
	print('\n==> Running Fred . . . \n')

	object_data, file_name_tle, file_name_horizons, file_name_lc = inputs

	#	create required subdirectories
	norad_id = object_data.norad_ident
	cwd = os.getcwd()
	object_dir = '{:d}'.format(norad_id)

	#	light curves
	lightcurve_path = 'Data/Outputs/Light_curves'
	lightcurve_file_path = os.path.join(cwd, lightcurve_path, object_dir)

	#	transformation
	transform(object_data, file_name_tle, file_name_horizons)

	if os.path.isfile(os.path.join(lightcurve_file_path, file_name_lc)):
		print('Lightcurve Exists.')
	else:
		#	Light Curve Generation
		light_curve = slc(object_data, file_name_lc, bdrf_data)

		#	save file
		np.savetxt(os.path.join(lightcurve_file_path, file_name_lc), light_curve, delimiter=',')
		print('	% Lightcurve Saved.')

	print('\n==> Script Complete.\n')

	return(light_curve)

#############################################################################################

def fred_ss(inputs, bdrf_data):

	#	Initilization
	print('\n==> Running Fred . . . \n')

	object_data, file_name_tle, file_name_horizons, \
		file_name_lc, lightcurve_path, root_path, archive_path, overwrite = inputs

	#	create required subdirectories
	norad_id = object_data.norad_ident
	cwd = os.getcwd()
	object_dir = '{:d}'.format(norad_id)

	#	light curves
	lightcurve_file_path = os.path.join(root_path, lightcurve_path, object_dir)

	#	transformation
	transform(object_data, root_path, archive_path, file_name_tle, file_name_horizons)

	#	light curve existance check

	if overwrite is 1:
		if os.path.isfile(os.path.join(lightcurve_file_path, file_name_lc)):
			print('Lightcurve Exists.')

			#	Light Curve Generation
			light_curve = slc(object_data, root_path, archive_path, file_name_lc, bdrf_data)
			np.savetxt(os.path.join(lightcurve_file_path, file_name_lc), light_curve, delimiter=',')
			print('Overwritten.')

			light_curve = 0
			return(light_curve)

		else:
			#	Light Curve Generation
			light_curve = slc(object_data, root_path, archive_path, file_name_lc, bdrf_data)

			#	save file
			np.savetxt(os.path.join(lightcurve_file_path, file_name_lc), light_curve, delimiter=',')
			print('	% Lightcurve Saved.')

			return(light_curve)

	elif overwrite is 0:
		if os.path.isfile(os.path.join(lightcurve_file_path, file_name_lc)):
			print('Lightcurve Exists.')
			print('No overwriting.')

			light_curve = 0
			return(light_curve)

		else:
			print('Lightcurve Doesnt Exist.')
#			print(lightcurve_file_path, file_name_lc)

			#	Light Curve Generation
			light_curve = slc(object_data, root_path, archive_path, file_name_lc, bdrf_data)
	
			#	save file
			np.savetxt(os.path.join(lightcurve_file_path, file_name_lc), light_curve, delimiter=',')

			return(light_curve)

	print('\n==> Script Complete.\n')

