###################################################################################################
import sys
import numpy as np
import csv
import math
import os
import urllib.request
import shutil
import collections


###################################################################################################

# Function: date2jd


def date2jd(year, mon, day, hour, mins, sec):

	# days
	temp = np.floor((7.0 * (year + np.floor((mon + 9.0) / 12.0))) * 0.25)
	jday = 367.0 * year - temp + np.floor(275.0 * mon / 9.0) + day + 1721013.5

	# day fraction
	jfrac = (sec + mins * 60.0 + hour * 3600.0) / 86400.0
	if jfrac > 1.0:
		jday = jday + 1.0
		jfrac = jfrac - 1.0

	return jday, jfrac

###################################################################################################

# Function: jd2date


def jd2date(jday, jfrac):

		# correct for day fraction greater than 1
		if (abs(jfrac) >= 1):
			jday = jday + np.floor(jdfrac)
			jfrac = jfrac - np.floor(jfrac)

		dt = jday - int(jday)
		if (abs(dt - 0.5) > 0.00000001):
			jfrac = jfrac + dt - 0.5

		# number of leap years
		temp = jday + jfrac - 2415019.5
		tu = temp / 365.25
		year = int(1900 + np.floor(tu))
		leap_years = np.floor((year - 1901) * 0.25)

		# elapsed days
		days = np.floor(temp - ((year - 1900) * 365 + leap_years))

		# correct for day numbers less than 0.5
		if (days < 1):
			year = year - 1
			leap_years = np.floor((year - 1901) * 0.25)
			days = np.floor(temp - ((year - 1900) * 365 + leap_years))

		days = days + jfrac

		# create days in month list
		length_mon = np.zeros((12), dtype=np.int)
		for i in range(12):
			length_mon[i] = 31
			if (i == 1):
				length_mon[i] = 28
			if (i == 3 or i == 5 or i == 8 or i == 10):
				length_mon[i] = 30

		# day of year
		doy = np.floor(days)
		if ((year - 1900) % 4 == 0):
			length_mon[1] = 29

		# month
		j = 0
		temp = 0
		while (doy > (temp + length_mon[j]) and j < 11):
			temp = temp + length_mon[j]
			j = j + 1

		month = j + 1
		day = int(doy - temp)
		temp = (days - doy) * 24
		hour = int(np.fix(temp))
		temp = (temp - hour) * 60
		minute = int(np.fix(temp))
		sec = (temp - minute) * 60.0

		return year, month, day, hour, minute, sec

###################################################################################################

# Function: get_IERS


def get_iers(jday, jfrac):

	# arcsecond to radian conversion
	as2r = np.pi / (180 * 3600)

	# modified julian date
	mjd = (jday + jfrac) - 2400000.5

	# get working directory
	cwd = os.getcwd()

	dat_file = 'tai-utc.dat'
	eop_file = 'eop_c04_14_1962-now.csv'

	dat_path = os.path.join(cwd, dat_file)
	eop_path = os.path.join(cwd, eop_file)

	# read dat data file
	with open(dat_path) as csvfile:
		readCSV = csv.reader(csvfile)
		for i, row in enumerate(readCSV):

			jddate = np.array(row[0].split(' ')[6])
			dat_current = np.array(row[0].split(' ')[10])

			# find most recent dat value
			if (jddate.astype(np.float) - 2400000.5) > mjd:
				break
			dat = dat_current

	dat = dat.astype(np.float)

	# read eop data file
	firstline = True
	with open(eop_file) as csvfile:
		readCSV = csv.reader(csvfile, delimiter=',')
		for i, row in enumerate(readCSV):

			# skip header
			if firstline:
				firstline = False
				continue

			# get MJD
			splitrow = np.array(row[0].split(';'))
			mjd_temp = int(splitrow[0])

			if mjd_temp == int(mjd):
				line = np.array(splitrow)
				linep1 = np.array(readCSV.__next__()[0].split(';'))
				utc = (mjd - np.floor(mjd)) * 86400

				# break iers  data
				mjd, year, mon, day, \
					type1, \
					x_pole, sigma_x_pole, y_pole, sigma_y_pole, \
					type2, \
					dut, sigma_dut, lod, sigma_lod, \
					type3, \
					dPsi, sigma_dPsi, \
					depsilon, sigma_depsilon, \
					dX, sigma_dX, dY, sigma_dY = line

				# get day plus 1 data
				mjdp1, yearp1, monp1, dayp1, \
					type1p1, \
					x_polep1, sigma_x_polep1, y_polep1, sigma_y_polep1, \
					type2p1, \
					dutp1, sigma_dutp1, lodp1, sigma_lodp1, \
					type3p1, \
					dPsip1, sigma_dPsip1, \
					depsilonp1, sigma_depsilonp1, \
					dXp1, sigma_dXp1, dYp1, sigma_dYp1 = linep1

				# Interpolation
				lod = lod.astype(np.float)
				dut = dut.astype(np.float)

				# xpole
				x_pole = x_pole.astype(np.float) * as2r
				sigma_x_pole = sigma_x_pole.astype(np.float) * as2r
				x_polep1 = x_polep1.astype(np.float) * as2r
				sigma_x_polep1 = sigma_x_polep1.astype(np.float) * as2r

				x_poleps = (1 / 86400) * (x_polep1 - x_pole)

				# ypole
				y_pole = y_pole.astype(np.float) * as2r
				sigma_y_pole = sigma_y_pole.astype(np.float) * as2r
				y_polep1 = y_polep1.astype(np.float) * as2r
				sigma_y_polep1 = sigma_y_polep1.astype(np.float) * as2r

				y_poleps = (1 / 86400) * (y_polep1 - y_pole)

				# dut
				dutps = (1 / 86400) * (dutp1.astype(np.float) - dut.astype(np.float))

				# dX
				dX = dX.astype(np.float) * as2r
				sigma_dX = sigma_dX.astype(np.float) * as2r
				dXp1 = dXp1.astype(np.float) * as2r
				sigma_dXp1 = sigma_dXp1.astype(np.float) * as2r

				dXps = (1 / 86400) * (dXp1 - dX)

				# dY
				dY = dY.astype(np.float) * as2r
				sigma_dY = sigma_dY.astype(np.float) * as2r
				dYp1 = dYp1.astype(np.float) * as2r
				sigma_dYp1 = sigma_dYp1.astype(np.float) * as2r

				dYps = (1 / 86400) * (dYp1 - dY)

				return dut, dutps, lod, x_pole, x_poleps, y_pole, y_poleps, dX, dXps, dY, dYps, utc, dat

###################################################################################################

# Get Times


def get_times(year, mon, day, hour, minute, sec, dut1, dat):

	# time object output
	time_obj = collections.namedtuple('time_object', [
		'jday_ut1',
		'jfrac_ut1',
		'tut1',
		'jday_tai',
		'jfrac_tai',
		'ttai',
		'jday_tt',
		'jfrac_tt',
		'ttt',
		'jday_tdb',
		'jfrac_tdb',
		'ttdb',
		'jday_tcg',
		'jfrac_tcg',
		'ttcg',
		'jday_tcb',
		'jfrac_tcb',
		'ttcb',
	])

	# UTC
	utc = (hour * 3600) + (minute * 60) + sec
	temp = date2jd(year, mon, day, hour, minute, sec)
	jdate_utc = temp[0] + temp[1]

	# UT1
	ut1 = utc + dut1
	ut1_hour = int(ut1 / 3600)
	ut1_minute = int(ut1 / 60 - ut1_hour * 3600)
	ut1_sec = ut1 - ut1_minute * 60 - ut1_hour * 3600
	jday_ut1, jfrac_ut1 = date2jd(year, mon, day, ut1_hour, ut1_minute, ut1_sec)
	jdut1 = jday_ut1 + jfrac_ut1
	tut1 = (jdut1 - 2451545) / 36525

	# TAI
	tai = utc + dat
	tai_hour = int(tai / 3600)
	tai_minute = int(tai / 60 - tai_hour * 60)
	tai_sec = tai - tai_minute * 60 - tai_hour * 3600
	jday_tai, jfrac_tai = date2jd(year, mon, day, tai_hour, tai_minute, tai_sec)
	jdtai = jday_tai + jfrac_tai
	ttai = (jdtai - 2451545) / 36525

	# TT
	tt = tai + 32.184
	tt_hour = int(tt / 3600)
	tt_minute = int(tt / 60 - tt_hour * 60)
	tt_sec = tt - tt_minute * 60 - tt_hour * 3600
	jday_tt, jfrac_tt = date2jd(year, mon, day, tt_hour, tt_minute, tt_sec)
	jdtt = jday_tt + jfrac_tt
	ttt = (jdtt - 2451545) / 36525

	# TDB
	tdb = (
		tt + 0.001657 * math.sin(628.3076 * ttt + 6.2401) + 0.000022 * math.sin(575.3385 * ttt + 4.2970) +
		0.000014 * math.sin(1256.6152 * ttt + 6.1969) + 0.000005 * math.sin(606.9777 * ttt + 4.0212) +
		0.000005 * math.sin(52.9691 * ttt + 0.4444) + 0.000002 * math.sin(21.3299 * ttt + 5.5431) +
		0.000010 * ttt * math.sin(628.3076 * ttt + 4.2490)
	)

	tdb_hour = int(tdb / 3600)
	tdb_minute = int(tdb / 60 - tdb_hour * 60)
	tdb_sec = tdb - tdb_minute * 60 - tdb_hour * 3600
	jday_tdb, jfrac_tdb = date2jd(year, mon, day, tdb_hour, tdb_minute, tdb_sec)
	jdtdb = jday_tdb + jfrac_tdb
	ttdb = (jdtdb - 2451545) / 36525

	# TCG
	tcg = tt + 6.969290134e-10 * (jdtai - 2443144.5) * 86400.0
	tcg_hour = int(tcg / 3600)
	tcg_minute = int(tcg / 60 - tcg_hour * 60)
	tcg_sec = tcg - tcg_minute * 60 - tcg_hour * 3600
	jday_tcg, jfrac_tcg = date2jd(year, mon, day, tcg_hour, tcg_minute, tcg_sec)
	jdtcg = jday_tcg + jfrac_tcg
	ttcg = (jdtcg - 2451545) / 36525

	# TCB
	tcb = tdb + 1.55051976772e-8 * (jdtai - 2443144.5) * 86400
	tcb_hour = int(tcb / 3600)
	tcb_minute = int(tcb / 60 - tcb_hour * 60)
	tcb_sec = tcb - tcb_minute * 60 - tcb_hour * 3600
	jday_tcb, jfrac_tcb = date2jd(year, mon, day, tcb_hour, tcb_minute, tcb_sec)
	jdtcb = jday_tcb + jfrac_tcb
	ttcb = (jdtcb - 2451545) / 36525

	time_object = time_obj(
		jday_ut1=jday_ut1,
		jfrac_ut1=jfrac_ut1,
		tut1=tut1,
		jday_tai=jday_tai,
		jfrac_tai=jfrac_tai,
		ttai=ttai,
		jday_tt=jday_tt,
		jfrac_tt=jfrac_tt,
		ttt=ttt,
		jday_tdb=jday_tdb,
		jfrac_tdb=jfrac_tdb,
		ttdb=ttdb,
		jday_tcg=jday_tcg,
		jfrac_tcg=jfrac_tcg,
		ttcg=ttcg,
		jday_tcb=jday_tcb,
		jfrac_tcb=jfrac_tcb,
		ttcb=ttcb,
	)

	return time_object

###################################################################################################
