import os
import sys
import csv


from mpl_toolkits import mplot3d
import numpy as np
import matplotlib.pyplot as plt
import scipy
from stl import mesh

def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	Data Acquisition

# set directories
cwd = os.getcwd()
data_path = 'sensitivity_study/results'
data_dir = os.listdir(os.path.join(cwd, data_path))

# read nfaces
geom_path = 'Data/Inputs/Geometry/Sensitivity_study'
geom_dir = os.listdir(geom_path)

nfaces = []

#	Read Data Files
geom_files = []
for case in geom_dir:
	case_num, case_str, _ = case.split('_', maxsplit=2)
	geom_files.append([int(case_num), case_str, case])

geom_files = sorted(geom_files, key=lambda x: x[0])

plot = 1
fignum = 1
file_save_path = '/home/ldjb/PhD/Reports/Conferences/AMOS_2018/technical_paper/'

for geom_file in geom_files:

	case_num, case_name, geom_file_name = geom_file

	#	Load mesh
	my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file_name))
	casenum, namestr, _ = geom_file_name.split('_')
	# rotate for xaxis orientation
	my_mesh.rotate([0,1,0], (np.pi / 2))

	if plot is 1:
		# Geometry Graphics
		figure = plt.figure(num=None, figsize=(8, 6), facecolor='w', edgecolor='k')
		axes = mplot3d.Axes3D(figure)
		
		axes.add_collection3d(mplot3d.art3d.Poly3DCollection(
			my_mesh.vectors,
			edgecolors='black',
			facecolors='b'
		))
		
		scale = my_mesh.points.flatten(-1)
		axes.auto_scale_xyz(scale, scale, scale)

		axes.set_xlabel('X')
		axes.set_ylabel('Y')
		axes.set_zlabel('Z')
		axes.set_axis_off()

		figure.savefig(os.path.join(file_save_path, 'fig_geom_' + str(fignum) + '.pdf'), bbox_inches='tight')
	
	fignum += 1

print(nfaces)
