#############################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
#############################################################################################
#
#
#
#
#
#
#
#############################################################################################

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d

#	imports

#	standard packages
import os
import sys
import time
import numpy as np

#	numba
import numba

#	transformation
from functions.function_transform import teme2itrf, itrf2orb, site_vec, itrf2razel, \
											rot2, rot3, rot1

#	function list
from functions.function_list import date2jd, jd2date, get_times

#############################################################################################

# @numba.guvectorize(
# 	[(
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:],
# 		numba.float64[:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:]
# 	)],
# 	'(i,a),(i,b),(i,c),(i,d),(i,d),(i,d),(i,d),(e),(f)->(i,d),(i,e),(i,e),(i,d),(i,e),(i,e)'
# 	#	[i, 13], [i, 16], [i, 2], [i, 9], [i, 9], [i, 9], [i, 9], [6], [4] -> [i, 9], [i, 6], [i,
# 	#	6], [i, 9], [i, 6], [i, 6]
# )
def glim_trans(
		data_array,
		eop_data,
		dat_data,
		pn_array,
		pm_array,
		st_array,
		c2t_array,
		site_data,
		time_data,
		out_data_itrf,
		out_data_gcrf_sat,
		out_data_gcrf,
		out_data_orb,
		out_data_orb_sat,
		out_data_azel
	):

	#########################################################################################

	#	initilise

	#	time data
	step_size = data_array[1, 0] - data_array[0, 0]
	jdaytt_start, jfractt_start, jdayut1_start, jfracut1_start = time_data
	jdtt_start = jdaytt_start + jfractt_start
	jdut1_start = jdayut1_start + jfracut1_start

	#	parse site data
	site_itrf = site_data[0:3]
	site_latgd = site_data[3]
	site_lon = site_data[4]
	site_alt = site_data[5]

	#########################################################################################

	#	time loop

	for step, (data_line, eop_line, dat_line, pn_line, pm_line, st_line, c2t_line) in enumerate(
			zip(data_array, eop_data, dat_data, pn_array, pm_array, st_array, c2t_array)):

		#	convert c2t array
		pn = np.array([
			[pn_line[0], pn_line[1], pn_line[2]],
			[pn_line[3], pn_line[4], pn_line[5]],
			[pn_line[6], pn_line[7], pn_line[8]]
		])

		#	inverse c2t array
		inv_pn = np.transpose(pn)

		#	convert c2t array
		pm = np.array([
			[pm_line[0], pm_line[1], pm_line[2]],
			[pm_line[3], pm_line[4], pm_line[5]],
			[pm_line[6], pm_line[7], pm_line[8]]
		])

		#	inverse c2t array
		inv_pm = np.transpose(pm)

		#	convert c2t array
		st = np.array([
			[st_line[0], st_line[1], st_line[2]],
			[st_line[3], st_line[4], st_line[5]],
			[st_line[6], st_line[7], st_line[8]]
		])

		#	inverse c2t array
		inv_st = np.transpose(st)

		#	convert c2t array
		c2t = np.array([
			[c2t_line[0], c2t_line[1], c2t_line[2]],
			[c2t_line[3], c2t_line[4], c2t_line[5]],
			[c2t_line[6], c2t_line[7], c2t_line[8]]
		])

		#	inverse c2t array
		inv_c2t = np.transpose(c2t)

		#	parse inputs
		jdate = data_line[0]
		sat_teme_line = data_line[1:7]
		sun_icrf_line = data_line[7:10]
		earth_icrf_line = data_line[10:13]
	
		#	parse eops
		eop_mjd, eop_year, eop_mon, eop_day, \
		x_p, _, y_p, _, \
		ut_offset, _, \
		lod, _, \
		dX, _, dY, _ = eop_line

		dat = dat_line[1]
	
		#	generate time data
		time_since = step * step_size
	
		jdtt = jdtt_start + time_since
		jdut1 = jdut1_start + time_since
	
		#	as fraction
		if jdtt - np.floor(jdtt) < 0.5:
			jdaytt_epoch = np.floor(jdtt) - 0.5
			jfractt_epoch = jdtt - jdaytt_epoch
	
		elif jdtt - np.floor(jdtt) > 0.5:
			jdaytt_epoch = np.floor(jdtt) + 0.5
			jfractt_epoch = jdtt - jdaytt_epoch
	
		if jdut1 - np.floor(jdut1) < 0.5:
			jdayut1_epoch = np.floor(jdut1) - 0.5
			jfracut1_epoch = jdut1 - jdayut1_epoch
	
		elif jdut1 - np.floor(jdut1) > 0.5:
			jdayut1_epoch = np.floor(jdut1) + 0.5
			jfracut1_epoch = jdut1 - jdayut1_epoch

		#####################################################################################

		#	celestial positions
		
		#	ICRF to GCRF translation
		sun_gcrf_line = sun_icrf_line - earth_icrf_line
		earth_gcrf_line = earth_icrf_line - earth_icrf_line

		#	GCRF to ITRF transformation
		sun_itrf_line = np.dot(c2t, sun_gcrf_line)
		earth_itrf_line = np.dot(c2t, earth_gcrf_line)

		#	site ITRF to GCRF
		site_pef_pos = np.dot(inv_pm, site_itrf)
		temp = np.dot(inv_st, site_pef_pos)
		site_gcrf_line = np.dot(inv_pn, temp)

		#####################################################################################

		#	object position

		#	Perform TEME to ITRF transformation
		sat_teme_pos = sat_teme_line[0:3]
		sat_teme_vel = sat_teme_line[3:6]

		sat_itrf_pos, sat_itrf_vel = teme2itrf(
			sat_teme_pos,
			sat_teme_vel,
			jdayut1_epoch,
			jfracut1_epoch,
			lod,
			x_p,
			y_p
		)

		#	earth rotation
		thetasa= 7.29211514670698e-05 * (1.0  - lod / 86400.0 )
		omegaearth = np.array((0, 0, thetasa))

		#	sat position ITRF to GCRF
		sat_pef_pos = np.dot(inv_pm, sat_itrf_pos)
		temp = np.dot(inv_st, sat_pef_pos)
		sat_gcrf_pos = np.dot(inv_pn, temp)

		#	sat velocity ITRF to GCRF
		sat_pef_vel = np.dot(inv_pm, sat_itrf_vel)
		temp2 = np.cross(omegaearth, sat_pef_pos)
		
		temp = np.dot(inv_st, sat_pef_vel + temp2)
		sat_gcrf_vel = np.dot(inv_pn, temp)

		#####################################################################################

		#	orbital frame
		central_body = sat_gcrf_pos

		#	earth
		transform_body = earth_gcrf_line
		rot_z, rot_y = itrf2orb(central_body, transform_body)
		earth_temp_line = rot2(rot3(np.subtract(transform_body, central_body), rot_z), rot_y)

		#	sun
		transform_body = sun_gcrf_line
		sun_temp_line = rot2(rot3(np.subtract(transform_body, central_body), rot_z), rot_y)

		#	site
		transform_body = site_gcrf_line
		site_temp_line = rot2(rot3(np.subtract(transform_body, central_body), rot_z), rot_y)

		#	sat position and velocity
		transform_body = sat_gcrf_pos
		sat_temp_pos = rot2(rot3(np.subtract(transform_body, central_body), rot_z), rot_y)
		sat_temp_vel = rot2(rot3(sat_gcrf_vel, rot_z), rot_y)

		#	find final x-rotation angle

		#	plane normal
		plane_norm = np.cross(-earth_temp_line, sat_temp_vel)

		normal_yz = [0.0, plane_norm[1], plane_norm[2]]

		#	z unit vector
		z_unit = np.array([0.0, 0.0, 1.0])

		rot_x = np.arccos(
			np.dot(normal_yz, z_unit)
			/ (np.linalg.norm(normal_yz) * np.linalg.norm(z_unit))
		)

		#	rot_x negative check
		if normal_yz[1] > 0:
			rot_x = -rot_x

		#	apply
		earth_orb_line = rot1(earth_temp_line, rot_x)
		sun_orb_line = rot1(sun_temp_line, rot_x)
		site_orb_line = rot1(site_temp_line, rot_x)

		sat_orb_pos = rot1(sat_temp_pos, rot_x)
		sat_orb_vel = rot1(sat_temp_vel, rot_x)

		#####################################################################################

		#	azel
		sat_azel_line = itrf2razel(site_latgd, site_lon, site_alt, sat_itrf_pos)
		sun_azel_line = itrf2razel(site_latgd, site_lon, site_alt, sun_itrf_line)

		#####################################################################################

		#	populate return arrays

		#	ITRF data
		out_data_itrf[step, 0:3] = sat_itrf_pos
		out_data_itrf[step, 3:6] = sat_itrf_vel
		out_data_itrf[step, 6:9] = sun_itrf_line

		#	GCRF data
		out_data_gcrf_sat[step, 0:3] = sat_gcrf_pos
		out_data_gcrf_sat[step, 3:6] = sat_gcrf_vel
		out_data_gcrf[step, 0:3] = sun_gcrf_line
		out_data_gcrf[step, 3:6] = site_gcrf_line

		#	Orb data
		out_data_orb_sat[step, 0:3] = sat_orb_pos
		out_data_orb_sat[step, 3:6] = sat_orb_vel
		out_data_orb[step, 0:3] = sun_orb_line
		out_data_orb[step, 3:6] = earth_orb_line
		out_data_orb[step, 6:9] = site_orb_line

		#	AZEL data
		out_data_azel[step, 0:3] = sat_azel_line
		out_data_azel[step, 3:6] = sun_azel_line


#		###########################################################################################
#		#	INERTIAL FRAME
#		###########################################################################################
#	
#		###########################################################################################
#		#	setup plot data
#
#		#	translated vectors
#		earth_trans_vec = earth_gcrf_line - sat_gcrf_pos
#		sun_trans_vec = sun_gcrf_line - sat_gcrf_pos
#		site_trans_vec = site_gcrf_line - sat_gcrf_pos
#	
#		#	translated unit vectors
#		earth_trans_unit = np.divide(earth_trans_vec, np.linalg.norm(earth_trans_vec))
#		sun_trans_unit = np.divide(sun_trans_vec, np.linalg.norm(sun_trans_vec))
#		site_trans_unit = np.divide(site_trans_vec, np.linalg.norm(site_trans_vec))
#
#		#	sun/site body vectors
#		sun_vec_plot = 4 * (np.vstack(([0, 0, 0], sun_trans_unit)))
#		site_vec_plot = 4 * (np.vstack(([0, 0, 0], site_trans_unit)))
#		earth_vec_plot = 4 * (np.vstack(([0, 0, 0], earth_trans_unit)))
#	
#		#	radial vector in the body frame
#		radial_gcrf_vec = 4 * (np.vstack(([0, 0, 0], -earth_trans_unit)))
#
#		#	velocity vector in the body frame
#		sat_gcrf_vel_unit = np.divide(sat_gcrf_vel, np.linalg.norm(sat_gcrf_vel))
#		vel_gcrf_vec = 4 * (np.vstack(([0, 0, 0], sat_gcrf_vel_unit)))
#
#		#	plane normal
#		plane_norm = np.cross(-earth_trans_unit, sat_gcrf_vel_unit)
#		plane_plot =  4 * (np.vstack(([0, 0, 0], plane_norm)))
#
#		###########################################################################################
#		#	plot graph
#
#		#	generate figure
#		fig = plt.figure(num=1, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
#		ax1 = mplot3d.Axes3D(fig)
#		
#		#	plot observation and illumination vectors
#		ax1.plot(sun_vec_plot[:, 0], sun_vec_plot[:, 1], sun_vec_plot[:, 2], color='r')
#		ax1.plot(site_vec_plot[:, 0], site_vec_plot[:, 1], site_vec_plot[:, 2], color='b')
#		ax1.text(*sun_vec_plot[1, :], 'sun_vec_gcrf')
#		ax1.text(*site_vec_plot[1, :], 'site_vec_gcrf')
#	
#		ax1.plot(radial_gcrf_vec[:, 0], radial_gcrf_vec[:, 1], radial_gcrf_vec[:, 2], color='g')
#		ax1.text(*radial_gcrf_vec[1, :], 'radial_vec_gcrf')
#
#		ax1.plot(vel_gcrf_vec[:, 0], vel_gcrf_vec[:, 1], vel_gcrf_vec[:, 2], color='m')
#		ax1.text(*vel_gcrf_vec[1, :], 'velocity_vec_gcrf')
#	
#		ax1.plot(plane_plot[:, 0], plane_plot[:, 1], plane_plot[:, 2], color='black')
#		ax1.text(*plane_plot[1, :], 'plane normal')
#	
#		ax1.set_xlim([-4, 4])
#		ax1.set_ylim([-4, 4])
#		ax1.set_zlim([-4, 4])
#		ax1.set_aspect('equal')
#		ax1.set_xlabel('x axis')
#		ax1.set_ylabel('y axis')
#		ax1.set_zlabel('z axis')
#		ax1.set_title('GCRF')
#
#		###########################################################################################
#		#	ORBITAL FRAME
#		###########################################################################################
#
#		###########################################################################################
#		#	setup plot data
#
#		#	translated unit vectors
#		sun_orb_unit = np.divide(sun_orb_line, np.linalg.norm(sun_orb_line))
#		earth_orb_unit = np.divide(earth_orb_line, np.linalg.norm(earth_orb_line))
#		site_orb_unit = np.divide(site_orb_line, np.linalg.norm(site_orb_line))
#
#		#	sun/site body vectors
#		sun_vec_plot = 4 * (np.vstack(([0, 0, 0], sun_orb_unit)))
#		site_vec_plot = 4 * (np.vstack(([0, 0, 0], site_orb_unit)))
#		earth_vec_plot = 4 * (np.vstack(([0, 0, 0], earth_orb_unit)))
#	
#		#	radial vector in the body frame
#		radial_orb_vec = 4 * (np.vstack(([0, 0, 0], -earth_orb_unit)))
#
#		#	velocity vector in the body frame
#		sat_orb_vel_unit = np.divide(sat_orb_vel, np.linalg.norm(sat_orb_vel))
#		vel_orb_vec = 4 * (np.vstack(([0, 0, 0], sat_orb_vel_unit)))
#
#		#	plane normal
#		plane_norm = np.cross(-earth_orb_unit, sat_orb_vel_unit)
#		plane_plot =  4 * (np.vstack(([0, 0, 0], plane_norm)))
#
#		###########################################################################################
#		#	plot graph
#
#		#	generate figure
#		fig = plt.figure(num=3, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
#		ax1 = mplot3d.Axes3D(fig)
#		
#		#	plot observation and illumination vectors
#		ax1.plot(sun_vec_plot[:, 0], sun_vec_plot[:, 1], sun_vec_plot[:, 2], color='r')
#		ax1.plot(site_vec_plot[:, 0], site_vec_plot[:, 1], site_vec_plot[:, 2], color='b')
#		ax1.text(*sun_vec_plot[1, :], 'sun_vec_orb')
#		ax1.text(*site_vec_plot[1, :], 'site_vec_orb')
#	
#		ax1.plot(radial_orb_vec[:, 0], radial_orb_vec[:, 1], radial_orb_vec[:, 2], color='g')
#		ax1.text(*radial_orb_vec[1, :], 'radial_vec_orb')
#
#		ax1.plot(vel_orb_vec[:, 0], vel_orb_vec[:, 1], vel_orb_vec[:, 2], color='m')
#		ax1.text(*vel_orb_vec[1, :], 'velocity_vec_orb')
#
#		ax1.plot(plane_plot[:, 0], plane_plot[:, 1], plane_plot[:, 2], color='black')
#		ax1.text(*plane_plot[1, :], 'plane normal')
#	
#		ax1.set_xlim([-4, 4])
#		ax1.set_ylim([-4, 4])
#		ax1.set_zlim([-4, 4])
#		ax1.set_aspect('equal')
#		ax1.set_xlabel('x axis')
#		ax1.set_ylabel('y axis')
#		ax1.set_zlabel('z axis')
#		ax1.set_title('Orbital Frame')
#
#		plt.show()
