import os
import sys
import csv

from pprint import pprint

import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import signal

from stl import mesh
import math

from tools.domain_splitting import cheb_app
from tools.face_counter import face_count

###################################################################################################

def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	Outputs

#	case designation string
# case_desig = 'bdrf_sd'
# case_desig = 'bdrf_omega'
# case_desig = 'bdrf_m'

case_desig = 'attitude_motion'
case_desig = 'attitude'

case_desig2 = '_d0.5'
case_desig2 = '_d0.0'
case_desig2 = '_d1.0'

norad_id = 25885

# base_file = '20samp-attit-mot0-x'
# base_file = '20samp-d-bdrf1.0'
# base_file = '20samp-omega-bdrf0.1'
# base_file = '20samp-m-bdrf0.3'

base_file = 'div15-attit-mot0-x'
base_file = 'div14-attit0-x'

#	directories
# case_path = '../../../sensitivity_study_archive/light_curves/' + case_desig + case_desig2
case_path = os.path.join('sensitivity_study_newdata/light_curves', case_desig + case_desig2, '25885')
case_path = os.path.join(
	'/home/ldjb/codebase/sensitivity_study_newdata/light_curves',
	case_desig + case_desig2,
	str(norad_id)
)

# case_path_out = '../../../sensitivity_study_archive/distances/' + case_desig + case_desig2
case_path_out = '/home/ldjb/codebase/sensitivity_study_newdata/distances/' + case_desig + case_desig2

###################################################################################################

#	Inputs

#	set light curve directories
cwd = os.getcwd()
case_dir = os.listdir(os.path.join(cwd, case_path))

face_file = 'nfaces_file.csv'

# geom_path = 'Data/Inputs/Geometry/Sensitivity_study'
# geom_path = '/home/ldjb/codebase/data_archive/LCG_data/Inputs/Geometry/Sensitivity_study'
geom_path = '/home/ldjb/codebase/sensitivity_study_newdata/Geometry/new_geoms'
face_path = '/home/ldjb/codebase/git_dir/python/LCG/sensitivity_study'

if os.path.isfile(os.path.join(face_path, face_file)):
	nfaces = np.genfromtxt(os.path.join(face_path, face_file), delimiter=',')
else:
	#	set geometry directory
	geom_dir = os.listdir(geom_path)
	
	#	Read and sort geometry files
	geom_files = []
	for case in geom_dir:
		case_num, case_str, _ = case.split('_', maxsplit=2)
		geom_files.append([int(case_num), case_str, case])
	
	geom_files = sorted(geom_files, key=lambda x: x[0])
	
	#	read geometry files for face numbers
	nfaces = np.zeros([len(geom_files)])
	for i, geom_case in enumerate(geom_files):
	
		geom_file = geom_case[2]
	
		#	Load mesh
		my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
		#	count faces
		nfaces[i] = face_count(my_mesh.normals)
	
	np.savetxt(os.path.join(face_path, face_file), nfaces, delimiter=',')

	
#	number of geometry files
ngeom = len(nfaces)


#	Read and sort data files
case_data = []
for case in case_dir:
	case_num, case_str, _ = case.split('_', maxsplit=2)
	case_data.append([int(case_num), case_str, case])

case_data = sorted(case_data, key=lambda x: x[0])
num_cases = len(case_data)

###################################################################################################

#	get base case

for case_line in case_data:
	if case_line[1] == base_file:
		base_case = case_data[10]
		base_case_num = case_line[0]
		base_case_name = case_line[1]
		base_case_file = case_line[2]
		break

print('Using base case: ', base_case_name)
input('\nContinue with this base file?\n')

#	signal parameters
base_case_data = np.genfromtxt(os.path.join(case_path, base_case_file), delimiter=',')
base_signal = base_case_data[:, 2]

#	convert infinite values
if np.isinf(base_signal).any():
 	for i, case_line in enumerate(base_signal):
 		if np.isinf(case_line):
 			base_signal[i] = 30

#	space
npts = np.arange(0, len(base_signal), 1)

# approximate base signal
cheb_base_signal, cheb_base_domain, rms = cheb_app(base_signal)

###################################################################################################

previous_case = 'empty'
previous_case_file = 'empty'
previous_case_str = 'empty'
first_flag = 1


for case_itt, case in enumerate(case_data):

	#	set case data
	case_num = case[0]
	case_name = case[1]
	case_file = case[2]

	#	read case data
	case_data = np.genfromtxt(os.path.join(case_path, case_file), delimiter=',')
	case_signal = case_data[:, 2]
	geom_str, case_str = case_name.split('-', maxsplit=1)

	if case_str != previous_case_str:
		print('\n\nNew attitude case: ', case_str)
		previous_case_str = case_str
	
	###############################################################################################
	#	NAN value correction

	if np.isnan(case_signal).any():
		print('\nFound nan values.')

		#	number of nan occurances
		nnans = np.isnan(case_signal).sum()
		print('{:d} nans found.'.format(nnans))

		#	ensure nan occurances are sequential
		nan_indices = np.argwhere(np.isnan(case_signal))
		nan_current = nan_indices[0]
		nan_others = nan_indices[1:]
		sequent = True

		for nan_index in nan_others:
			if nan_index != (nan_current + 1):
				sequent = False
				break

			nan_current = nan_index

		#	correction
		if sequent is False:
			print('NAN values are not sequential! Exiting..')

		elif sequent is True:
			print('nans are sequential, correcting..')

			#	indices of surrounding non-NAN values
			preval = case_signal[nan_indices[0] - 1]
			postval = case_signal[nan_indices[-1] + 1]

			#	define correction array
			correction_array = np.zeros((nnans + 2))
			correction_array[0] = preval
			correction_array[-1] = postval

			#	values to interpolate
			x = np.arange(1, len(correction_array) - 1, 1)

			#	known values
			xp = np.array([0, len(correction_array) - 1])
			fp = np.array([preval, postval])[:, 0]

			#	perform interpolation
			interp_data = np.interp(x, xp, fp)

			print('Interpolated data = ', interp_data)
			print('Interpolated data shape = ', interp_data.shape)

			#	correct case_signal array
			start_index = nan_indices[0][0]
			end_index = nan_indices[-1][0]

			#	reshape interp_data
			interp_data = np.reshape(interp_data, [len(interp_data), 1])

			#	replace nans
			case_signal[nan_indices] = interp_data

			print('nans replaced.')

		#	confirm nan values are removed
		if np.isnan(case_signal).any():
			print('ERROR: nan values still found')
		else:
			print('\nCONFIRMED: nans corrected')
	
	###############################################################################################
	#	

	#	output file name
	case_file_name = 'distance_' + case_str + '_case.csv'
	print('\n	Measuring distance from:	{}\
			\n	To:				{}'.format(geom_str + '-' + case_str, base_file))

	#	run continuation
	if case_str == previous_case:

		#	convert infinite values
		if np.isinf(case_signal).any():
			for i, case_line in enumerate(case_signal):
				if np.isinf(case_line):
					case_signal[i] = 30
		
		#	perform approximation
		cheb_signal, cheb_domain, rms = cheb_app(case_signal)

		#	calcualte correlation
		chisq, p_value = scipy.stats.chisquare(case_signal, cheb_signal)
#		print('	rms = ', rms_calc(case_signal, cheb_signal))
#		print('	chisq =	', chisq)
#		print('	p_value =	', p_value)
			
		# distance
		temp = (np.subtract(cheb_signal, cheb_base_signal))**2

		distances[geom_num - 1] = np.trapz(temp, npts)
		print('	Distance = ', distances[geom_num - 1])

	else:
		#	new run

		if first_flag is 1:
			#	initilize an empty distance array
			distances = np.zeros([ngeom])
		else:
			print('\n	Saving previous run')
			#	save previous run:
			np.savetxt(os.path.join(cwd, case_path_out, previous_case_file), distances, delimiter=',')
			distances = np.zeros([ngeom])

		geom_num = 1

		#	convert infinite values
		if np.isinf(case_signal).any():
			for i, case_line in enumerate(case_signal):
				if np.isinf(case_line):
					case_signal[i] = 30
		
		#	perform approximation
		cheb_signal, cheb_domain, rms = cheb_app(case_signal)

		#	calcualte correlation
		chisq, p_value = scipy.stats.chisquare(case_signal, cheb_signal)
#		print('	rms = ', rms_calc(case_signal, cheb_signal))
#		print('	chisq =	', chisq)
#		print('	p_value =	', p_value)
			
		# distance
		temp = (np.subtract(cheb_signal, cheb_base_signal))**2
		distances[geom_num - 1] = np.trapz(temp, npts)
		print('	Distance = ', distances[geom_num - 1])


	geom_num += 1
	previous_case = case_str
	previous_case_file = case_file_name

	first_flag = 0

	if case_itt is num_cases - 1:

		print(distances)
		print('end')
		np.savetxt(os.path.join(cwd, case_path_out, previous_case_file), distances, delimiter=',')
