import os
import sys
import csv
import numpy as np
import matplotlib.pyplot as plt

#############################################################################################

# signal parameters

# space
npts = 800
#npts = len(data_signal)

lowerbound = 0
L = npts / 2

xvals = np.linspace(lowerbound, (2 * L), npts)

# nterms
klower = 1
kupper = 51
spacing = 1
kvals = np.arange(klower, kupper, spacing)

print('\nHilbert Fourier Analysis Example.')
print(
	'\nFor a space of {:d} equally spaced points from {:f} to {:f}.'.format(
		len(xvals),
		xvals[0],
		xvals[-1]
	)
)
print('\nUsing k values from {:d} to {:d}.'.format(int(kvals[0]), int(kvals[-1])))

###################################################################################################

# set directories
cwd = os.getcwd()
slc_path = 'Data/Outputs/Light_curves/ss_light_curves/test1/25885'
slc_dir = os.listdir(os.path.join(cwd, slc_path))

slc_files = []

for slc in slc_dir:
	case_id = slc.split('_', maxsplit=1)

	slc_files.append([int(case_id[0]), slc])

slc_files = sorted(slc_files, key=lambda x: x[0])
num_cases = len(slc_files)

signal1_distance = np.zeros((num_cases))
cases = np.arange(1, num_cases + 1, 1)

diff_data1 = []

# get synthetic lightcurve
for itteration, slc in enumerate(slc_files):

	# get slc data
	slc = slc[1]
	slc_data = np.genfromtxt(os.path.join(slc_path, slc), delimiter=',')
	case_num = np.int(slc.split('_')[0])

	if case_num is 1:

		print('Light curve generated from case: {:d}\n '.format(case_num))
	
		# fix infinite magnitudes
		for index, slc_line in enumerate(slc_data):
			mag = slc_line[2]
			if np.isinf(mag):
				slc_data[index, 2] = 0
	
		jdate = slc_data[0, 0]
		signal = slc_data[:, 2]
	
		# Fourier Analysis
		npts = len(signal)
		L = npts / 2
		
		xvals = np.linspace(lowerbound, (2 * L), npts)
	
		# initilization
		ak_signal = np.zeros((len(xvals)))
		bk_signal = np.zeros((len(xvals)))
		app_signal = np.zeros((len(xvals)))
		tot_signal = np.zeros((len(xvals)))
		
		ak_vals = np.zeros((len(kvals)))
		bk_vals = np.zeros((len(kvals)))
	
		# integrate signals wrt x
		a0_val = (1 / L) * np.trapz(signal, xvals)
		tot_signal.fill(a0_val / 2)
	
		print('Generating Fourier Signal. . .')
		# generate integrable signals
		for n, k in enumerate(kvals):
		
			# Calculate Fourier Coefficients
			for i, x in enumerate(xvals):
				ak_signal[i] = signal[i] * np.cos(k * x * np.pi / L)
				bk_signal[i] = signal[i] * np.sin(k * x * np.pi / L)
		
			ak_val = (1 / L) * np.trapz(ak_signal, xvals)
			bk_val = (1 / L) * np.trapz(bk_signal, xvals)
		
			# Calculate Approximate Signal
			for i, x in enumerate(xvals):
		
				app_signal[i] = (
					(ak_val * np.cos(k * x * np.pi / L))
					+ (bk_val * np.sin(k * x * np.pi / L))
				)
		
			# Sum for total signal
			tot_signal = np.add(tot_signal, app_signal)
	
		print('Complete.')
		print('Baseline is set.')
		# save as baseline
		base_line = tot_signal

		# normed distance
		int_signal = (base_line - base_line)**2
		base_distance = np.sqrt((1 / L) * np.trapz(int_signal, xvals))
		signal1_distance[itteration] = base_distance

		diff_data1.append(int_signal)

	elif 1 < case_num:

		print('Light curve generated from case: {:d}\n '.format(case_num))
	
		# fix infinite magnitudes
		for index, slc_line in enumerate(slc_data):
			mag = slc_line[2]
			if np.isinf(mag):
				slc_data[index, 2] = 0
	
		jdate = slc_data[0, 0]
		signal = slc_data[:, 2]
	
		# Fourier Analysis
		npts = len(signal)
		L = npts / 2
		
		xvals = np.linspace(lowerbound, (2 * L), npts)
	
		# initilization
		ak_signal = np.zeros((len(xvals)))
		bk_signal = np.zeros((len(xvals)))
		app_signal = np.zeros((len(xvals)))
		tot_signal = np.zeros((len(xvals)))
		
		ak_vals = np.zeros((len(kvals)))
		bk_vals = np.zeros((len(kvals)))
	
		# integrate signals wrt x
		a0_val = (1 / L) * np.trapz(signal, xvals)
		tot_signal.fill(a0_val / 2)
	
		print('Generating Fourier Signal. . .')
		# generate integrable signals
		for n, k in enumerate(kvals):
		
			# Calculate Fourier Coefficients
			for i, x in enumerate(xvals):
				ak_signal[i] = signal[i] * np.cos(k * x * np.pi / L)
				bk_signal[i] = signal[i] * np.sin(k * x * np.pi / L)
		
			ak_val = (1 / L) * np.trapz(ak_signal, xvals)
			bk_val = (1 / L) * np.trapz(bk_signal, xvals)
		
			# Calculate Approximate Signal
			for i, x in enumerate(xvals):
		
				app_signal[i] = (
					(ak_val * np.cos(k * x * np.pi / L))
					+ (bk_val * np.sin(k * x * np.pi / L))
				)
		
			# Sum for total signal
			tot_signal = np.add(tot_signal, app_signal)
	
		print('Complete.')
		# normed distance
		int_signal = (np.subtract(tot_signal, base_line))**2
		signal1_distance[itteration] = np.sqrt((1 / L) * np.trapz(int_signal, xvals))

		diff_data1.append(int_signal)
		print('integrated signal = ', np.trapz(int_signal, xvals))
		print('signal_distance = ', signal1_distance[itteration])

###################################################################################################

slc_path = 'Data/Outputs/Light_curves/ss_light_curves/test2/25885'
slc_dir = os.listdir(os.path.join(cwd, slc_path))

slc_files = []

for slc in slc_dir:
	case_id = slc.split('_', maxsplit=1)

	slc_files.append([int(case_id[0]), slc])

slc_files = sorted(slc_files, key=lambda x: x[0])
num_cases = len(slc_files)

signal2_distance = np.zeros((num_cases))
cases = np.arange(1, num_cases + 1, 1)

diff_data1 = []

# get synthetic lightcurve
for itteration, slc in enumerate(slc_files):

	# get slc data
	slc = slc[1]
	slc_data = np.genfromtxt(os.path.join(slc_path, slc), delimiter=',')
	case_num = np.int(slc.split('_')[0])

	if case_num is 1:

		print('Light curve generated from case: {:d}\n '.format(case_num))
	
		# fix infinite magnitudes
		for index, slc_line in enumerate(slc_data):
			mag = slc_line[2]
			if np.isinf(mag):
				slc_data[index, 2] = 0
	
		jdate = slc_data[0, 0]
		signal = slc_data[:, 2]
	
		# Fourier Analysis
		npts = len(signal)
		L = npts / 2
		
		xvals = np.linspace(lowerbound, (2 * L), npts)
	
		# initilization
		ak_signal = np.zeros((len(xvals)))
		bk_signal = np.zeros((len(xvals)))
		app_signal = np.zeros((len(xvals)))
		tot_signal = np.zeros((len(xvals)))
		
		ak_vals = np.zeros((len(kvals)))
		bk_vals = np.zeros((len(kvals)))
	
		# integrate signals wrt x
		a0_val = (1 / L) * np.trapz(signal, xvals)
		tot_signal.fill(a0_val / 2)
	
		print('Generating Fourier Signal. . .')
		# generate integrable signals
		for n, k in enumerate(kvals):
		
			# Calculate Fourier Coefficients
			for i, x in enumerate(xvals):
				ak_signal[i] = signal[i] * np.cos(k * x * np.pi / L)
				bk_signal[i] = signal[i] * np.sin(k * x * np.pi / L)
		
			ak_val = (1 / L) * np.trapz(ak_signal, xvals)
			bk_val = (1 / L) * np.trapz(bk_signal, xvals)
		
			# Calculate Approximate Signal
			for i, x in enumerate(xvals):
		
				app_signal[i] = (
					(ak_val * np.cos(k * x * np.pi / L))
					+ (bk_val * np.sin(k * x * np.pi / L))
				)
		
			# Sum for total signal
			tot_signal = np.add(tot_signal, app_signal)
	
		print('Complete.')
		print('Baseline is set.')
		# save as baseline
		base_line = tot_signal

		# normed distance
		int_signal = (base_line - base_line)**2
		base_distance = np.sqrt((1 / L) * np.trapz(int_signal, xvals))
		signal2_distance[itteration] = base_distance

		diff_data1.append(int_signal)

	elif 1 < case_num:

		print('Light curve generated from case: {:d}\n '.format(case_num))
	
		# fix infinite magnitudes
		for index, slc_line in enumerate(slc_data):
			mag = slc_line[2]
			if np.isinf(mag):
				slc_data[index, 2] = 0
	
		jdate = slc_data[0, 0]
		signal = slc_data[:, 2]
	
		# Fourier Analysis
		npts = len(signal)
		L = npts / 2
		
		xvals = np.linspace(lowerbound, (2 * L), npts)
	
		# initilization
		ak_signal = np.zeros((len(xvals)))
		bk_signal = np.zeros((len(xvals)))
		app_signal = np.zeros((len(xvals)))
		tot_signal = np.zeros((len(xvals)))
		
		ak_vals = np.zeros((len(kvals)))
		bk_vals = np.zeros((len(kvals)))
	
		# integrate signals wrt x
		a0_val = (1 / L) * np.trapz(signal, xvals)
		tot_signal.fill(a0_val / 2)
	
		print('Generating Fourier Signal. . .')
		# generate integrable signals
		for n, k in enumerate(kvals):
		
			# Calculate Fourier Coefficients
			for i, x in enumerate(xvals):
				ak_signal[i] = signal[i] * np.cos(k * x * np.pi / L)
				bk_signal[i] = signal[i] * np.sin(k * x * np.pi / L)
		
			ak_val = (1 / L) * np.trapz(ak_signal, xvals)
			bk_val = (1 / L) * np.trapz(bk_signal, xvals)
		
			# Calculate Approximate Signal
			for i, x in enumerate(xvals):
		
				app_signal[i] = (
					(ak_val * np.cos(k * x * np.pi / L))
					+ (bk_val * np.sin(k * x * np.pi / L))
				)
		
			# Sum for total signal
			tot_signal = np.add(tot_signal, app_signal)
	
		print('Complete.')
		# normed distance
		int_signal = (np.subtract(tot_signal, base_line))**2
		signal2_distance[itteration] = np.sqrt((1 / L) * np.trapz(int_signal, xvals))

		diff_data1.append(int_signal)
		print('integrated signal = ', np.trapz(int_signal, xvals))
		print('signal2_distance = ', signal2_distance[itteration])
	
#for int_signal in diff_data:
#	plt.figure()
#	plt.plot(xvals, int_signal, label='int_signal')
#	plt.legend()
##	plt.pause(1)

plt.figure(3)
plt.xlabel('Case Number')
plt.ylabel('Fourier Space Distance')
plt.title('X-Axis Rotation Case Study.')
plt.grid(True)
plt.scatter(cases, signal1_distance)
plt.scatter(cases, signal2_distance)
plt.show(plt.figure(3))
