#############################################################################################
#
#   LIGHT CURVE GENERATION FRONT END
#
#############################################################################################
import os
import sys
import time

import matplotlib.pyplot as plt
import numpy as np

from fred_file import fred_ss
from setup import fred_setup, object_gen

from pprint import pprint

from multiprocessing import Pool
from subprocess import call
#############################################################################################

#	Input

# case_desig = 'attitude_d0.5'
case_desig = 'attitude_d0.0'
case_desig = 'attitude_motion_d1.0'
case_desig = 'attitude_motion_d0.0'
case_desig = 'attitude_d1.0'

#	free parameter: propagation timestep
prop_step = 0.1

#	set input directories
cwd = os.getcwd()
root_path = '/home/ldjb/codebase'
archive_path = 'data_archive/LCG_data'
path_data = os.path.join(root_path, archive_path)

#	set input casepath
case_path = os.path.join(root_path, 'sensitivity_study_newdata/case_files', case_desig)
case_dir = os.listdir(case_path)

#	set output directory
lightcurve_path = os.path.join(root_path, 'sensitivity_study_newdata/light_curves', case_desig)

#############################################################################################

#	clear terminal
call(['clear'])
print('Started.')

#	initilise variables
object_list = []
flist_horz = []
flist_tle = []
flist_lc = []
bdrf_list = []

print('Generating case objects...')
for case_file in case_dir:

	#	generatate case data from case file
	object_data, fname_horz, fname_tle, fname_lc, bdrf_data =  object_gen(
		case_file,
		case_path,
		lightcurve_path,
		root_path,
		archive_path,
		prop_step
	)

	if object_data is 0:
		print('Excluded: Beyond the end of 2017')
	else:
		object_list.append(object_data)
		flist_horz.append(fname_horz)
		flist_tle.append(fname_tle)
		flist_lc.append(fname_lc)
		bdrf_list.append(bdrf_data)

print('{:d} case objects generated.'.format(len(object_list)))

#############################################################################################

#	perform setup
print('Performing setup...')
horz_files = 0
tle_files = 0

for i in range(len(object_list)):

	inputs = [object_list[i],
		flist_horz[i],
		flist_tle[i],
		root_path,
		archive_path
	]

	horz_files_gen, tle_files_gen = fred_setup(inputs)

	tle_files = tle_files + tle_files_gen
	horz_files = horz_files + horz_files_gen

print('	{} tle files generated.'.format(tle_files))
print('	{} horz files generated.'.format(horz_files))

#############################################################################################

input('\nInitilisation Complete. Press ENTER to begin simulations')

sim_start = time.time()

#	generate parellelised inputs
inputs = []
overwrite = 0

for i in range(len(object_list)):
	inputs.append([
		object_list[i],
		flist_tle[i],
		flist_horz[i],
		flist_lc[i],
		lightcurve_path,
		root_path,
		archive_path,
		overwrite
	])

para = False
ncores = 8

inputs = zip(inputs, bdrf_list)

if para is True:
	#	initilise pool
	p = Pool(ncores)
	p.starmap(fred_ss, inputs)

elif para is False:

	for inps in inputs:

		inp, bdrf_data = inps
		light_curve = fred_ss(inp, bdrf_data)
	
		if light_curve is 0:
			print('No Lightcurve Generation')
		else:

			time_data = np.zeros([len(light_curve)])
			for i, time_line in enumerate(light_curve[:, 0]):
				time_data[i] = 86400 * (light_curve[i, 0] - light_curve[0, 0])

			plt.plot(time_data, light_curve[:,2])
			plt.axis([0, time_data[-1], 12, 0])
			plt.grid()
			plt.show()

			sys.exit()

#############################################################################################

print('\nSimulation Complete!\n')
sim_end = time.time()
sim_time = (sim_end - sim_start) / 60

print('Simulation time:	{} mins'.format(sim_time))
print('Number of Cores:	{}'.format(ncores))
