#############################################################################################
import os
import sys
import csv
import numpy as np
from matplotlib import pyplot
from pprint import pprint


#############################################################################################
# free parameter: propagation timestep
prop_step = 0.1
# set directories
cwd = os.getcwd()

#############################################################################################
#############################################################################################
#############################################################################################

# 32263

case_path = 'Data/source/MMT/GLOBALSTAR/light_curves/32263'
slc_path = 'Data/Outputs/Light_curves/32263'
case_dir = os.listdir(os.path.join(cwd, case_path))
slc_dir = os.listdir(os.path.join(cwd, slc_path))

print('Started on path: ', slc_path)

#############################################################################################
var_data = []

# get synthetic lightcurve
for slc in slc_dir:

	# get slc data
	slc_data = np.genfromtxt(os.path.join(slc_path, slc), delimiter=',')

	track_id = np.int(slc.split('_')[0])

	# find matching real light curve
	for case in case_dir:

		track_id_case = np.int(case.split('_')[0])

		if track_id_case == track_id:

			# get true mag data
			case_data = np.genfromtxt(os.path.join(case_path, case), delimiter=',')
			break

	for index, slc_line in enumerate(slc_data):
		mag = slc_line[2]
		if np.isinf(mag):
			slc_data[index, 2] = 0


	case_avg_mag = np.mean(case_data[:,7])
	case_std = np.std(case_data[:,7])

	slc_avg_mag = np.mean(slc_data[:,2])
	slc_std = np.std(slc_data[:,2])

	jdate = slc_data[0, 0]
	var_data.append([jdate, case_avg_mag, case_std, slc_avg_mag, slc_std])

var_data = np.array(var_data).astype(np.float)
var_data = var_data[var_data[:,0].argsort()]

#############################################################################################

# moving average date

N = 3
cumsum, maves_date = [0], []
for i, x in enumerate(var_data[:,0], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_date.append(moving_ave)
maves_date = np.array(maves_date).astype(np.float)
maves_date = np.reshape(maves_date, (len(maves_date), 1))

#############################################################################################

# moving average of averages 1

N = 3
cumsum, maves_ave1 = [0], []
for i, x in enumerate(var_data[:,1], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_ave1.append(moving_ave)
maves_ave1 = np.array(maves_ave1).astype(np.float)
maves_ave1 = np.reshape(maves_ave1, (len(maves_ave1), 1))
ave1_trend63 = np.append(maves_date, maves_ave1, axis = 1)

#############################################################################################

# moving average of averages 2

N = 3
cumsum, maves_ave2 = [0], []
for i, x in enumerate(var_data[:,3], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_ave2.append(moving_ave)
maves_ave2 = np.array(maves_ave2).astype(np.float)
maves_ave2 = np.reshape(maves_ave2, (len(maves_ave2), 1))
ave2_trend63 = np.append(maves_date, maves_ave2, axis = 1)

#############################################################################################

# moving average of standard deviations 1

N = 3
cumsum, maves_std1 = [0], []
for i, x in enumerate(var_data[:,2], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std1.append(moving_ave)
maves_std1 = np.array(maves_std1).astype(np.float)
maves_std1 = np.reshape(maves_std1, (len(maves_std1), 1))
std1_trend63 = np.append(maves_date, maves_std1, axis = 1)

#############################################################################################

# moving average of standard deviations 2

N = 3
cumsum, maves_std2 = [0], []
for i, x in enumerate(var_data[:,4], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std2.append(moving_ave)
maves_std2 = np.array(maves_std2).astype(np.float)
maves_std2 = np.reshape(maves_std2, (len(maves_std2), 1))
std2_trend63 = np.append(maves_date, maves_std2, axis = 1)

#############################################################################################
#############################################################################################

# PLOT: FIGURE 1 - AVERAGE MAG 32263

pyplot.figure(1)
pyplot.scatter(var_data[:,0], var_data[:,1], s=20, c='r', label="Real")
pyplot.scatter(var_data[:,0], var_data[:,3], s=20, c='g', label="Synthetic")

pyplot.plot(ave1_trend63[:,0], ave1_trend63[:,1], 'r', label="Real")
pyplot.plot(ave2_trend63[:,0], ave2_trend63[:,1], 'g', label="Synthetic")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Mean Magnitude')
pyplot.title('Average Magnitudes for: 32263')
pyplot.grid(True)
pyplot.axis([var_data[0,0], var_data[-1,0], 12, 0])
pyplot.legend()

#############################################################################################
#############################################################################################

# PLOT: FIGURE 2 - STD DEV 32263

pyplot.figure(2)
pyplot.scatter(var_data[:,0], var_data[:,2], s=20, c='r', label="Real")
pyplot.scatter(var_data[:,0], var_data[:,4], s=20, c='g', label="Synthetic")

pyplot.plot(std1_trend63[:,0], std1_trend63[:,1], 'r', label="Real")
pyplot.plot(std2_trend63[:,0], std2_trend63[:,1], 'g', label="Synthetic")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Standard Deviation')
pyplot.title('Standard Deviations for: 32263')
pyplot.grid(True)
pyplot.axis([var_data[0,0], var_data[-1,0], 0, 2])
pyplot.legend()

#############################################################################################
#############################################################################################
#############################################################################################

# 32264

case_path = 'Data/source/MMT/GLOBALSTAR/light_curves/32264'
slc_path = 'Data/Outputs/Light_curves/32264'
case_dir = os.listdir(os.path.join(cwd, case_path))
slc_dir = os.listdir(os.path.join(cwd, slc_path))

print('Started on path: ', slc_path)

#############################################################################################
var_data = []

# get synthetic lightcurve
for slc in slc_dir:

	# get slc data
	slc_data = np.genfromtxt(os.path.join(slc_path, slc), delimiter=',')

	track_id = np.int(slc.split('_')[0])

	# find matching real light curve
	for case in case_dir:

		track_id_case = np.int(case.split('_')[0])

		if track_id_case == track_id:

			# get true mag data
			case_data = np.genfromtxt(os.path.join(case_path, case), delimiter=',')
			break

	for index, slc_line in enumerate(slc_data):
		mag = slc_line[2]
		if np.isinf(mag):
			slc_data[index, 2] = 0


	case_avg_mag = np.mean(case_data[:,7])
	case_std = np.std(case_data[:,7])

	slc_avg_mag = np.mean(slc_data[:,2])
	slc_std = np.std(slc_data[:,2])

	jdate = slc_data[0, 0]
	var_data.append([jdate, case_avg_mag, case_std, slc_avg_mag, slc_std])

var_data = np.array(var_data).astype(np.float)
var_data = var_data[var_data[:,0].argsort()]

#############################################################################################

# moving average date

N = 3
cumsum, maves_date = [0], []
for i, x in enumerate(var_data[:,0], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_date.append(moving_ave)
maves_date = np.array(maves_date).astype(np.float)
maves_date = np.reshape(maves_date, (len(maves_date), 1))

#############################################################################################

# moving average of averages 1

N = 3
cumsum, maves_ave1 = [0], []
for i, x in enumerate(var_data[:,1], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_ave1.append(moving_ave)
maves_ave1 = np.array(maves_ave1).astype(np.float)
maves_ave1 = np.reshape(maves_ave1, (len(maves_ave1), 1))
ave1_trend64 = np.append(maves_date, maves_ave1, axis = 1)

#############################################################################################

# moving average of averages 2

N = 3
cumsum, maves_ave2 = [0], []
for i, x in enumerate(var_data[:,3], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_ave2.append(moving_ave)
maves_ave2 = np.array(maves_ave2).astype(np.float)
maves_ave2 = np.reshape(maves_ave2, (len(maves_ave2), 1))
ave2_trend64 = np.append(maves_date, maves_ave2, axis = 1)

#############################################################################################

# moving average of standard deviations 1

N = 3
cumsum, maves_std1 = [0], []
for i, x in enumerate(var_data[:,2], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std1.append(moving_ave)
maves_std1 = np.array(maves_std1).astype(np.float)
maves_std1 = np.reshape(maves_std1, (len(maves_std1), 1))
std1_trend64 = np.append(maves_date, maves_std1, axis = 1)

#############################################################################################

# moving average of standard deviations 2

N = 3
cumsum, maves_std2 = [0], []
for i, x in enumerate(var_data[:,4], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std2.append(moving_ave)
maves_std2 = np.array(maves_std2).astype(np.float)
maves_std2 = np.reshape(maves_std2, (len(maves_std2), 1))
std2_trend64 = np.append(maves_date, maves_std2, axis = 1)

#############################################################################################
#############################################################################################

# PLOT: FIGURE 3 - AVERAGE MAG 32264

pyplot.figure(3)
pyplot.scatter(var_data[:,0], var_data[:,1], s=20, c='r', label="Real")
pyplot.scatter(var_data[:,0], var_data[:,3], s=20, c='g', label="Synthetic")

pyplot.plot(ave1_trend64[:,0], ave1_trend64[:,1], 'r', label="Real")
pyplot.plot(ave2_trend64[:,0], ave2_trend64[:,1], 'g', label="Synthetic")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Mean Magnitude')
pyplot.title('Average Magnitudes for: 32264')
pyplot.grid(True)
#pyplot.savefig("test.png")
pyplot.axis([var_data[0,0], var_data[-1,0], 12, 0])
pyplot.legend()


#############################################################################################
#############################################################################################

# PLOT: FIGURE 4 - STD DEV 32264

pyplot.figure(4)
pyplot.scatter(var_data[:,0], var_data[:,2], s=20, c='r', label="Real")
pyplot.scatter(var_data[:,0], var_data[:,4], s=20, c='g', label="Synthetic")

pyplot.plot(std1_trend64[:,0], std1_trend64[:,1], 'r', label="Real")
pyplot.plot(std2_trend64[:,0], std2_trend64[:,1], 'g', label="Synthetic")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Standard Deviation')
pyplot.title('Standard Deviations for: 32264')
pyplot.grid(True)
#pyplot.savefig("test.png")
pyplot.axis([var_data[0,0], var_data[-1,0], 0, 2])
pyplot.legend()


#############################################################################################
#############################################################################################
#############################################################################################

# 25885

case_path = 'Data/source/MMT/GLOBALSTAR/light_curves/25885'
slc_path = 'Data/Outputs/Light_curves/25885'
case_dir = os.listdir(os.path.join(cwd, case_path))
slc_dir = os.listdir(os.path.join(cwd, slc_path))

print('Started on path: ', slc_path)

#############################################################################################
var_data = []

# get synthetic lightcurve
for slc in slc_dir:

	# get slc data
	slc_data = np.genfromtxt(os.path.join(slc_path, slc), delimiter=',')

	track_id = np.int(slc.split('_')[0])

	# find matching real light curve
	for case in case_dir:

		track_id_case = np.int(case.split('_')[0])

		if track_id_case == track_id:

			# get true mag data
			case_data = np.genfromtxt(os.path.join(case_path, case), delimiter=',')
			break

	for index, slc_line in enumerate(slc_data):
		mag = slc_line[2]
		if np.isinf(mag):
			slc_data[index, 2] = 0


	case_avg_mag = np.mean(case_data[:,7])
	case_std = np.std(case_data[:,7])

	slc_avg_mag = np.mean(slc_data[:,2])
	slc_std = np.std(slc_data[:,2])

	jdate = slc_data[0, 0]
	var_data.append([jdate, case_avg_mag, case_std, slc_avg_mag, slc_std])

var_data = np.array(var_data).astype(np.float)
var_data = var_data[var_data[:,0].argsort()]

#############################################################################################

# moving average date

N = 3
cumsum, maves_date = [0], []
for i, x in enumerate(var_data[:,0], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_date.append(moving_ave)
maves_date = np.array(maves_date).astype(np.float)
maves_date = np.reshape(maves_date, (len(maves_date), 1))

#############################################################################################

# moving average of averages 1

N = 3
cumsum, maves_ave1 = [0], []
for i, x in enumerate(var_data[:,1], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_ave1.append(moving_ave)
maves_ave1 = np.array(maves_ave1).astype(np.float)
maves_ave1 = np.reshape(maves_ave1, (len(maves_ave1), 1))
ave1_trend85 = np.append(maves_date, maves_ave1, axis = 1)

#############################################################################################

# moving average of averages 2

N = 3
cumsum, maves_ave2 = [0], []
for i, x in enumerate(var_data[:,3], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_ave2.append(moving_ave)
maves_ave2 = np.array(maves_ave2).astype(np.float)
maves_ave2 = np.reshape(maves_ave2, (len(maves_ave2), 1))
ave2_trend85 = np.append(maves_date, maves_ave2, axis = 1)

#############################################################################################

# moving average of standard deviations 1

N = 3
cumsum, maves_std1 = [0], []
for i, x in enumerate(var_data[:,2], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std1.append(moving_ave)
maves_std1 = np.array(maves_std1).astype(np.float)
maves_std1 = np.reshape(maves_std1, (len(maves_std1), 1))
std1_trend85 = np.append(maves_date, maves_std1, axis = 1)

#############################################################################################

# moving average of standard deviations 2

N = 3
cumsum, maves_std2 = [0], []
for i, x in enumerate(var_data[:,4], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std2.append(moving_ave)
maves_std2 = np.array(maves_std2).astype(np.float)
maves_std2 = np.reshape(maves_std2, (len(maves_std2), 1))
std2_trend85 = np.append(maves_date, maves_std2, axis = 1)

#############################################################################################
#############################################################################################

# PLOT: FIGURE 5 - AVERAGE MAG 25885

pyplot.figure(5)
pyplot.scatter(var_data[:,0], var_data[:,1], s=20, c='r', label="Real")
pyplot.scatter(var_data[:,0], var_data[:,3], s=20, c='g', label="Synthetic")

pyplot.plot(ave1_trend85[:,0], ave1_trend85[:,1], 'r', label="Real")
pyplot.plot(ave2_trend85[:,0], ave2_trend85[:,1], 'g', label="Synthetic")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Mean Magnitude')
pyplot.title('Average Magnitudes for: 25885')
pyplot.grid(True)
#pyplot.savefig("test.png")
pyplot.axis([var_data[0,0], var_data[-1,0], 12, 0])
pyplot.legend()

#############################################################################################
#############################################################################################

# PLOT: FIGURE 6 - STD DEV 25885

pyplot.figure(6)
pyplot.scatter(var_data[:,0], var_data[:,2], s=20, c='r', label="Real")
pyplot.scatter(var_data[:,0], var_data[:,4], s=20, c='g', label="Synthetic")

pyplot.plot(std1_trend85[:,0], std1_trend85[:,1], 'r', label="Real")
pyplot.plot(std2_trend85[:,0], std2_trend85[:,1], 'g', label="Synthetic")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Standard Deviation')
pyplot.title('Standard Deviations for: 25885')
pyplot.grid(True)
#pyplot.savefig("test.png")
pyplot.axis([var_data[0,0], var_data[-1,0], 0, 4])
pyplot.legend()

#############################################################################################
#############################################################################################

# PLOT: FIGURE 7 - ALL TRENDS

pyplot.figure(7)
pyplot.plot(std1_trend63[:,0], std1_trend63[:,1], 'r', label="Of Interest")
pyplot.plot(std1_trend64[:,0], std1_trend64[:,1], 'b', label="Active")
pyplot.plot(std1_trend85[:,0], std1_trend85[:,1], 'g', label="Inactive")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Standard Deviation')
pyplot.title('Standard Deviations for all Three')
pyplot.grid(True)
#pyplot.savefig("test.png")
pyplot.axis([var_data[0,0], var_data[-1,0], 0, 2])

pyplot.legend()

#############################################################################################
#############################################################################################

# PLOT: FIGURE 8 - ALL TRENDS

pyplot.figure(8)
pyplot.plot(std1_trend64[:,0], std1_trend64[:,1], 'r', linestyle='-', label="Active, Real")
pyplot.plot(std1_trend85[:,0], std1_trend85[:,1], 'r', linestyle='--', label="Inactive, Real")

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Standard Deviation')
pyplot.title('Standard Deviation Comparison')
pyplot.grid(True)
#pyplot.savefig("test.png")
pyplot.axis([var_data[0,0], var_data[-1,0], 0, 2])

pyplot.legend()
pyplot.show()


print(std1_trend63)
