import os
import sys
import csv
import numpy as np
from matplotlib import pyplot

# free parameter: propagation timestep
prop_step = 0.1

# set directories
cwd = os.getcwd()

# 25885

case_path = 'Data/source/MMT/GLOBALSTAR/light_curves/25885'
case_dir = os.listdir(os.path.join(cwd, case_path))

slc_path = 'Data/Outputs/Light_curves/25885'
slc_dir = os.listdir(os.path.join(cwd, slc_path))

print('Started on path: ', slc_path)
var_data = []

# get synthetic lightcurve
for slc in slc_dir:

	# get slc data
	slc_data = np.genfromtxt(os.path.join(slc_path, slc), delimiter=',')

	track_id = np.int(slc.split('_')[0])

	# find matching real light curve
	for case in case_dir:

		track_id_case = np.int(case.split('_')[0])

		if track_id_case == track_id:

			# get true mag data
			case_data = np.genfromtxt(os.path.join(case_path, case), delimiter=',')
			break

	for index, slc_line in enumerate(slc_data):
		mag = slc_line[2]
		if np.isinf(mag):
			slc_data[index, 2] = 0


	case_avg_mag = np.mean(case_data[:,7])
	case_std = np.std(case_data[:,7])

	slc_avg_mag = np.mean(slc_data[:,2])
	slc_std = np.std(slc_data[:,2])

	jdate = slc_data[0, 0]
	var_data.append([jdate, case_avg_mag, case_std, slc_avg_mag, slc_std])

	pyplot.figure(1)
	
	pyplot.plot(case_data[:,0], case_data[:,7], 'r')
	pyplot.plot(slc_data[:,0], slc_data[:,2], 'g')
	
	pyplot.xlabel('time (julian date)')
	pyplot.ylabel('Magnitude')
	pyplot.title('Magnitudes for: 25885')
	pyplot.grid(True)
	pyplot.axis([slc_data[0,0], slc_data[-1,0], 12, 0])
	pyplot.show()	

var_data = np.array(var_data).astype(np.float)
var_data = var_data[var_data[:,0].argsort()]

#############################################################################################

# moving average date

N = 3
cumsum, maves_date = [0], []
for i, x in enumerate(var_data[:,0], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_date.append(moving_ave)
maves_date = np.array(maves_date).astype(np.float)
maves_date = np.reshape(maves_date, (len(maves_date), 1))

#############################################################################################

# moving average of averages 1

N = 3
cumsum, maves_ave1 = [0], []
for i, x in enumerate(var_data[:,1], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_ave1.append(moving_ave)
maves_ave1 = np.array(maves_ave1).astype(np.float)
maves_ave1 = np.reshape(maves_ave1, (len(maves_ave1), 1))
ave1_trend85 = np.append(maves_date, maves_ave1, axis = 1)

#############################################################################################

# moving average of averages 2

N = 3
cumsum, maves_ave2 = [0], []
for i, x in enumerate(var_data[:,3], 1):
	cumsum.append(cumsum[i-1] + x)
	if i>=N:
		print(cumsum[i])
		moving_ave = (cumsum[i] - cumsum[i-N])/N
		maves_ave2.append(moving_ave)
maves_ave2 = np.array(maves_ave2).astype(np.float)
maves_ave2 = np.reshape(maves_ave2, (len(maves_ave2), 1))
ave2_trend85 = np.append(maves_date, maves_ave2, axis = 1)

#############################################################################################

# moving average of standard deviations 1

N = 3
cumsum, maves_std1 = [0], []
for i, x in enumerate(var_data[:,2], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std1.append(moving_ave)
maves_std1 = np.array(maves_std1).astype(np.float)
maves_std1 = np.reshape(maves_std1, (len(maves_std1), 1))
std1_trend85 = np.append(maves_date, maves_std1, axis = 1)

#############################################################################################

# moving average of standard deviations 2

N = 3
cumsum, maves_std2 = [0], []
for i, x in enumerate(var_data[:,4], 1):
    cumsum.append(cumsum[i-1] + x)
    if i>=N:
        moving_ave = (cumsum[i] - cumsum[i-N])/N
        maves_std2.append(moving_ave)
maves_std2 = np.array(maves_std2).astype(np.float)
maves_std2 = np.reshape(maves_std2, (len(maves_std2), 1))
std2_trend85 = np.append(maves_date, maves_std2, axis = 1)


# PLOT: FIGURE 5 - AVERAGE MAG 25885

pyplot.figure(2)
pyplot.scatter(var_data[:,0], var_data[:,1], s=10, c='r')
pyplot.scatter(var_data[:,0], var_data[:,3], s=10, c='g')

pyplot.plot(ave1_trend85[:,0], ave1_trend85[:,1], 'r')
pyplot.plot(ave2_trend85[:,0], ave2_trend85[:,1], 'g')

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Mean Magnitude')
pyplot.title('Average Magnitudes for: 25885')
pyplot.grid(True)
pyplot.axis([var_data[0,0], var_data[-1,0], 0, 12])

#############################################################################################
#############################################################################################

# PLOT: FIGURE 6 - STD DEV 25885

pyplot.figure(3)
pyplot.scatter(var_data[:,0], var_data[:,2], s=10, c='r')
pyplot.scatter(var_data[:,0], var_data[:,4], s=10, c='g')

pyplot.plot(std1_trend85[:,0], std1_trend85[:,1], 'r')
pyplot.plot(std2_trend85[:,0], std2_trend85[:,1], 'g')

pyplot.xlabel('time (julian date)')
pyplot.ylabel('Standard Deviation')
pyplot.title('Standard Deviations for: 25885')
pyplot.grid(True)
pyplot.axis([var_data[0,0], var_data[-1,0], 0, 2])



pyplot.show(5)







