
import os
import sys
import csv


import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import optimize

from stl import mesh

from domain_splitting import cheb_app
from face_counter import face_count

#########################################################################################
# set output directories

case_desig = 'attitude_motion'
case_desig = 'attitude'

case_desig2 = '_d0.5'
case_desig2 = '_d1.0'

distance_path = '/home/ldjb/codebase/sensitivity_study_newdata/distances/' + case_desig + case_desig2

fig1name = 'fig_attitx' + case_desig2 + '.pdf'
fig2name = 'fig_attity' + case_desig2 + '.pdf'
fig3name = 'fig_attitz' + case_desig2 + '.pdf'
fig4name = 'fig_attitxyz' + case_desig2 + '.pdf'
fig20name = 'fig_attitxyz' + case_desig2 + '.pdf'

distance_dir = os.listdir(distance_path)

file_save_path = '/home/ldjb/codebase/sensitivity_study_newdata/new_figs/'

face_path = '/home/ldjb/codebase/git_dir/python/LCG/sensitivity_study'
face_file = 'nfaces_file.csv'

geom_path = '/home/ldjb/codebase/sensitivity_study_newdata/Geometry/new_geoms'

def fit_func(x, A, K, C):
    return A * np.exp(K * x) + C

def fit_exp_linear(x, y, C=0):
    y = y - C
    y = np.log(y)
    K, A_log = np.polyfit(x, y, 1)
    A = np.exp(A_log)
    return A, K

#########################################################################################
#	sort distance files

distance_files = []

for distance_file in distance_dir:
	attit_case = distance_file.split('_')[1]

	if 'mot' in attit_case:
		attit_num = attit_case.split('-')[1][-1]
	else:
		if '-' in attit_case:
			attit_num = attit_case.split('-')[0][-1]
		else:
			attit_num = attit_case[-1]

	distance_files.append([int(attit_num), distance_file])

distance_files = sorted(distance_files, key=lambda x: x[0])

#########################################################################################
#	read geometry files for face numbers
face_file = 'nfaces_file.csv'

if os.path.isfile(os.path.join(face_path, face_file)):
	print('Found nfaces file')
	nfaces = np.genfromtxt(os.path.join(face_path, face_file), delimiter=',')
else:
	print('Finding face numbers')
	#	set geometry directory
	geom_dir = os.listdir(geom_path)
	
	#	Read and sort geometry files
	geom_files = []
	for case in geom_dir:
		case_num, case_str, _ = case.split('_', maxsplit=2)
		geom_files.append([int(case_num), case_str, case])
	
	geom_files = sorted(geom_files, key=lambda x: x[0])
	
	#	read geometry files for face numbers
	nfaces = np.zeros([len(geom_files)])
	for i, geom_case in enumerate(geom_files):
	
		geom_file = geom_case[2]
	
		#	Load mesh
		my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
		#	count faces
		nfaces[i] = face_count(my_mesh.normals)
	
#	np.savetxt(os.path.join(cwd, face_path, face_file), nfaces, delimiter=',')

	
#########################################################################################

plt.rc('font', family='serif')
plt.rc('xtick', labelsize='x-small')
plt.rc('ytick', labelsize='x-small')

#	Plot X cases
fig1 = plt.figure(num=1, figsize=(8, 6), facecolor='w', edgecolor='k')
ax1 = fig1.add_subplot(111)

i = 0

color_settings = np.linspace(0.5, 1.0, 10)
color_index = 0

markers = [
	'o',
	'v',
	's',
	'p',
	'*',
	'+',
	'x',
	'D',
	'h',
	'8'
]


nfaces = nfaces[3:]
all_dist = np.zeros([11, len(nfaces)])

for distance_file in distance_files:

	distance_file = distance_file[1]
	_, case_name, _ = distance_file.split('_')


	if 'x' in case_name:

		print('\nWorking on: ', case_name)

		case_numid = case_name.split('-')[0][-1]

		if int(case_numid) % 2 == 0:
			continue

		case_label = '0.' + case_numid + ' ' + '$\pi$'
	
		case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
		distances = case_data

		distances = distances[3:]

		ax1.scatter(
			nfaces,
			distances,
#			color = [color_settings[color_index], 0, 0],
			label=case_label,
			marker = markers[color_index],
			s=20
		)

		all_dist[i, :] = distances
		i += 1

		#	perform fitting
		xdata = nfaces
		ydata = distances

		C0 = distances[-1]

		print('C0 set to: ', C0)

		A, K = fit_exp_linear(xdata, ydata, C0)

		xin = np.arange(xdata[0], xdata[-1], 1)
		y_fit = fit_func(xin, A, K, C0)

		print('y_fit = ', y_fit)

		ax1.plot(
			xin,
			y_fit,
		)
	
ax1.set_xlabel('Number of Facets')
ax1.set_ylabel('Hilbert Distance')
ax1.tick_params(direction='in')
# ax1.set_xlim(150, 800)
# ax1.set_ylim(0, 1e5)
ax1.legend(prop={'size': 6})
ax1.grid()

#fig1.savefig(os.path.join(file_save_path, fig1name), bbox_inches='tight')

#########################################################################################
#	Plot Y cases
fig2 = plt.figure(num=2, figsize=(8, 6), facecolor='w', edgecolor='k')
ax2 = fig2.add_subplot(111)

color_index = 0

for distance_file in distance_files:
	distance_file = distance_file[1]
	case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
	_, case_name, _ = distance_file.split('_')

	if 'y' in case_name:


		case_numid = case_name.split('-')[0][-1]

		if int(case_numid) % 2 == 0:
			continue

		case_label = '0.' + case_numid + ' ' + '$\pi$'
		distances = case_data
		distances = distances[3:]

		ax2.scatter(
			nfaces,
			distances,
#			color = [0, color_settings[color_index], 0],
			label=case_label,
			marker = markers[color_index],
			s=20
		)

		#	perform fitting
		xdata = nfaces
		ydata = distances

		C0 = distances[-1]

		A, K = fit_exp_linear(xdata, ydata, C0)

		xin = np.arange(xdata[0], xdata[-1], 1)
		y_fit = fit_func(xin, A, K, C0)

		ax2.plot(
			xin,
			y_fit,
		)
	
		color_index += 1


ax2.set_xlabel('Number of Facets')
ax2.set_ylabel('Hilbert Distance')
ax2.tick_params(direction='in')
# ax2.set_xlim(150, 800)
# ax2.set_ylim(0, 1e5)
ax2.legend(prop={'size': 6})
ax2.grid()

#fig2.savefig(os.path.join(file_save_path, fig2name), bbox_inches='tight')

#########################################################################################
#	Plot Z cases

fig3 = plt.figure(num=3, figsize=(8, 6), facecolor='w', edgecolor='k')
ax3 = fig3.add_subplot(111)
color_index = 0

for distance_file in distance_files:
	distance_file = distance_file[1]
	case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
	_, case_name, _ = distance_file.split('_')

	if 'z' in case_name:

		case_numid = case_name.split('-')[0][-1]

		if int(case_numid) % 2 == 0:
			continue

		case_label = '0.' + case_numid + ' ' + '$\pi$'

		distances = case_data
		distances = distances[3:]

		ax3.scatter(
			nfaces,
			distances,
#			color = [0, 0, color_settings[color_index]],
			label=case_label,
			marker = markers[color_index],
			s=20
		)

		#	perform fitting
		xdata = nfaces
		ydata = distances

		C0 = distances[-1]

		A, K = fit_exp_linear(xdata, ydata, C0)

		xin = np.arange(xdata[0], xdata[-1], 1)
		y_fit = fit_func(xin, A, K, C0)

		ax3.plot(
			xin,
			y_fit,
		)
	
		color_index += 1


ax3.set_xlabel('Number of Facets')
ax3.set_ylabel('Hilbert Distance')
ax3.tick_params(direction='in')
# ax3.set_xlim(150, 800)
# ax3.set_ylim(0, 1e5)
ax3.legend(prop={'size': 6})
ax3.grid()

#fig3.savefig(os.path.join(file_save_path, fig3name), bbox_inches='tight')

plt.show()
sys.exit()


#########################################################################################
#	Plot All Attitude cases
#fig = plt.figure(num=4, figsize=(9, 8), dpi=80, facecolor='w', edgecolor='k')
fig4 = plt.figure(num=4, figsize=(4, 3), facecolor='w', edgecolor='k')
ax4 = fig4.add_subplot(111)
# ax4.set_title('Attitude-cases')
# ax4.set_yscale('symlog')

color_index = 0

for distance_file in distance_files:
	distance_file = distance_file[1]
	case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')
	_, case_name, _ = distance_file.split('_')

	if 'x' not in case_name:
		if 'y' not in case_name:
			if 'z' not in case_name:
				distances = case_data
				ax4.scatter(
					nfaces,
					distances,
#					color = [color_settings[color_index], color_settings[color_index], color_settings[color_index]],
					label=case_name,
					marker = markers[color_index],
					s=20
				)

				#	perform fitting
				xdata = nfaces
				ydata = distances
				fit_para, pcov = optimize.curve_fit(fit_func, xdata, ydata, maxfev=20000)

				xin = np.arange(xdata[0], xdata[-1], 1)
				y_fit = fit_func(xin, *fit_para)
			
				ax4.plot(
					xin,
					y_fit,
#					color = [color_settings[color_index], color_settings[color_index], color_settings[color_index]],
#					linestyle = ':',
#					label = case_name
				)
			
				ax20.plot(
					xin,
					y_fit,
#					color = [color_settings[color_index], color_settings[color_index], color_settings[color_index]],
#					linestyle = ':',
					label = case_name
				)
	

				color_index += 1

#########################################################################################

ax4.set_xlabel('Number of Facets')
ax4.set_ylabel('Hilbert Distance')
ax4.tick_params(direction='in')
# ax4.set_xlim(150, 800)
# ax4.set_ylim(0, 1e5)
ax4.legend(prop={'size': 6})
ax4.grid()
#fig4.savefig(os.path.join(file_save_path, fig4name), bbox_inches='tight')

ax20.set_xlabel('nfaces')
ax20.set_ylabel('Hilbert Distance')
ax20.tick_params(direction='in')
ax1.legend(prop={'size': 6})
ax20.grid()
#fig20.savefig(os.path.join(file_save_path, fig20name), bbox_inches='tight')

plt.show()
