
import os
import sys
import csv

from pprint import pprint

import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import signal
from scipy import optimize

from stl import mesh

from domain_splitting import cheb_app
from face_counter import face_count

#########################################################################################
# set input directory

#	case designation string
case_desig = 'bdrf_omega'
case_desig = 'bdrf_sd'
case_desig = 'bdrf_m'

distance_path = 'sensitivity_study/results/' + case_desig + '_cases'
distance_path = '../../../sensitivity_study_archive/distances/bdrf_m_omega0.5'

file_save_path = '/home/ldjb/PhD/Reports/Conferences/AMOS_2018/technical_paper'

distance_dir = os.listdir(distance_path)

# fitting function
def fit_func(x, c, m):
    return x**m * c


#########################################################################################
#	sort distance files

distance_files = []
for distance_file in distance_dir:

	bdrf_case = distance_file.split('_')[1]
	attit_num = bdrf_case.split('-')[1][-3:]

	distance_files.append([float(attit_num), distance_file])

distance_files = sorted(distance_files, key=lambda x: x[0])

#########################################################################################
#	read geometry files for face numbers

face_path = 'sensitivity_study'
face_file = 'nfaces_file.csv'

if os.path.isfile(os.path.join(face_path, face_file)):
	nfaces = np.genfromtxt(os.path.join(face_path, face_file), delimiter=',')
else:
	#	set geometry directory
	geom_path = 'Data/Inputs/Geometry/Sensitivity_study'
	geom_dir = os.listdir(geom_path)
	
	#	Read and sort geometry files
	geom_files = []
	for case in geom_dir:
		case_num, case_str, _ = case.split('_', maxsplit=2)
		geom_files.append([int(case_num), case_str, case])
	
	geom_files = sorted(geom_files, key=lambda x: x[0])
	
	#	read geometry files for face numbers
	nfaces = np.zeros([len(geom_files)])
	for i, geom_case in enumerate(geom_files):
	
		geom_file = geom_case[2]
	
		#	Load mesh
		my_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
		#	count faces
		nfaces[i] = face_count(my_mesh.normals)
	
	#	number of geometry files
	ngeom = len(nfaces)
	
	np.savetxt(os.path.join(cwd, face_path, face_file), nfaces, delimiter=',')

#########################################################################################
all_dist = np.zeros([11, 11])

#	Plot cases
plt.rc('font', family='serif')
plt.rc('xtick', labelsize='x-small')
plt.rc('ytick', labelsize='x-small')


fig1 = plt.figure(num=9, figsize=(4, 3), facecolor='w', edgecolor='k')
ax1 = fig1.add_subplot(111)
# ax1.set_title('BDRF_' + case_desig + '-cases')
# ax1.set_yscale('symlog')

#fig = plt.figure(20)
fig10 = plt.figure(num=20, figsize=(8, 6), facecolor='w', edgecolor='k')
ax10 = fig10.add_subplot(111)
# ax10.set_title('All-cases')
# ax10.set_yscale('symlog')

i = 0

color_settings = np.linspace(0, 1, 11)
color_index = 0
markers = [
	'o',
	'v',
	's',
	'p',
	'*',
	'+',
	'x',
	'D',
	'h',
	'8',
	'>'
]

for distance_file in distance_files:

	distance_file = distance_file[1]
	_, case_name, _ = distance_file.split('_')

	print('Working on: ', case_name)

	case_data = np.genfromtxt(os.path.join(distance_path, distance_file), delimiter=',')

	distances = case_data

	ax1.scatter(
		nfaces,
		distances,
#		color = [color_settings[color_index], 0, 0],
		label=case_name,
		marker = markers[color_index],
		s=20
	)

	ax10.scatter(
		nfaces,
		distances,
#		color = [color_settings[color_index], 0, 0],
#		label=case_name
	)

	all_dist[i, :] = distances
	i += 1

	#	perform fitting
	xdata = nfaces
	ydata = distances

	fit_para, pcov = optimize.curve_fit(fit_func, xdata, ydata, maxfev=20000)

	xin = np.arange(xdata[0], xdata[-1], 1)
	y_fit = fit_func(xin, *fit_para)

	ax1.plot(
		xin,
		y_fit,
#		color = [color_settings[color_index], 0, 0],
#		linestyle = ':',
	)

	ax10.plot(
		xin,
		y_fit,
#		color = [color_settings[color_index], 0, 0],
#		linestyle = ':',
	)

	color_index += 1

ax1.set_xlabel('Number of Faces')
ax1.set_ylabel('Hilbert Distance')
#ax1.set_xlim(150, 800)
#ax1.set_ylim(0, 1e5)
ax1.grid()

handles, labels = ax1.get_legend_handles_labels()
#ax1.legend(handles[::-1], labels[::-1], title='', loc='upper left')
#ax1.legend(loc='center left',  bbox_to_anchor=(1, 0.5),  prop={'size': 6})
ax1.legend(prop={'size': 6})

fig1.savefig(os.path.join(file_save_path, 'fig_m1.pdf'), bbox_inches='tight')

plt.show()
