import os
import sys
import csv

from pprint import pprint

import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import signal

###################################################################################################
###################################################################################################
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
###################################################################################################
def dom_split(domain_in, signal_in):

	#	chebyshev sampling limit
	samples = 20

	# correct of odd domain size
	if len(domain_in) % 2 == 0:
		odd_correction = 0
	else:
		odd_correction = 1

	#	domain start value
	zero_point =  int(domain_in[0])

	#	signal start index
	index_point = np.int(np.ceil(len(signal_in) / 2))

	#	domain halfway point
	half_point = index_point + zero_point

	#	generate signal indexing arrays
	dom1_index = np.arange(0, index_point, 1)
	dom2_index = np.arange(index_point, len(domain_in), 1)

	#	generate domain
	dom1_space = np.arange(zero_point, half_point, 1)
	dom2_space = np.arange(half_point, half_point + len(dom1_space) - odd_correction, 1)

	#	slice signal
	dom1_signal = signal_in[dom1_index]
	dom2_signal = signal_in[dom2_index]

	#	generate Chebyshev approximation
	p1 = np.polynomial.Chebyshev.fit(dom1_space, dom1_signal, samples)
	p2 = np.polynomial.Chebyshev.fit(dom2_space, dom2_signal, samples)

	#	calculate Chebyshev signal
	dom1_cheb_signal = p1(dom1_space)
	dom2_cheb_signal = p2(dom2_space)

	#	calculate RMS values
	rms_1 = rms_calc(dom1_signal, dom1_cheb_signal)
	rms_2 = rms_calc(dom2_signal, dom2_cheb_signal)

	#	assemble outputs
	outputs = [
		dom1_space,
		dom1_signal,
		dom1_cheb_signal,
		rms_1,
		dom2_space,
		dom2_signal,
		dom2_cheb_signal,
		rms_2
	]

	return outputs

###################################################################################################
###################################################################################################

def cheb_app(base_signal):
	
	#	Recursive approximation
	samples = 20
	
	#	space
	npts = np.arange(0, len(base_signal), 1)

	#	approximate signal
	p = np.polynomial.Chebyshev.fit(npts, base_signal, samples)
	cheb_signal = p(npts)
	
	#	calculate rms
	rms_init = rms_calc(base_signal, cheb_signal)
#	print('Initial RMS = ', rms_init)
	
###############################################################################################
	#	split level 0

#	print('\nPerforming split level 0.')
	outputs = dom_split(npts, base_signal)
	dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

#	print('\nSplit level 0 complete.')

	cheb_domain = np.concatenate([
		dom1_space_out,
		dom2_space_out
	])

	cheb_signal = np.concatenate([
		dom1_cheb_signal_out,
		dom2_cheb_signal_out
	])

	#	calculate rms
	rms = rms_calc(base_signal, cheb_signal)
#	print('Total RMS = ', rms)

###############################################################################################
	#	split level 1

	#	initilise loop
	rms_vals = [rms_1_out, rms_2_out]
	dom_spaces = [dom1_space_out, dom2_space_out]
	dom_signals = [dom1_signal_out, dom2_signal_out]

	if any(i > 1e-6 for i in rms_vals):
#		print('\nPerforming split level 1. . .')

		split_num = 1
		first_flag = 1

		for dom_space, dom_signal in zip(dom_spaces, dom_signals):

			space_in = dom_space
			signal_in = dom_signal

			outputs = dom_split(space_in, signal_in)
			dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

			if first_flag is 1:
				rms_vals = [rms_1_out, rms_2_out]
				dom_spaces1 = [dom1_space_out, dom2_space_out]
				dom_signals1 = [dom1_signal_out, dom2_signal_out]
				dom_cheb_signals1 = [dom1_cheb_signal_out, dom2_cheb_signal_out]
				first_flag = 0
			else:
				rms_vals.append(rms_1_out)
				rms_vals.append(rms_2_out)
				dom_spaces1.append(dom1_space_out)
				dom_spaces1.append(dom2_space_out)
				dom_signals1.append(dom1_signal_out)
				dom_signals1.append(dom2_signal_out)
				dom_cheb_signals1.append(dom1_cheb_signal_out)
				dom_cheb_signals1.append(dom2_cheb_signal_out)
			split_num += 1

		# assemble Chebeshev domain
		cheb_domain = dom_spaces1[0]
		domspas = dom_spaces1[1:]
		for domspa in domspas:
			cheb_domain = np.concatenate([cheb_domain, domspa])

		# assemble Chebeshev signal
		cheb_signal = dom_cheb_signals1[0]
		domsigs = dom_cheb_signals1[1:]
		for domsig in domsigs:
			cheb_signal = np.concatenate([cheb_signal, domsig])
	
		#	calculate rms
		rms = rms_calc(base_signal, cheb_signal)
#		print('Total RMS = ', rms)

	else:
		return cheb_signal, cheb_domain, rms

###############################################################################################
	#	split level 2
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 2. . .')

		split_num = 1
		first_flag = 1

		for dom_space, dom_signal in zip(dom_spaces1, dom_signals1):

			space_in = dom_space
			signal_in = dom_signal

			outputs = dom_split(space_in, signal_in)
			dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

			if first_flag is 1:
				rms_vals = [rms_1_out, rms_2_out]
				dom_spaces2 = [dom1_space_out, dom2_space_out]
				dom_signals2 = [dom1_signal_out, dom2_signal_out]
				dom_cheb_signals2 = [dom1_cheb_signal_out, dom2_cheb_signal_out]
				first_flag = 0
			else:
				rms_vals.append(rms_1_out)
				rms_vals.append(rms_2_out)
				dom_spaces2.append(dom1_space_out)
				dom_spaces2.append(dom2_space_out)
				dom_signals2.append(dom1_signal_out)
				dom_signals2.append(dom2_signal_out)
				dom_cheb_signals2.append(dom1_cheb_signal_out)
				dom_cheb_signals2.append(dom2_cheb_signal_out)
			split_num += 1

		# assemble Chebeshev domain
		cheb_domain = dom_spaces2[0]
		domspas = dom_spaces2[1:]
		for domspa in domspas:
			cheb_domain = np.concatenate([cheb_domain, domspa])

		# assemble Chebeshev signal
		cheb_signal = dom_cheb_signals2[0]
		domsigs = dom_cheb_signals2[1:]
		for domsig in domsigs:
			cheb_signal = np.concatenate([cheb_signal, domsig])
	
		#	calculate rms
		rms = rms_calc(base_signal, cheb_signal)
#		print('Total RMS = ', rms)
	
	else:
		return cheb_signal, cheb_domain, rms

###############################################################################################
	#	split level 3
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 3. . .')

		split_num = 1
		first_flag = 1

		for dom_space, dom_signal in zip(dom_spaces2, dom_signals2):

			space_in = dom_space
			signal_in = dom_signal

			outputs = dom_split(space_in, signal_in)
			dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

			if first_flag is 1:
				rms_vals = [rms_1_out, rms_2_out]
				dom_spaces3 = [dom1_space_out, dom2_space_out]
				dom_signals3 = [dom1_signal_out, dom2_signal_out]
				dom_cheb_signals3 = [dom1_cheb_signal_out, dom2_cheb_signal_out]
				first_flag = 0
			else:
				rms_vals.append(rms_1_out)
				rms_vals.append(rms_2_out)
				dom_spaces3.append(dom1_space_out)
				dom_spaces3.append(dom2_space_out)
				dom_signals3.append(dom1_signal_out)
				dom_signals3.append(dom2_signal_out)
				dom_cheb_signals3.append(dom1_cheb_signal_out)
				dom_cheb_signals3.append(dom2_cheb_signal_out)
			split_num += 1

		# assemble Chebeshev domain
		cheb_domain = dom_spaces3[0]
		domspas = dom_spaces3[1:]
		for domspa in domspas:
			cheb_domain = np.concatenate([cheb_domain, domspa])

		# assemble Chebeshev signal
		cheb_signal = dom_cheb_signals3[0]
		domsigs = dom_cheb_signals3[1:]
		for domsig in domsigs:
			cheb_signal = np.concatenate([cheb_signal, domsig])
	
		#	calculate rms
		rms = rms_calc(base_signal, cheb_signal)
#		print('Total RMS = ', rms)
	
	else:
		return cheb_signal, cheb_domain, rms

###############################################################################################
	#	split level 4
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 4. . .')

		split_num = 1
		first_flag = 1

		for dom_space, dom_signal in zip(dom_spaces3, dom_signals3):

			space_in = dom_space
			signal_in = dom_signal

			outputs = dom_split(space_in, signal_in)
			dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

			if first_flag is 1:
				rms_vals = [rms_1_out, rms_2_out]
				dom_spaces4 = [dom1_space_out, dom2_space_out]
				dom_signals4 = [dom1_signal_out, dom2_signal_out]
				dom_cheb_signals4 = [dom1_cheb_signal_out, dom2_cheb_signal_out]
				first_flag = 0
			else:
				rms_vals.append(rms_1_out)
				rms_vals.append(rms_2_out)
				dom_spaces4.append(dom1_space_out)
				dom_spaces4.append(dom2_space_out)
				dom_signals4.append(dom1_signal_out)
				dom_signals4.append(dom2_signal_out)
				dom_cheb_signals4.append(dom1_cheb_signal_out)
				dom_cheb_signals4.append(dom2_cheb_signal_out)
			split_num += 1

		# assemble Chebeshev domain
		cheb_domain = dom_spaces4[0]
		domspas = dom_spaces4[1:]
		for domspa in domspas:
			cheb_domain = np.concatenate([cheb_domain, domspa])

		# assemble Chebeshev signal
		cheb_signal = dom_cheb_signals4[0]
		domsigs = dom_cheb_signals4[1:]
		for domsig in domsigs:
			cheb_signal = np.concatenate([cheb_signal, domsig])
	
		#	calculate rms
		rms = rms_calc(base_signal, cheb_signal)
#		print('Total RMS = ', rms)
	
	else:
		return cheb_signal, cheb_domain, rms

###############################################################################################
	#	split level 5
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 5. . .')

		split_num = 1
		first_flag = 1

		for dom_space, dom_signal in zip(dom_spaces4, dom_signals4):

			space_in = dom_space
			signal_in = dom_signal

			outputs = dom_split(space_in, signal_in)
			dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

			if first_flag is 1:
				rms_vals = [rms_1_out, rms_2_out]
				dom_spaces5 = [dom1_space_out, dom2_space_out]
				dom_signals5 = [dom1_signal_out, dom2_signal_out]
				dom_cheb_signals5 = [dom1_cheb_signal_out, dom2_cheb_signal_out]
				first_flag = 0
			else:
				rms_vals.append(rms_1_out)
				rms_vals.append(rms_2_out)
				dom_spaces5.append(dom1_space_out)
				dom_spaces5.append(dom2_space_out)
				dom_signals5.append(dom1_signal_out)
				dom_signals5.append(dom2_signal_out)
				dom_cheb_signals5.append(dom1_cheb_signal_out)
				dom_cheb_signals5.append(dom2_cheb_signal_out)
			split_num += 1

		# assemble Chebeshev domain
		cheb_domain = dom_spaces5[0]
		domspas = dom_spaces5[1:]
		for domspa in domspas:
			cheb_domain = np.concatenate([cheb_domain, domspa])

		# assemble Chebeshev signal
		cheb_signal = dom_cheb_signals5[0]
		domsigs = dom_cheb_signals5[1:]
		for domsig in domsigs:
			cheb_signal = np.concatenate([cheb_signal, domsig])
	
		#	calculate rms
		rms = rms_calc(base_signal, cheb_signal)
#		print('Total RMS = ', rms)
	
	else:
		return cheb_signal, cheb_domain, rms

###############################################################################################
	#	split level 6
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 6. . .')

		split_num = 1
		first_flag = 1

		for dom_space, dom_signal in zip(dom_spaces5, dom_signals5):

			space_in = dom_space
			signal_in = dom_signal

			outputs = dom_split(space_in, signal_in)
			dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

			if first_flag is 1:
				rms_vals = [rms_1_out, rms_2_out]
				dom_spaces6 = [dom1_space_out, dom2_space_out]
				dom_signals6 = [dom1_signal_out, dom2_signal_out]
				dom_cheb_signals6 = [dom1_cheb_signal_out, dom2_cheb_signal_out]
				first_flag = 0
			else:
				rms_vals.append(rms_1_out)
				rms_vals.append(rms_2_out)
				dom_spaces6.append(dom1_space_out)
				dom_spaces6.append(dom2_space_out)
				dom_signals6.append(dom1_signal_out)
				dom_signals6.append(dom2_signal_out)
				dom_cheb_signals6.append(dom1_cheb_signal_out)
				dom_cheb_signals6.append(dom2_cheb_signal_out)
			split_num += 1

		# assemble Chebeshev domain
		cheb_domain = dom_spaces6[0]
		domspas = dom_spaces6[1:]
		for domspa in domspas:
			cheb_domain = np.concatenate([cheb_domain, domspa])

		# assemble Chebeshev signal
		cheb_signal = dom_cheb_signals6[0]
		domsigs = dom_cheb_signals6[1:]
		for domsig in domsigs:
			cheb_signal = np.concatenate([cheb_signal, domsig])
	
		#	calculate rms
		rms = rms_calc(base_signal, cheb_signal)
#		print('Total RMS = ', rms)
	
	else:
		return cheb_signal, cheb_domain, rms

###############################################################################################
	#	split level 7
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 7. . .')

		split_num = 1
		first_flag = 1

		for dom_space, dom_signal in zip(dom_spaces6, dom_signals6):

			space_in = dom_space
			signal_in = dom_signal

			outputs = dom_split(space_in, signal_in)
			dom1_space_out, dom1_signal_out, dom1_cheb_signal_out, rms_1_out, dom2_space_out, dom2_signal_out, dom2_cheb_signal_out, rms_2_out = outputs

			if first_flag is 1:
				rms_vals = [rms_1_out, rms_2_out]
				dom_spaces7 = [dom1_space_out, dom2_space_out]
				dom_signals7 = [dom1_signal_out, dom2_signal_out]
				dom_cheb_signals7 = [dom1_cheb_signal_out, dom2_cheb_signal_out]
				first_flag = 0
			else:
				rms_vals.append(rms_1_out)
				rms_vals.append(rms_2_out)
				dom_spaces7.append(dom1_space_out)
				dom_spaces7.append(dom2_space_out)
				dom_signals7.append(dom1_signal_out)
				dom_signals7.append(dom2_signal_out)
				dom_cheb_signals7.append(dom1_cheb_signal_out)
				dom_cheb_signals7.append(dom2_cheb_signal_out)
			split_num += 1

		# assemble Chebeshev domain
		cheb_domain = dom_spaces7[0]
		domspas = dom_spaces7[1:]
		for domspa in domspas:
			cheb_domain = np.concatenate([cheb_domain, domspa])

		# assemble Chebeshev signal
		cheb_signal = dom_cheb_signals7[0]
		domsigs = dom_cheb_signals7[1:]
		for domsig in domsigs:
			cheb_signal = np.concatenate([cheb_signal, domsig])
	
		#	calculate rms
		rms = rms_calc(base_signal, cheb_signal)
#		print('level 7 Total RMS = ', rms)
	
	return cheb_signal, cheb_domain, rms
