
##############################################################################################

import os
import sys
import csv
import numpy as np
from subprocess import call

from stl import mesh
from pprint import pprint
from progressbar import ProgressBar, Percentage, Bar

import matplotlib.pyplot as plt
import matplotlib.patches as patch
from mpl_toolkits import mplot3d

from function_list import jd2date, date2jd, get_iers, get_times
from trans_func import site_vec, teme2itrf, it2rsw, rot1, rot2, rot3, rot_mat
from sofa_interface import gc2it

#############################################################################################
from setup import fred_setup, object_gen
from function_list import date2jd, jd2date
from transform import transform
#############################################################################################

#	numpy runtime error divide by zero exception
np.seterr(divide='ignore', invalid='ignore')

#	Inputs
case_desig = 'attitude'

#	free parameter: propagation timestep
prop_step = 0.1

#	set input directories
cwd = os.getcwd()
# case_path = 'Data/source/MMT/GLOBALSTAR/case_files/25885'
# case_path = '../../../sensitivity_study_archive/case_files/' + case_desig + '_spec0.1'

case_path = 'sensitivity_study/render/render_file'
case_dir = os.listdir(os.path.join(cwd, case_path))


lightcurve_path = '../../../sensitivity_study_archive/light_curves/' + case_desig + '_time'
 
#	image path
image_path = '../../../output_graphics/Images/3d'
graph_path = '../../../output_graphics/Images/2d'


#############################################################################################

#	clear terminal
call(['clear'])
print('Started.')

#	initilise variables
object_list = []
flist_horz = []
flist_tle = []
flist_lc = []
bdrf_list = []

print('Generating case objects...')
for case_file in case_dir:

	#	generatate case data from case file
	object_data, fname_horz, fname_tle, fname_lc, bdrf_data =  object_gen(
		case_file,
		case_path,
		lightcurve_path,
		prop_step
	)

	if object_data is 0:
		print('Excluded: Beyond the end of 2017')
	else:
		object_list.append(object_data)
		flist_horz.append(fname_horz)
		flist_tle.append(fname_tle)
		flist_lc.append(fname_lc)
		bdrf_list.append(bdrf_data)

print('{:d} case objects generated.'.format(len(object_list)))

#############################################################################################

#	perform setup

for i in range(len(object_list)):

	inputs = [object_list[i],
		flist_horz[i],
		flist_tle[i],
	]

	fred_setup(inputs)

#############################################################################################

input('\nInitilisation Complete. Press ENTER to begin simulations')

#	generate parellelised inputs
inputs = []

for i in range(len(object_list)):
	inputs.append([
		object_list[i],
		flist_tle[i],
		flist_horz[i],
		flist_lc[i]
	])


overwrite = 1
inputs = zip(inputs, bdrf_list)

for inps in inputs:

	inp, bdrf_data = inps
	#	Initilization
	print('\n==> Running Fred . . . \n')

	object_data, file_name_tle, file_name_horizons, file_name_lc = inp

	#	create required subdirectories
	norad_id = object_data.norad_ident
	cwd = os.getcwd()
	object_dir = '{:d}'.format(norad_id)

	#	light curves
	lightcurve_path = 'Data/Outputs/Light_curves/ss_light_curves/test2'
	lightcurve_file_path = os.path.join(cwd, lightcurve_path, object_dir)

	#	transformation
	transform(object_data, file_name_tle, file_name_horizons)

	#	light curve existance check

#	if os.path.isfile(os.path.join(lightcurve_file_path, file_name_lc)):
	if overwrite is 0:
		print('Lightcurve Exists.')
		light_curve = 0
	else:
		#	Light Curve Generation
		# unpack object data
		norad_id = object_data.norad_ident
		jday_start = object_data.jday_utc_start
		jfrac_start = object_data.jfrac_utc_start
		jday_stop = object_data.jday_utc_stop
		jfrac_stop = object_data.jfrac_utc_stop
		site_latgd = np.radians(object_data.site_latitudegd)
		site_lon = np.radians(object_data.site_longitude)
		site_alt = object_data.site_altitude
		attit = object_data.attitude
		attit_mot = object_data.attitude_motion
		geom_file = object_data.geometry_file

		# time
		# convert julian to utc date
		year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)
		date_vec1 = [year1, mon1, day1, hour1, min1, sec1] 

		# simulation parameters
		sim_start = jday_start + jfrac_start
		sim_seconds = int(86400 * ((jday_stop + jfrac_stop) - (jday_start + jfrac_start)))
		sim_time = np.linspace(0, sim_seconds, 10 * (sim_seconds) + 1)

		light_curve = np.zeros((len(sim_time), 3))
		light_curve[:,1] = sim_time

		# files
		# directories
		cwd = os.getcwd()
		transform_path = 'Data/Outputs/Transformation/{:d}'.format(norad_id)
		geom_path = 'Data/Inputs/Geometry/Sensitivity_study'
		slc_path = 'Data/Outputs/Light_Curves/{:d}'.format(norad_id)

		# create file name
		date_str = '{:4}-{:02}-{:02}_{:02}-{:02}-{:1.4f}'.format(*date_vec1)
		file_name_rsw =  '{:d}_rsw_{:s}.csv'.format(norad_id, date_str)
		data_pos = []

		# read positional data	
		with open(os.path.join(transform_path, file_name_rsw)) as csvfile:
			readCSV = csv.reader(csvfile, delimiter=',')
			for i, row in enumerate(readCSV):
				data_pos.append(row)

		# parse data
		data_pos = np.array(data_pos[:]).astype(np.float)
		sun_pos = data_pos[:, 0:3]
		earth_pos = data_pos[:, 3:6]
		site_pos = data_pos[:, 6:9]

		# import geometry
		static_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
		object_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))

		# rotate for xaxis orientation
		object_mesh.rotate([0,0,0.5], (np.pi / 2))
		# translate by CoM
		volume, cog, inertia = object_mesh.get_mass_properties()
		object_mesh.translate(-cog)

		#	convert units
		areas = object_mesh.areas / 1000**2
		vectors = object_mesh.vectors / 1000

		#	graphical output
		figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
		figure2d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')

		axes3d = mplot3d.Axes3D(figure3d)
		axes2d = figure2d.add_subplot(111)


		####################################################################################################
		# CodePoint: Initilization complete
		####################################################################################################

		# Prepare for propagation


		#	Apply initial attitude to vectors
		#	Converts to body-centred frame
		
		# Rotation Matrices		
		att_mat = rot_mat(attit)

		# Generate position data
		sun_vec_data = np.zeros((sun_pos.shape))
		site_vec_data = np.zeros((site_pos.shape))
		earth_vec_data = np.zeros((earth_pos.shape))

		for time_step, (sun_vec, site_vec, earth_vec) in enumerate(zip(sun_pos, site_pos, earth_pos)):

			time_since = light_curve[time_step, 1]

			# apply initial attitude rotations
			sun_vec_step = np.dot(sun_vec, att_mat)
			site_vec_step = np.dot(site_vec, att_mat)
			earth_vec_step = np.dot(earth_vec, att_mat)

			# time step rotation matrix
			total_attit_rot = np.multiply(attit_mot, time_since)

			# reform attitude motion rotation matrix
			rot_mat_step = rot_mat(total_attit_rot)

			# apply attitude motion rotations
			sun_vect = np.dot(sun_vec_step, rot_mat_step)
			sun_vec_data[time_step, :] = sun_vect
			site_vect = np.dot(site_vec_step, rot_mat_step)
			site_vec_data[time_step, :] = site_vect
			earth_vect = np.dot(earth_vec_step, rot_mat_step)
			earth_vec_data[time_step, :] = earth_vect

		####################################################################################################
		# CodePoint: Time Loop
		####################################################################################################

		print('Started Light Curve Generation...')
		print('	Duration = ', sim_seconds)
		print('	NPTS = ', len(sim_time)) 

		pbar = ProgressBar(widgets=[Percentage(), Bar()], maxval=len(sim_time)).start()

		for time_step, row in enumerate(light_curve):

			if time_step < 10:
				image_name = '000' + str(time_step)
			elif time_step < 100:
				image_name = '00' + str(time_step)
			elif time_step < 1000:
				image_name = '0' + str(time_step)
			else:
				image_name = str(time_step)

			graph_name = image_name + '_graph.png'
			image_name = image_name + '_image.png'

			# Clear graphics output
			if time_step != 0:
				axes3d.cla()

			# lightcurve julian date data
			time_since = light_curve[time_step, 1]
			jdate = sim_start + (time_since / 86400)
			light_curve[time_step, 0] = jdate

			sun_vec_step = sun_vec_data[time_step, :]
			site_vec_step = site_vec_data[time_step, :]
			earth_vec_step = earth_vec_data[time_step, :]

			sun_vec_unit = sun_vec_step / np.linalg.norm(sun_vec_step)
			site_vec_unit = site_vec_step / np.linalg.norm(site_vec_step)
			earth_vec_unit = earth_vec_step / np.linalg.norm(earth_vec_step)


			### FACET CALCULATIONS

			H_vec = (sun_vec_unit + site_vec_unit) / len(sun_vec_unit + site_vec_unit) 
			face_obs = []

			#	graphical output
			#	3D
			plot_vect = np.array([[0, 0, 0], sun_vec_unit]) * 10
			plot_vect2 = np.array([[0, 0, 0], site_vec_unit]) * 10
			plot_vect3 = np.array([[0, 0, 0], earth_vec_unit]) * 10

			sun_line = axes3d.plot(plot_vect[:, 0], plot_vect[:, 1], plot_vect[:, 2], color='r')
			obs_line = axes3d.plot(plot_vect2[:, 0], plot_vect2[:, 1], plot_vect2[:, 2], color='b')
			earth_line = axes3d.plot(plot_vect3[:, 0], plot_vect3[:, 1], plot_vect3[:, 2], color='g')

			elev = np.degrees(np.arctan2(plot_vect2[1, 2], np.sqrt(plot_vect2[1, 0]**2 + plot_vect2[1, 1]**2)))
			az = np.degrees(np.arctan2(plot_vect2[1, 1], plot_vect2[1, 0]))

#			axes3d.view_init(elev, az)
			axes3d.set_xlim([-0.5, 0.5])
			axes3d.set_ylim([-0.5, 0.5])
			axes3d.set_zlim([-0.5, 0.5])
			axes3d.set_aspect('equal')
#			axes3d.set_axis_off()

			for vector, area, bdrf_line in zip(vectors, areas, bdrf_data):

				#	parse bdrf data
				m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf = bdrf_line

				vect12 = vector[1] - vector[0]
				vect23 = vector[2] - vector[1]
				normal = np.cross(vect12, vect23)
				normal = normal / np.linalg.norm(normal)

				sun_ang = np.arccos(
					np.dot(normal, sun_vec_step)
					/ (np.linalg.norm(normal) *  np.linalg.norm(sun_vec_step))
				)

				site_ang = np.arccos(
					np.dot(normal, site_vec_step)
					/ (np.linalg.norm(normal) * np.linalg.norm(site_vec_step))
				)


				# RENDER
				alpha = np.absolute(np.cos(sun_ang))

				#	Add final vert
				vector = np.vstack([vector, vector[0, :]])

				xs = vector[:,0]
				ys = vector[:,1]
				zs = vector[:,2]

				verts = [list(zip(xs, ys, zs))]

				if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):

					alpha_ang = np.arccos(
						np.dot(H_vec, normal) / (np.linalg.norm(H_vec) * np.linalg.norm(normal))
					)

					temp1 = 2 * np.dot(normal, H_vec) * np.dot(normal, site_vec_unit) / np.dot(site_vec_unit, H_vec)
					
					temp2 = 2 * np.dot(normal, H_vec) * np.dot(normal, sun_vec_unit) / np.dot(sun_vec_unit, H_vec)
		
					if temp1 > 1 and temp2 > 1:
					    G_bdrf = 1
					elif temp1 < 1 and temp2 > 1:
						G_bdrf = temp1
					elif temp1 > 1 and temp2 < 1:
						G_bdrf = temp2
					elif temp1 < 1 and temp2 < 1:
						G_bdrf = np.min((temp1, temp2))

					c_bdrf = np.dot(site_vec_unit, H_vec)
					g_bdrf = n_bdrf**2 + c_bdrf**2 - 1
		
					F_bdrf = (
						((g_bdrf - c_bdrf)**2 / (2 * (g_bdrf + g_bdrf))) * 
						((1 + (c_bdrf * (g_bdrf + c_bdrf) - 1))**2 / ((c_bdrf * 
						(g_bdrf - c_bdrf) + 1)**2))
					)

					D_bdrf = (
						(1 / (m_bdrf**2 * np.cos(alpha_ang)**4)) * 
						np.exp(-((np.tan(alpha_ang) / m_bdrf)**2))
					)

					R_s = (
						(D_bdrf * G_bdrf * F_bdrf) / (np.pi * np.dot(normal, sun_vec_unit) * 
						np.dot(normal, site_vec_unit))
					)

					R_d = omega_bdrf / np.pi;
		
					facet_brightness = (
						(d_bdrf * R_d + s_bdrf * R_s) * area * np.pi * 
						np.dot(normal, sun_vec_unit) * 
						np.dot(normal, site_vec_unit)
					)

					r_obs = np.linalg.norm(site_vec_step)
					face_obs.append(facet_brightness / (4 * np.pi * r_obs**2))

					#	Graphics Output
					pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
					pc.set_alpha(alpha)
					pc.set_facecolor('k')

					axes3d.add_collection3d(pc)

				else:

					alpha = 0.01
					
					#	Graphics Output
					#	Face
					pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
					pc.set_alpha(alpha)
					pc.set_facecolor('k')
					axes3d.add_collection3d(pc)

					# Line

					pc = mplot3d.art3d.Line3DCollection(verts, colors='k', linewidth=0.2)
					axes3d.add_collection3d(pc)



			brightness_step = np.sum(face_obs)
			magnitude = -26.74 - 2.5 * np.log10(brightness_step)
			light_curve[time_step, 2] = magnitude


#			#	2D
#			axes2d.cla()
#			axes2d.set_xlim([light_curve[0, 1], light_curve[-1, 1]])
#			axes2d.set_ylim([18, 0])
#			axes2d.plot(
#				light_curve[0:time_step + 1, 1],
#				light_curve[0:time_step + 1, 2],
#				linewidth=1,
#				marker='.',
#				markersize=5,
#				markerfacecolor='k',
#				markeredgecolor='k'
#			)

#			figure2d.canvas.draw()

			#	work around for correct aspect ratio in 3d
			X = vectors[:, 0]
			Y = vectors[:, 1]
			Z = vectors[:, 2]

			# Create cubic bounding box to simulate equal aspect ratio
			max_range = np.array([X.max()-X.min(), Y.max()-Y.min(), Z.max()-Z.min()]).max()
			Xb = 0.5*max_range*np.mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(X.max()+X.min())
			Yb = 0.5*max_range*np.mgrid[-1:2:2,-1:2:2,-1:2:2][1].flatten() + 0.5*(Y.max()+Y.min())
			Zb = 0.5*max_range*np.mgrid[-1:2:2,-1:2:2,-1:2:2][2].flatten() + 0.5*(Z.max()+Z.min())
			# Comment or uncomment following both lines to test the fake bounding box:
			for xb, yb, zb in zip(Xb, Yb, Zb):
					axes3d.plot([xb], [yb], [zb], 'w')


			plt.show()
			sys.exit()


			#	save output figures
#			figure3d.savefig(os.path.join(cwd, image_path, image_name))
#			figure2d.savefig(os.path.join(cwd, graph_path, graph_name))

			pbar.update(time_step + 1)

		pbar.finish()
		print('Lightcurve Generated.')

	print('\n==> Script Complete.\n')
