
import sys
import numpy as np

def rot1(vector, rotation_angle):

	cosang = np.cos(rotation_angle)
	sinang = np.sin(rotation_angle)

	outvec = np.zeros((3))
	outvec[0] = vector[0]
	outvec[1] = np.dot(cosang, vector[1]) + np.dot(sinang, vector[2])
	outvec[2] = np.dot(cosang, vector[2]) - np.dot(sinang, vector[1])

	return outvec

def rot2(vector, rotation_angle):

	cosang = np.cos(rotation_angle)
	sinang = np.sin(rotation_angle)

	outvec = np.zeros((3))
	outvec[0] = np.dot(cosang, vector[0]) - np.dot(sinang, vector[2])
	outvec[1] = vector[1]
	outvec[2] = np.dot(cosang, vector[2]) + np.dot(sinang, vector[0])

	return outvec

def rot3(vector, rotation_angle):

	cosang = np.cos(rotation_angle)
	sinang = np.sin(rotation_angle)

	outvec = np.zeros((3))
	outvec[0] = np.dot(cosang, vector[0]) + np.dot(sinang, vector[1])
	outvec[1] = np.dot(cosang, vector[1]) - np.dot(sinang, vector[0])
	outvec[2] = vector[2]

	return outvec

def rot_mat(attit_mot):

	rot_mat = np.array([
		[
			np.cos(attit_mot[1]) * np.cos(attit_mot[2]),
			-np.cos(attit_mot[0]) * np.sin(attit_mot[2])
			+ np.sin(attit_mot[0]) * np.sin(attit_mot[1]) * np.cos(attit_mot[2]),
			np.sin(attit_mot[0]) * np.sin(attit_mot[2])
			+ np.cos(attit_mot[0]) * np.sin(attit_mot[1]) * np.cos(attit_mot[2])
		],
		[
			np.cos(attit_mot[1]) * np.sin(attit_mot[2]),
			np.cos(attit_mot[0]) * np.cos(attit_mot[2])
			+ np.sin(attit_mot[0]) * np.sin(attit_mot[1]) * np.sin(attit_mot[2]),
			-np.sin(attit_mot[0]) * np.cos(attit_mot[2])
			+ np.cos(attit_mot[0]) * np.sin(attit_mot[1]) * np.sin(attit_mot[2])
		],
		[
			-np.sin(attit_mot[1]),
			np.sin(attit_mot[0]) * np.cos(attit_mot[1]),
			np.cos(attit_mot[0]) * np.cos(attit_mot[1])
		]
	])

	return rot_mat

def site_vec(latgd, lon, alt):

	r_site = np.zeros((3))

	# const
	earth_r = 6378137
	flatness = 1 / 298.257223563
	earth_e = np.sqrt(2 * flatness - flatness**2)
	earth_e2 = earth_e**2

	sinlat = np.sin(latgd)

	cearth = earth_r / np.sqrt(1 - (earth_e2 * sinlat**2))
	rdel = (cearth + alt ) * np.cos(latgd)
	rk = ((1 - earth_e2) * cearth + alt ) * sinlat

	r_site[0] = rdel * np.cos(lon)
	r_site[1] = rdel * np.sin(lon)
	r_site[2] = rk

	return r_site

def gcrf2itrf(state, x_p, y_p, dX, dY, lod, ttt, jdut1):
	
	# precession nutation matrix
	pn = pn_rot(ttt, dX, dY)

	# sidreal motion rotation
	st = sidreal_rot(jdut1)

	# polar motion rotation
	pm = polar_rot(xp, yp, ttt)

	# gcrf to psudo equal frame
	rotation_matrix = np.dot(st, pn)

	if len(state) is 6:
		r_gcrf = state[0:3]
		v_gcrf = state[4:6]

		# calculation psudo equal frame
		r_pef = np.dot(rotation_matrix, r_gcrf)
		# calculate earth centered earth fixed frame
		r_ecef = np.dot(pm, r_pef)

		# velocity
		rot_vel = 7.292115146706979e-05 * (1.0  - lod / 86400.0 )
		omega_earth = np.array([[0], [0], [rot_vel]])
		v_pef = np.dot(rotation_matrix, v_)
		v_ecef = np.dot(pm, v_pef)

		return r_ecef, v_ecef

	elif len(state) is 3:
		r_gcrf = state[0:3]

		# calculation psudo equal frame
		r_pef = np.dot(rotation_matrix, r_gcrf)
		# calculate earth centered earth fixed frame
		r_ecef = np.dot(pm, r_pef)

		return r_ecef


def gtime(jdayut1, jfracut1):

	jdut1 = jdayut1 + jfracut1
	tut1 = (jdut1 - 2451545.0) / 36525

	gmst_total = np.radians(67310.54841
		+ (876600 * 3600 + 8640184.812866) * tut1
		+ 0.093104 * tut1 * tut1
		- 6.2E-6 * tut1 * tut1 * tut1
	)

	gmst = np.remainder(gmst_total * (360/86400), 2 * np.pi)

	return gmst


def teme2itrf(jdayut1, jfracut1, lod, x_p, y_p):

	# greenwich mean sidreal time
	gmst = gtime(jdayut1, jfracut1)

	# sidreal motion rotation
	st = np.zeros((3,3))
	st[0,0] =  np.cos(gmst)
	st[0,1] = -np.sin(gmst)
	st[0,2] = 0
	st[1,0] =  np.sin(gmst)
	st[1,1] =  np.cos(gmst)
	st[1,2] = 0
	st[2,0] = 0
	st[2,1] = 0
	st[2,2] = 1

	# polar motion rotation
	pm = np.zeros((3,3))

	cosxp = np.cos(x_p)
	sinxp = np.sin(x_p)
	cosyp = np.cos(y_p)
	sinyp = np.sin(y_p)

	pm[0,0] =  cosxp
	pm[0,1] =  0
	pm[0,2] = -sinxp
	pm[1,0] =  sinxp * sinyp
	pm[1,1] =  cosyp
	pm[1,2] =  cosxp * sinyp
	pm[2,0] =  sinxp * cosyp
	pm[2,1] = -sinyp
	pm[2,2] =  cosxp * cosyp
	
	teme2itrf = np.dot(np.transpose(pm), np.transpose(st))

	return teme2itrf


def it2rsw(central_body, transform_body):

	# x unit vector
	x_unit = np.array([1, 0, 0])

	# set object positions
	obj_r = transform_body
	sat_r = central_body
	earth_r = np.array([0, 0, 0])

	# translate
	rho_rsw = np.subtract(obj_r, sat_r)
	
	# angles
	earth_rho = np.subtract(earth_r, sat_r)
	earth_xy = np.array([earth_rho[0], earth_rho[1], 0])
	rot_z = np.arccos(np.dot(earth_xy, x_unit) / (np.linalg.norm(earth_xy) * np.linalg.norm(x_unit)))

	if earth_xy[1] < 0:
		rot_z = -rot_z

	
	rho_rotz = rot3(rho_rsw, rot_z)
	temp = rot3(earth_rho, rot_z)
	earth_zx = np.array([temp[0], 0, temp[2]])
	rot_y = np.arccos(np.dot(earth_zx, x_unit) / (np.linalg.norm(earth_zx) * np.linalg.norm(x_unit)))

	rho_roty = rot2(rho_rotz, rot_y)

	r_rsw = np.zeros((3))

	for i, elements in enumerate(rho_roty):
		if np.absolute(elements) < 0.001:
			 r_rsw[i] = 0
		else:
			r_rsw[i] = elements

	return r_rsw


