import sys
import os

import numpy as np

from scipy import stats

import matplotlib.pyplot as plt

from scipy.cluster.hierarchy import fcluster
import scipy.cluster.hierarchy as hac

####################################################################################################

def print_clusters(signal_data, Z, k, plot=False):

	# k Number of clusters I'd like to extract
	results = fcluster(Z, k, criterion='maxclust')
	
	index_array = np.arange(0, len(results), 1)
	
	s_array = np.zeros([60, 2])
	s_array[:, 0] = index_array
	s_array[:, 1] = results
	
	clusters = np.unique(s_array[:, 1])
	print('\nclusters = ', clusters)

	for c in clusters:

		print('\nFor cluster {}:'.format(c))

		#	signal ids of elements of this cluster
		cluster_ids = np.where(s_array[:, 1]==c)[0]

		print('signal ids of cluster: ', cluster_ids)
		print("\n	Cluster {:1.0f} number of entries {:1.0f}".format(c, len(cluster_ids)))

		#	plot these signals
		if plot:

			plt.figure(6)

			for signal_id in cluster_ids:
				#	extract signal
				signal = signal_data[signal_id, :]
				
				plt.plot(xvals, signal)

			plt.show()

####################################################################################################
#	build 6 time series groups for testing, called: a, b, c, d, e, f

num_samples = 61
group_size = 10

####################################################################################################
#	create the main time series for each group

xvals = np.linspace(0, 5, num_samples)
scale = 4

# generate time series data
a = scale * np.sin(xvals)
b = scale * (np.cos(1 + xvals * 3) + np.linspace(0, 1, num_samples))
c = scale * (np.sin(2 + xvals * 6) + np.linspace(0, -1, num_samples))
d = scale * (np.cos(3 + xvals * 9) + np.linspace(0, 4, num_samples))
e = scale * (np.sin(4 + xvals * 12) + np.linspace(0, -4, num_samples))
f = scale * np.cos(xvals)

####################################################################################################
#	from each main series build 'group_size' series

timeseries_set = [a,b,c,d,e,f]

fig = plt.figure(1)
ax1 = plt.axes()

for data_set in timeseries_set:
	ax1.plot(xvals, data_set)

signal_data = np.zeros([len(timeseries_set) * group_size, num_samples])

i_lower = 0
i_upper = 10

fig = plt.figure(2)
ax2 = plt.axes()

#	loop through time series groups
for data_set in timeseries_set:

	#	generate 10 different signals from base signal
	data_set = data_set + np.random.rand(group_size, num_samples) + np.random.randn(group_size, 1)

	signal_data[i_lower:i_upper, :] = data_set

	i_lower += 10
	i_upper += 10

	for lineplot in data_set:
		ax2.plot(xvals, lineplot)




# Do the clustering
Z = hac.linkage(signal_data, method='single', metric='correlation')

# Plot dendogram
plt.figure(3, figsize=(25, 10))
plt.title('Hierarchical Clustering Dendrogram')
plt.xlabel('sample index')
plt.ylabel('distance')
hac.dendrogram(
    Z,
    leaf_rotation=90.,  # rotates the x axis labels
    leaf_font_size=8.,  # font size for the x axis labels
)

plt.grid()









nclusters = 6

print_clusters(signal_data, Z, nclusters, plot=True)




plt.show()

sys.exit()



# Here we use spearman correlation
def my_metric(x, y):
    r = stats.pearsonr(x, y)[0]
    return 1 - r # correlation to distance: range 0 to 2

# Do the clustering    
Z = hac.linkage(timeSeries,  method='single', metric=my_metric)

# Plot dendogram
plt.figure(figsize=(25, 10))
plt.title('Hierarchical Clustering Dendrogram')
plt.xlabel('sample index')
plt.ylabel('distance')
hac.dendrogram(
    Z,
    leaf_rotation=90.,  # rotates the x axis labels
    leaf_font_size=8.,  # font size for the x axis labels
)




plt.show()
