
import numpy as np
import sys

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors

#############################################################################################
#	initilisation

#	define golden ratio
GR = (1.0 + np.sqrt(5.0)) / 2.0

#	define target radius
radius = 1.0

#	set division level
division_level = 3

#	calculate required edge length

#	vertices radius
edge_length = 0.5 * ((radius * 2.0) / (np.sqrt(GR * np.sqrt(5))))

print('\nRadius = ', radius)
print('\nRequired edge length = ', edge_length)

vert_coord = edge_length *  GR

#############################################################################################
#	define initial icosphere

vertices = np.array([
	[-edge_length,  vert_coord, 0],
	[ edge_length,  vert_coord, 0],
	[-edge_length, -vert_coord, 0],
	[ edge_length, -vert_coord, 0],

	[0, -edge_length,  vert_coord],
	[0,  edge_length,  vert_coord],
	[0, -edge_length, -vert_coord],
	[0,  edge_length, -vert_coord],

	[ vert_coord, 0, -edge_length],
	[ vert_coord, 0,  edge_length],
	[-vert_coord, 0, -edge_length],
	[-vert_coord, 0,  edge_length]
])

#	5 faces around point 0
faces = np.array([
	[0, 11, 5],
	[0, 5, 1],
	[0, 1, 7],
	[0, 7, 10],
	[0, 10, 11],
	[1, 5, 9],
	[5, 11, 4],
	[11, 10, 2],
	[10, 7, 6],
	[7, 1, 8],
	[3, 9, 4],
	[3, 4, 2],
	[3, 2, 6],
	[3, 6, 8],
	[3, 8, 9],
	[4, 9, 5],
	[2, 4, 11],
	[6, 2, 10],
	[8, 6, 7],
	[9, 8, 1]
])

print('\ninitial face array =\n', faces)
print('\ninitial vertex array =\n', vertices)

#############################################################################################
#	refinemnet

print('\nRefining with Division Level of: ', division_level)

if division_level == 1:

	new_faces = np.copy(faces)
	new_verts = np.copy(vertices)

elif division_level > 1:

	print('\nLooping through faces...')

	#	graphical output
	figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
	axes3d = mplot3d.Axes3D(figure3d)

	#	number of required triangles
	ntri = division_level**2

	for index, face in enumerate(faces):
	
		#	set current verts
		verts = vertices[face]
		vert1 = verts[0]
		vert2 = verts[1]
		vert3 = verts[2]

		for vert in verts:
			axes3d.scatter(*vert, marker='x', s=100)

		#############################################################################################
		#	triangle generation

		#	full triangle vectors
		vector1 = vert2 - vert1
		vector2 = vert3 - vert2
		vector3 = vert1 - vert3

		#	building block vectors
		prime_vec1 = np.divide(-vector3, division_level)
		prime_vec2 = np.divide(vector1, division_level)

		#	initilise vertices of step array
		verts_step = []

		#	generate face end points
		num_end_points = division_level + 1
		end_points = []

		n_temp = 0
		m_temp = division_level

		for end_point in range(1, num_end_points + 1):

			end_points.append([m_temp, n_temp])

			m_temp -= 1
			n_temp += 1

		end_point = np.array(end_points)

		#	initilise new faces array
		new_faces = np.zeros((ntri, 3))

		#	row end flag
		row_end = False
		
		current_end_point = 0
		itt1 = 0

		row_tri = (2 * division_level) - 1

		#	generate new vertices
		print('\nGenerating new vertices.\n')

		for n in range(0, division_level + 1):
			for m in range(0, division_level + 1):

				#	restrict to verts in tri
				if n + m < (division_level + 1):

					#	generate new vert
					temp_vert = vert1 + np.multiply(n, prime_vec1) + np.multiply(m, prime_vec2)

					#	append to verts of step
					verts_step.append(temp_vert)

					if m == 0 and n == 0:
						face_verts = verts_step[itt1]

					#	row end found
					if [m, n] == end_points[current_end_point]:

						#	end of start row, finding face indices
						if row_end is False:
							print('End of start row, vert = ', str(itt1))
							face_verts = np.vstack((face_verts, verts_step[itt1]))

							row_end = True

						#	end of second row, reccord indices
						elif row_end is True:
							print('End of End row, vert = ', str(itt1))

							row1_verts = face_verts

							#	reccord face indices
							face1 = []


							print(face_verts)
							sys.exit()

						current_end_point += 1

					if row_end is True:
						face_verts = np.vstack((face_verts, verts_step[itt1]))

					axes3d.scatter(*temp_vert)
					axes3d.text(*temp_vert, 'vert: ' + str(itt1))
					itt1 += 1

		verts_step = np.array(verts_step)

		print('Verts of step = \n', verts_step)

		plt.show()
		sys.exit()

		#############################################################################################
		#	scale verts by desired radius

		#	initilise scaled vertices array
		verts_step_scaled = np.zeros((verts_step.shape))

		for i, vert in enumerate(verts_step):

			#	vert magnitude
			vert_mag = np.linalg.norm(vert)

			#	vert unit vector
			vert_unit = np.divide(vert, vert_mag)
	
			#	scale by radius
			verts_step_scaled[i, :] = np.multiply(vert_unit, radius)


		#############################################################################################
		#	face generation

		if division_level == 2:
			#	define new faces
			new_face1 = [0, 1, 3]
			new_face2 = [1, 4, 3]
			new_face3 = [3, 4, 5]
			new_face4 = [1, 2, 4]

			faces_step = np.vstack((new_face1, new_face2, new_face3, new_face4))

		elif division_level == 3:

			#	define new faces
			new_face1 = [0, 1, 4]
			new_face2 = [1, 2, 5]
			new_face3 = [2, 3, 6]
			new_face4 = [4, 5, 7]
			new_face5 = [7, 8, 9]
			new_face6 = [1, 5, 4]
			new_face7 = [2, 6, 5]
			new_face8 = [5, 8, 7]
			new_face9 = [5, 6, 8]
	
			faces_step = np.array([
				new_face1,
				new_face2,
				new_face3,
				new_face4,
				new_face5,
				new_face6,
				new_face7,
				new_face8,
				new_face9
			])

		#############################################################################################
		#	graphical output



		for plot_face in faces_step:
		
			plot_verts = verts_step_scaled[plot_face]
		
			print(plot_verts)
			
			vert = np.vstack([plot_verts, plot_verts[0, :]])
			
			xs = vert[:, 0]
			ys = vert[:, 1]
			zs = vert[:, 2]
			
			verts = [list(zip(xs, ys, zs))]
			
			alpha = 0.5
			
			#	Graphics Output
			pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
			pc.set_alpha(alpha)
			pc.set_edgecolor([0.5, 0.5, 0.5])
			pc.set_facecolor('b')
			
			axes3d.add_collection3d(pc)
			
		axes3d.set_xlim([-2, 2])
		axes3d.set_ylim([-2, 2])
		axes3d.set_zlim([-2, 2])
		axes3d.set_aspect('equal')
		axes3d.set_xlabel('x axis')
		axes3d.set_ylabel('y axis')
		axes3d.set_zlabel('z axis')

		plt.show()
		sys.exit()

		#############################################################################################
		#	add data to outputs

		if index == 0:

			print('\nInitilising outputs...\n')

			#	for first face, initilise outputs
			new_verts = np.copy(verts_step_scaled)
			new_faces = np.copy(faces_step)
			end_index = len(new_verts) - 1

		else:

			print('\nchecking for vert of face existance in output...\n')

			for subface in faces_step:

				print('\n############################################################')
				print('\ntest face = ', subface)
				new_face_index = np.array([0, 0, 0])

				subverts = verts_step_scaled[subface]
				subvert1 = subverts[0]
				subvert2 = subverts[1]
				subvert3 = subverts[2]

				flag1 = False
				flag2 = False
				flag3 = False

				#	check for subvert presence in output verts
				for subindex, subvert in enumerate(new_verts):

					#	found subvert1
					if np.array_equal(subvert1, subvert) is True:
						flag1 = True
						index1 = subindex
						print('Subvert 1 found at row: ', index1)

					#	found subvert1
					if np.array_equal(subvert2, subvert) is True:
						flag2 = True
						index2 = subindex
						print('Subvert 2 found at row: ', index2)

					#	found subvert1
					if np.array_equal(subvert3, subvert) is True:
						flag3 = True
						index3 = subindex
						print('Subvert 3 found at row: ', index3)

				#	add additional vertices as required
				if flag1 is False:
					new_verts = np.vstack((new_verts, subvert1))
					end_index += 1
					index1 = np.copy(end_index)
					print('Vert 1 is new, added to array at index = ', index1)

				if flag2 is False:
					new_verts = np.vstack((new_verts, subvert2))
					end_index += 1
					index2 = np.copy(end_index)
					print('Vert 2 is new, added to array at index = ', index2)

				if flag3 is False:
					new_verts = np.vstack((new_verts, subvert3))
					end_index += 1
					index3 = end_index
					print('Vert 3 is new, added to array at index = ', index3)

				print('indexes = ', index1, index2, index3)

				#	new face indices
				new_face_index = np.array([index1, index2, index3])

				print('\nNew Face indices = ', new_face_index)

				#	add face to new_faces
				new_faces = np.vstack((new_faces, new_face_index))

				print('Added to new_faces array.')


		print('\nFace completed.')

print('New nfaces = ', len(new_faces))
print('New nverts = ', len(new_verts))

print(new_verts)

print('Finished.')

plt.show()
sys.exit()

#	graphical output
figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
axes3d = mplot3d.Axes3D(figure3d)
	
for plot_face in new_faces:

	plot_verts = new_verts[plot_face]

	vert = np.vstack([plot_verts, plot_verts[0, :]])
	
	xs = vert[:, 0]
	ys = vert[:, 1]
	zs = vert[:, 2]
	
	verts = [list(zip(xs, ys, zs))]
	
	alpha = 0.5
	
	#	Graphics Output
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_alpha(alpha)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	pc.set_facecolor('b')
	
	axes3d.add_collection3d(pc)
	
axes3d.set_xlim([-2, 2])
axes3d.set_ylim([-2, 2])
axes3d.set_zlim([-2, 2])
axes3d.set_aspect('equal')
axes3d.set_xlabel('x axis')
axes3d.set_ylabel('y axis')
axes3d.set_zlabel('z axis')
	
plt.show()
sys.exit()
