import os
import sys

import numpy as np
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

#############################################################################################
#	initialise
#############################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

geom_out_path = os.path.join(path_data, 'geometry_dir/geom_dir/CZ-2D')

#############################################################################################
#	range of x values
#############################################################################################

# #	main tank
# top_width = 2.4 / 2
# bottom_width = 2.4 / 2
# length = 3.9
# geom_file_name = 'sl8_main_tank.stl'

# #	bottom_tank
# bottom_width = 3.238 / 2
# top_width = 3.238 / 2
# length = 1.952
# geom_file_name = 'bottom_tank.stl'

# #	tank_connector
# top_width = 1.68 / 2
# bottom_width = 1.02 / 2
# length = 0.84
# geom_file_name = 'sl8_connector.stl'

# #	nozzel
# top_width = 0.6 / 2
# bottom_width = 1.2 / 2
# length = 0.84
# geom_file_name = 'sl8_nozzel.stl'

# #	nozzel
# top_width = 0.6 / 2
# bottom_width = 1.2 / 2
# length = 0.84
# geom_file_name = 'sl8_nozzel.stl'

#	CZ-2D cylinder
top_width = 3.35 / 2
bottom_width = 3.35 / 2
length = 7.0
geom_file_name = 'cz-2d.stl'

n_radial = 37

ang_sep = np.linspace(0, 2 * np.pi, n_radial)

pos_data_top = np.zeros([n_radial, 2])
pos_data_bottom = np.zeros([n_radial, 2])

#	bottom
i = 0
x_vec = np.array([bottom_width, 0.0])

for ang in ang_sep:
	#	rotation matrix
	R = np.array([
		[np.cos(ang), -np.sin(ang)],
		[np.sin(ang), np.cos(ang)]
	])

	vec = np.dot(x_vec, R)
	pos_data_bottom[i, :] = vec
	i += 1


#	top
i = 0
x_vec = np.array([top_width, 0.0])

for ang in ang_sep:
	#	rotation matrix
	R = np.array([
		[np.cos(ang), -np.sin(ang)],
		[np.sin(ang), np.cos(ang)]
	])

	vec = np.dot(x_vec, R)
	pos_data_top[i, :] = vec
	i += 1

zero_col = np.zeros([len(pos_data_top), 1])
top_col = np.full(zero_col.shape, length)

bottom_ring = np.hstack([pos_data_bottom, zero_col])
bottom_ring = bottom_ring[0:-1]

top_ring = np.hstack([pos_data_top, top_col])
top_ring = top_ring[0:-1]

bottom_center = np.array([0, 0, 0])
top_center = np.array([0, 0, length])

verts_array = []
faces_array = []

#############################################################################################
#	bottom faces
#############################################################################################

for i, pnt in enumerate(bottom_ring):

	#	generate vertices
	pos1 = pnt

	if i == len(bottom_ring) - 1:
		pos2 = bottom_ring[0, :]
	else:
		pos2 = bottom_ring[i + 1, :]

	pos3 = bottom_center

	#	add verts to array
	if i == 0:
		verts_array.append(pos1)
		verts_array.append(pos2)
		verts_array.append(pos3)

		faces_array.append([0, 1, 2])

	else:

		pos1_index = np.where((verts_array==pos1).all(axis=1))[0]
		pos2_index = np.where((verts_array==pos2).all(axis=1))[0]
		pos3_index = np.where((verts_array==pos3).all(axis=1))[0]

		if pos1_index.size == 0:
			verts_array.append(pos1)
			face_index = len(verts_array) - 1
		else:
			face_index = pos1_index

		if pos2_index.size == 0:
			verts_array.append(pos2)
			face_index = np.hstack([face_index, [len(verts_array) - 1]])
		else:
			face_index = np.hstack([face_index, pos2_index])

		if pos3_index.size == 0:
			verts_array.append(pos3)
			face_index = np.hstack([face_index, [len(verts_array) - 1]])
		else:
			face_index = np.hstack([face_index, pos3_index])

		#	add face indices to array
		faces_array.append(face_index)


#############################################################################################
#	side faces
#############################################################################################

for i, (bottom_pnt, top_pnt) in enumerate(zip(bottom_ring, top_ring)):

	###############################################
	# face 1

	pos1_face1 = bottom_pnt
	pos2_face1 = top_pnt

	if i == len(bottom_ring) - 1:
		pos3_face1 = bottom_ring[0, :]
	else:
		pos3_face1 = bottom_ring[i + 1, :]

	pos1_index = np.where((verts_array==pos1_face1).all(axis=1))[0]
	pos2_index = np.where((verts_array==pos2_face1).all(axis=1))[0]
	pos3_index = np.where((verts_array==pos3_face1).all(axis=1))[0]

	if pos1_index.size == 0:
		verts_array.append(pos1_face1)
		face_index = len(verts_array) - 1
	else:
		face_index = pos1_index

	if pos2_index.size == 0:
		verts_array.append(pos2_face1)
		face_index = np.hstack([face_index, [len(verts_array) - 1]])
	else:
		face_index = np.hstack([face_index, pos2_index])

	if pos3_index.size == 0:
		verts_array.append(pos3_face1)
		face_index = np.hstack([face_index, [len(verts_array) - 1]])
	else:
		face_index = np.hstack([face_index, pos3_index])

	#	add face indices to array
	faces_array.append(face_index)

	###############################################
	# face 2
	if i == len(bottom_ring) - 1:
		pos1_face2 = top_ring[0, :]
		pos2_face2 = bottom_ring[0, :]
	else:
		pos1_face2 = top_ring[i+1, :]
		pos2_face2 = bottom_ring[i+1, :]

	pos3_face2 = top_pnt

	pos1_index = np.where((verts_array==pos1_face2).all(axis=1))[0]
	pos2_index = np.where((verts_array==pos2_face2).all(axis=1))[0]
	pos3_index = np.where((verts_array==pos3_face2).all(axis=1))[0]

	if pos1_index.size == 0:
		verts_array.append(pos1_face2)
		face_index = len(verts_array) - 1
	else:
		face_index = pos1_index

	if pos2_index.size == 0:
		verts_array.append(pos2_face2)
		face_index = np.hstack([face_index, [len(verts_array) - 1]])
	else:
		face_index = np.hstack([face_index, pos2_index])

	if pos3_index.size == 0:
		verts_array.append(pos3_face2)
		face_index = np.hstack([face_index, [len(verts_array) - 1]])
	else:
		face_index = np.hstack([face_index, pos3_index])

	#	add face indices to array
	faces_array.append(face_index)

#############################################################################################
#	top faces
#############################################################################################

for i, pnt in enumerate(top_ring):

	#	generate vertices
	pos1 = pnt
	pos2 = top_center

	if i == len(top_ring) - 1:
		pos3 = top_ring[0, :]
	else:
		pos3 = top_ring[i + 1, :]

	pos1_index = np.where((verts_array==pos1).all(axis=1))[0]
	pos2_index = np.where((verts_array==pos2).all(axis=1))[0]
	pos3_index = np.where((verts_array==pos3).all(axis=1))[0]

	if pos1_index.size == 0:
		verts_array.append(pos1)
		face_index = len(verts_array) - 1
	else:
		face_index = pos1_index

	if pos2_index.size == 0:
		verts_array.append(pos2)
		face_index = np.hstack([face_index, [len(verts_array) - 1]])
	else:
		face_index = np.hstack([face_index, pos2_index])

	if pos3_index.size == 0:
		verts_array.append(pos3)
		face_index = np.hstack([face_index, [len(verts_array) - 1]])
	else:
		face_index = np.hstack([face_index, pos3_index])

	#	add face indices to array
	faces_array.append(face_index)


verts_array = np.array(verts_array)
faces_array = np.array(faces_array)

print(type(verts_array))
print(verts_array.shape)

print(type(faces_array))
print(faces_array.shape)
sys.exit()

print('nverts = ', verts_array.shape[0])
print('nfaces = ', faces_array.shape[0])

#	initialise output mesh
object_mesh = mesh.Mesh(np.zeros(faces_array.shape[0], dtype=mesh.Mesh.dtype))

for i, f in enumerate(faces_array):
    for j in range(3):
        object_mesh.vectors[i][j] = verts_array[f[j],:]

#	translate by com
vol, com, inertia = object_mesh.get_mass_properties()

object_mesh.z -= com[2]

#############################################################################################
#	plot mesh
#############################################################################################

vectors = object_mesh.vectors
normals = object_mesh.normals


###############################################
#	initilise figure
figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
ax3d = mplot3d.Axes3D(figure3d)

for vector, normal in zip(vectors, normals):

	#	parse input vectors and calculate facet normal
	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	###############################################
	# rending

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]


	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)

ax3d.set_xlim([-5, 5])
ax3d.set_ylim([-5, 5])
ax3d.set_zlim([-5, 5])

plt.show()

###############################################################################################

user_input = input('\nSave file as {}? [y/n] '.format(geom_file_name))

if user_input == 'y':
	#	save stl mesh file
	object_mesh.save(os.path.join(geom_out_path, geom_file_name))
	print('File saved to: ', geom_out_path)

elif user_input == 'no':
	print('END')
