###################################################################################################
#	Geoemtry Generation Functions
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################
###################################################################################################

#	standard packages
import os
import sys

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

###################################################################################################
#	Function: 		top_faces_gen()
#	Description: 	returns the top faces of an object geometry
#
###################################################################################################

def top_faces_gen(top_ring, top_center, verts_array, faces_array):

	for i, pnt in enumerate(top_ring):
	
		#	generate vertices
		if i == len(top_ring) - 1:
			pos1 = top_ring[0, :]
		else:
			pos1 = top_ring[i + 1, :]
	
		pos2 = pnt
		pos3 = top_center
	
		#	add verts to array
		if i == 0 and verts_array == []:
			verts_array.append(pos1)
			verts_array.append(pos2)
			verts_array.append(pos3)
	
			faces_array.append([0, 1, 2])
	
		else:
			#	check for vert in verts array
			pos1_index = np.where((verts_array==pos1).all(axis=1))[0]
			pos2_index = np.where((verts_array==pos2).all(axis=1))[0]
			pos3_index = np.where((verts_array==pos3).all(axis=1))[0]
	
			if pos1_index.size == 0:
				verts_array.append(pos1)
				face_index = len(verts_array) - 1
			else:
				face_index = pos1_index
	
			if pos2_index.size == 0:
				verts_array.append(pos2)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos2_index])
	
			if pos3_index.size == 0:
				verts_array.append(pos3)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos3_index])
	
			#	add face indices to array
			faces_array.append(face_index)

	return verts_array, faces_array

###################################################################################################
#	Function: 		side_faces_gen()
#	Description: 	returns the side faces of an object geometry
#
###################################################################################################

def side_faces_gen(top_ring, bottom_ring, verts_array, faces_array):

	for i, (bottom_pnt, top_pnt) in enumerate(zip(bottom_ring, top_ring)):
	
		###############################################
		# face 1
	
		pos1_face1 = bottom_pnt
		pos2_face1 = top_pnt
	
		if i == len(bottom_ring) - 1:
			pos3_face1 = bottom_ring[0, :]
		else:
			pos3_face1 = bottom_ring[i + 1, :]
	
		if i == 0 and verts_array == []:
			verts_array.append(pos1_face1)
			verts_array.append(pos2_face1)
			verts_array.append(pos3_face1)
	
			faces_array.append([0, 1, 2])
	
		else:
	
			pos1_index = np.where((verts_array==pos1_face1).all(axis=1))[0]
			pos2_index = np.where((verts_array==pos2_face1).all(axis=1))[0]
			pos3_index = np.where((verts_array==pos3_face1).all(axis=1))[0]
		
			if pos1_index.size == 0:
				verts_array.append(pos1_face1)
				face_index = len(verts_array) - 1
			else:
				face_index = pos1_index
		
			if pos2_index.size == 0:
				verts_array.append(pos2_face1)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos2_index])
		
			if pos3_index.size == 0:
				verts_array.append(pos3_face1)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos3_index])
		
			#	add face indices to array
			faces_array.append(face_index)
	
		###############################################
		# face 2
		if i == len(bottom_ring) - 1:
			pos1_face2 = top_ring[0, :]
			pos2_face2 = bottom_ring[0, :]
		else:
			pos1_face2 = top_ring[i+1, :]
			pos2_face2 = bottom_ring[i+1, :]
	
		pos3_face2 = top_pnt
	
		pos1_index = np.where((verts_array==pos1_face2).all(axis=1))[0]
		pos2_index = np.where((verts_array==pos2_face2).all(axis=1))[0]
		pos3_index = np.where((verts_array==pos3_face2).all(axis=1))[0]
	
		if pos1_index.size == 0:
			verts_array.append(pos1_face2)
			face_index = len(verts_array) - 1
		else:
			face_index = pos1_index
	
		if pos2_index.size == 0:
			verts_array.append(pos2_face2)
			face_index = np.hstack([face_index, [len(verts_array) - 1]])
		else:
			face_index = np.hstack([face_index, pos2_index])
	
		if pos3_index.size == 0:
			verts_array.append(pos3_face2)
			face_index = np.hstack([face_index, [len(verts_array) - 1]])
		else:
			face_index = np.hstack([face_index, pos3_index])
	
		#	add face indices to array
		faces_array.append(face_index)
	
	return verts_array, faces_array

###################################################################################################
#	Function: 		bottom_faces_gen()
#	Description: 	returns the bottom faces of an object geometry
#
###################################################################################################

def bottom_faces_gen(bottom_ring, bottom_center, verts_array, faces_array):

	for i, pnt in enumerate(bottom_ring):
	
		#	generate vertices
		pos1 = pnt
	
		if i == len(bottom_ring) - 1:
			pos2 = bottom_ring[0, :]
		else:
			pos2 = bottom_ring[i + 1, :]
	
		pos3 = bottom_center
	
		#	add verts to array
		if i == 0 and verts_array == []:
			verts_array.append(pos1)
			verts_array.append(pos2)
			verts_array.append(pos3)
	
			faces_array.append([0, 1, 2])
	
		else:
	
			pos1_index = np.where((verts_array==pos1).all(axis=1))[0]
			pos2_index = np.where((verts_array==pos2).all(axis=1))[0]
			pos3_index = np.where((verts_array==pos3).all(axis=1))[0]
	
			if pos1_index.size == 0:
				verts_array.append(pos1)
				face_index = len(verts_array) - 1
			else:
				face_index = pos1_index
	
			if pos2_index.size == 0:
				verts_array.append(pos2)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos2_index])
	
			if pos3_index.size == 0:
				verts_array.append(pos3)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos3_index])
	
			#	add face indices to array
			faces_array.append(face_index)

	return verts_array, faces_array

###################################################################################################
#	Function: 		connecting_faces_gen()
#	Description: 	returns the faces connecting two parts of an object geometry
#
###################################################################################################

def connecting_faces_gen(top_ring, bottom_ring, verts_array, faces_array):

	for i, (outer_pnt, inner_pnt) in enumerate(zip(bottom_ring, top_ring)):
	
		###############################################
		# face 1
	
		pos1_face1 = inner_pnt
		pos2_face1 = outer_pnt
	
		if i == len(bottom_ring) - 1:
			pos3_face1 = bottom_ring[0, :]
		else:
			pos3_face1 = bottom_ring[i + 1, :]
	
	
		if i == 0 and verts_array == []:
			verts_array.append(pos1_face1)
			verts_array.append(pos2_face1)
			verts_array.append(pos3_face1)
	
			faces_array.append([0, 1, 2])
	
		else:
	
			pos1_index = np.where((verts_array==pos1_face1).all(axis=1))[0]
			pos2_index = np.where((verts_array==pos2_face1).all(axis=1))[0]
			pos3_index = np.where((verts_array==pos3_face1).all(axis=1))[0]
		
			if pos1_index.size == 0:
				verts_array.append(pos1_face1)
				face_index = len(verts_array) - 1
			else:
				face_index = pos1_index
		
			if pos2_index.size == 0:
				verts_array.append(pos2_face1)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos2_index])
		
			if pos3_index.size == 0:
				verts_array.append(pos3_face1)
				face_index = np.hstack([face_index, [len(verts_array) - 1]])
			else:
				face_index = np.hstack([face_index, pos3_index])
		
			#	add face indices to array
			faces_array.append(face_index)
	
		###############################################
		# face 2
	
		if i == len(bottom_ring) - 1:
			pos1_face2 = bottom_ring[0, :]
			pos2_face2 = top_ring[0, :]
		else:
			pos1_face2 = bottom_ring[i+1, :]
			pos2_face2 = top_ring[i+1, :]
	
		pos3_face2 = inner_pnt
	
		pos1_index = np.where((verts_array==pos1_face2).all(axis=1))[0]
		pos2_index = np.where((verts_array==pos2_face2).all(axis=1))[0]
		pos3_index = np.where((verts_array==pos3_face2).all(axis=1))[0]
	
		if pos1_index.size == 0:
			verts_array.append(pos1_face2)
			face_index = len(verts_array) - 1
		else:
			face_index = pos1_index
	
		if pos2_index.size == 0:
			verts_array.append(pos2_face2)
			face_index = np.hstack([face_index, [len(verts_array) - 1]])
		else:
			face_index = np.hstack([face_index, pos2_index])
	
		if pos3_index.size == 0:
			verts_array.append(pos3_face2)
			face_index = np.hstack([face_index, [len(verts_array) - 1]])
		else:
			face_index = np.hstack([face_index, pos3_index])
	
		#	add face indices to array
		faces_array.append(face_index)

	return verts_array, faces_array
