
import numpy as np
import sys
import os

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors

from stl import mesh

#############################################################################################
#	inputs

root_path = '/home/ldjb/codebase'
geom_out_path = os.path.join(root_path, 'data_archive/paper_data/geometry_dir')

geom_file_name = '01_cube_cube.stl'

#	define target radius
radius = 1.0

#############################################################################################

vertices = np.array([
		[-1.0, -1.0, -1.0],		#	0
		[-1.0,  1.0, -1.0],		#	1
		[ 1.0, -1.0, -1.0],		#	2
		[ 1.0,  1.0, -1.0],		#	3
		[-1.0, -1.0,  1.0],		#	4
		[-1.0,  1.0,  1.0],		#	5
		[ 1.0, -1.0,  1.0],		#	6
		[ 1.0,  1.0,  1.0]		#	7
	]		
)

faces = np.array([
		[0, 1, 2],		#	bottom1
		[2, 1, 3],		#	bottom2
		[6, 4, 0],		#	front1
		[0, 2, 6],		#	front2
		[7, 6, 2],		#	right1
		[7, 2, 3],		#	right2
		[0, 4, 5],		#	left1
		[0, 5, 1],		#	left2
		[1, 5, 7],		#	back1
		[1, 7, 3],		#	back2
		[6, 5, 4],		#	top1
		[6, 7, 5]		#	top2
	]
)

#	save faces and verts
object_mesh = mesh.Mesh(np.zeros(faces.shape[0], dtype=mesh.Mesh.dtype))

for i, f in enumerate(faces):
    for j in range(3):
        object_mesh.vectors[i][j] = vertices[f[j],:]

object_mesh.save(os.path.join(geom_out_path, geom_file_name))

#	graphical output
figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
axes3d = mplot3d.Axes3D(figure3d)

vectors = object_mesh.vectors
normals = object_mesh.normals

#############################################################################################
#	graphical output

alpha = 0.5

for vector, normal in zip(vectors, normals):

	vec_in = vector

	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	# RENDER

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	facecol = [0.1, 0.1, 0.1]
	alpha = 0.5

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_alpha(alpha)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	pc.set_facecolor(facecol)
	
	axes3d.add_collection3d(pc)

	#	normal
	vert1 = np.array(vec_in[0])
	vert2 = np.array(vec_in[1])
	vert3 = np.array(vec_in[2])

	centroid_x = (vert1[0] + vert2[0] + vert3[0]) / 3
	centroid_y = (vert1[1] + vert2[1] + vert3[1]) / 3
	centroid_z = (vert1[2] + vert2[2] + vert3[2]) / 3

	centroid_pos = [centroid_x, centroid_y, centroid_z]

	norm_trans = np.add(normal, centroid_pos)

	normal_plot = np.vstack((centroid_pos, norm_trans))

	axes3d.plot(
		normal_plot[:, 0],
		normal_plot[:, 1],
		normal_plot[:, 2],
		color='black'
	)

	axes3d.set_xlim([-2, 2])
	axes3d.set_ylim([-2, 2])
	axes3d.set_zlim([-2, 2])
	axes3d.set_aspect('equal')
	axes3d.set_xlabel('x axis')
	axes3d.set_ylabel('y axis')
	axes3d.set_zlabel('z axis')

plt.show()
sys.exit()
