import os
import sys

import numpy as np
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

# #	import from transformation functions
# from tools.functions.functions_transform import site_vec, rot1, rot2, rot3, \
# 										rot_mat, \
#  										rot_mat_gen, \
# 										axis2mat, \
# 										mat2angles, \
# 										gen_vectors

#############################################################################################
#	initialise
#############################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output
geom_out_path = os.path.join(path_data, 'geometry_dir/geom_dir/starlink')
geom_file_name = 'starlink_full.stl'


#############################################################################################
#	assemble mesh
#############################################################################################

#############################################################################################
#	payload

x_dim = 3.0
y_dim = 1.5
z_dim = 0.20

x_offset = 0.0
y_offset = 0.0
z_offset = 0.0

#	generate verts
pl_verts_array = np.array([
	[x_dim/2 - x_offset, y_dim/2 - y_offset, z_dim/2 - z_offset],
	[x_dim/2 - x_offset, -y_dim/2 - y_offset, z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, -y_dim/2 - y_offset, z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, y_dim/2 - y_offset, z_dim/2 - z_offset],
	[x_dim/2 - x_offset, y_dim/2 - y_offset, -z_dim/2 - z_offset],
	[x_dim/2 - x_offset, -y_dim/2 - y_offset, -z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, -y_dim/2 - y_offset, -z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, y_dim/2 - y_offset, -z_dim/2 - z_offset]
])

#	generate faces
pl_faces_array = np.array([
	[0, 2, 1],
	[2, 0, 3],
	[0, 4, 3],
	[4, 7, 3],
	[2, 3, 6],
	[3, 7, 6],
	[4, 5, 7],
	[5, 6, 7],
	[1, 2, 6],
	[5, 1, 6],
	[1, 5, 0],
	[0, 5, 4]
])

#############################################################################################
#	solar panel
x_dim = 3.0
y_dim = 0.1
z_dim = 12.0

x_offset = 0.0
y_offset = 0.8
z_offset = -6.1

#	generate verts
sp_verts_array = np.array([
	[x_dim/2 - x_offset, y_dim/2 - y_offset, z_dim/2 - z_offset],
	[x_dim/2 - x_offset, -y_dim/2 - y_offset, z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, -y_dim/2 - y_offset, z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, y_dim/2 - y_offset, z_dim/2 - z_offset],
	[x_dim/2 - x_offset, y_dim/2 - y_offset, -z_dim/2 - z_offset],
	[x_dim/2 - x_offset, -y_dim/2 - y_offset, -z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, -y_dim/2 - y_offset, -z_dim/2 - z_offset],
	[-x_dim/2 - x_offset, y_dim/2 - y_offset, -z_dim/2 - z_offset]
])

#	generate faces
sp_faces_array = np.array([
	[0, 2, 1],
	[2, 0, 3],
	[0, 4, 3],
	[4, 7, 3],
	[2, 3, 6],
	[3, 7, 6],
	[4, 5, 7],
	[5, 6, 7],
	[1, 2, 6],
	[5, 1, 6],
	[1, 5, 0],
	[0, 5, 4]
])





print(pl_verts_array)
print('\n', sp_verts_array)



#	initialise output mesh
sp_object_mesh = mesh.Mesh(np.zeros(sp_faces_array.shape[0], dtype=mesh.Mesh.dtype))

for i, f in enumerate(sp_faces_array):
    for j in range(3):
        sp_object_mesh.vectors[i][j] = sp_verts_array[f[j],:]

#	initialise output mesh
pl_object_mesh = mesh.Mesh(np.zeros(pl_faces_array.shape[0], dtype=mesh.Mesh.dtype))

for i, f in enumerate(pl_faces_array):
    for j in range(3):
        pl_object_mesh.vectors[i][j] = pl_verts_array[f[j],:]
#############################################################################################
#	plot mesh
#############################################################################################

###############################################
#	initilise figure
figure3d = plt.figure(num=None, figsize=(9, 9), dpi=80, facecolor='w', edgecolor='k')
ax3d = mplot3d.Axes3D(figure3d)

vectors = sp_object_mesh.vectors
normals = sp_object_mesh.normals

minbound = -5
maxbound = 5
ax3d.auto_scale_xyz([minbound, maxbound], [minbound, maxbound], [minbound, maxbound])

for vector, normal in zip(vectors, normals):

	#	parse input vectors and calculate facet normal
	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	origin_x = (vector[0, 0] + vector[1, 0] + vector[2, 0]) / 3
	origin_y = (vector[0, 1] + vector[1, 1] + vector[2, 1]) / 3
	origin_z = (vector[0, 2] + vector[1, 2] + vector[2, 2]) / 3

	normal_origin = np.array([origin_x, origin_y, origin_z])

	normal_plot = np.vstack([normal_origin, normal_unit + normal_origin])

	ax3d.plot(normal_plot[:, 0], normal_plot[:, 1], normal_plot[:, 2])

	###############################################
	#	rendering

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)

vectors = pl_object_mesh.vectors
normals = pl_object_mesh.normals

minbound = -5
maxbound = 5
ax3d.auto_scale_xyz([minbound, maxbound], [minbound, maxbound], [minbound, maxbound])

for vector, normal in zip(vectors, normals):

	#	parse input vectors and calculate facet normal
	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	origin_x = (vector[0, 0] + vector[1, 0] + vector[2, 0]) / 3
	origin_y = (vector[0, 1] + vector[1, 1] + vector[2, 1]) / 3
	origin_z = (vector[0, 2] + vector[1, 2] + vector[2, 2]) / 3

	normal_origin = np.array([origin_x, origin_y, origin_z])

	normal_plot = np.vstack([normal_origin, normal_unit + normal_origin])

	ax3d.plot(normal_plot[:, 0], normal_plot[:, 1], normal_plot[:, 2])

	###############################################
	#	rendering

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)





ax3d.set_xlim([-5, 5])
ax3d.set_ylim([-5, 5])
ax3d.set_zlim([-5, 5])

ax3d.set_xlabel('X-axis')
ax3d.set_ylabel('Y-axis')
ax3d.set_zlabel('Z-axis')

plt.show()
sys.exit()

###############################################################################################
#	save file

user_input = input('\nSave file to:	{}\n	As:	{}?\n[y/n] '.format(geom_out_path, geom_file_name))

if user_input == 'y':
	#	save stl mesh file
	object_mesh.save(os.path.join(geom_out_path, geom_file_name))
	print('File saved to: ', geom_out_path)

elif user_input == 'no':
	print('END')
