###################################################################################################

import numpy as np

from mpi4py import MPI

print('\nScript started')

###################################################################################################

#	define integral function
def integral(a_i, h, n):

	integ = 0.0

	for j in range(n):
		a_ij = a_i + (j + 0.5) * h
		integ += np.cos(a_ij) * h

	return integ

###################################################################################################

#	initilise MPI communicator
comm = MPI.COMM_WORLD

#	get current rank
rank = comm.Get_rank()
print('Rank = ', rank)

#	get processor number
size = comm.Get_size()
print('Size = ', size)


#	define 'a' value as 0.0
a = 0.0

#	define 'b' value as pi / 2
b = np.pi / 2.0

my_int = np.zeros(1)

integral_sum = np.zeros(1)

# Initilise value of n only if this is rank 0
if rank == 0:
	n = np.full(1, 500, dtype=int)
else:
	n = np.zeros(1, dtype=int)

#	Broadcast n to all processes
print('Process ', rank, ' before n = ', n[0])
comm.Bcast(n, root=0)
print('Process ', rank, ' after n = ', n[0])


#	compute parition
#	calc h after reveicing n
h = (b - a) / (n * size)

a_i = a + rank * h * n
my_int[0] = integral(a_i, h, n[0])


#	send parition back to root process
print('Process ', rank, ' has the partial integral = ', my_int[0])

#	set output destination as the root process
dest = 0

comm.Reduce(my_int, integral_sum, MPI.SUM, dest)

#	only print result inside root process
if rank == 0:
	print('The Integral Sum = ', integral_sum[0])
